/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef QCAMCTRLVIEW_H
#define QCAMCTRLVIEW_H

#include <vector>

// include files for QT
#include <qwidget.h>

// application specific includes
#include <qapplication.h>
#include <qlayout.h>
#include <qslider.h>
#include <qstring.h>
#include <qlabel.h>
#include <qstring.h>
#include <qgroupbox.h>
#include <qtoolbutton.h>
#include <iostream>
#include <fstream>
#include "mycamknob.h"
#include "nmm/interfaces/video/camera/ICamera.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"
#include "nmm/base/format/Format.hpp"
#include "nmm/base/ProxyApplication.hpp"
#include "nmm/utils/thread/Thread.hpp"
/**
 * This class provides an incomplete base for your application view. 
 */

class QGraphList;
class QLineEdit;
class QIntValidator;

class QCamCtrlView : public QWidget
{
  Q_OBJECT
public:
  QCamCtrlView(QWidget *parent, NMM::NMMApplication&);
  ~QCamCtrlView();

  void setCamera(NMM::ICamera* camera);
 // NMM::ClientRegistry& getRegistry() { return  __client_registry; }

private:
  class InternalStruct;

  void initPaint();
  void initTechnical();
  void initSlider();
  void initConnect();
  void initRegistry();
  void update();
  void scanCameras(const string&, int );
  void scanDisplay();
  void buttonConnect();
  list<NMM::INode*> requestAllNodes(const list<NMM::Response>& );
private:
  QLabel *zoom_label, *focus_label;
  QBoxLayout *toplevel;
  vector<MyCamKnob*> paint_button, technical_button;
  NMM::ICamera *camera;
  QSlider *zoom, *focus;
  QGridLayout *technical_layout;
  QGroupBox *technical_box;
  QGroupBox* paint_box;
  QPushButton *one_push;
  QPushButton *start_;
  QPushButton *stop_;
  QGraphList  *__graph_list;
  QLineEdit   *__host_name;
  QLineEdit   *__port_line;
  QIntValidator* __int_validator;
  QPushButton * __add_button;  

  NMM::Format __camera_format;
  NMM::INode* __display;
  NMM::INode* __current_sender;


  list<NMM::INode*>       __reserved_node_list;
  NMM::NMMApplication&    __application;

  bool                    __is_display_active;

  NMM::ThreadMutex        __change_camera_mutex;
  NMM::ICamera_var             __current_camera;
private slots:
  void changeCamera(NMM::INode* new_camera);
  void addRegistry();

public slots:
//technical
  void changeAutoExposure(int);
  void changeGamma (int);
  void changeShutter (int);
  void changeGain(int);
  void changeBrightness(int);
  void changeSharpness(int);
  void changeIris(int);

//painting
  void changeHue(int);
  void changeSaturation (int);
  void changeRedBalance(int);
  void changeBlueBalance(int);
  void onePush();

//slider
  void changeFocus(int);
  void changeZoom(int);

  void start();
  void stop();

};

#endif
