/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Andrs Otn Urbano,
 *                    email: andres.oton@gmail.com
 *                     
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <stdio.h>

#include "AdvancedReadNode.hpp"
#include "nmm/utils/utils.hpp"
#include "nmm/utils/SecurityCheck.hpp"

#include "nmm/base/registry/Plugin.hpp"

#include "nmm/utils/DirReader.hpp"

NMM_REGISTER_PLUGIN(NMM::AdvancedReadNode, "AdvancedReadNode")

namespace NMM {
    AdvancedReadNode::AdvancedReadNode(const char *name /* = "AdvancedReadNode" */,
				   StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
				   int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericReadNode(name, mode, queue_size) {

    filled = false;
    audiof = "";
    videof = "";
  }
  
  AdvancedReadNode::~AdvancedReadNode() {
  }

  list<string> AdvancedReadNode::getDirectoryList(const string& dir) {
  	
	list<string> dirs;

	DirReader cdir;
	cdir.setRootDir("/");

	if(!cdir.changeDir(dir))
		return dirs;

	cdir.setNiceDir(1);

	vector<DirReader::directory_t> subdirs = cdir.getDirsOnlyEntries();

	for(int i = 0; i < subdirs.size() ; i++)
	{
		DirReader::directory_t t_dir = subdirs[ i ];
		DEBUG_STREAM("AdvancedReadNode::getDirList file: " << t_dir.name << endl);
		dirs.push_back(t_dir.name);
	}
	
	return dirs;
  }

  list<string> AdvancedReadNode::getFileList(const string& dir) {

        list<string> files;

        DirReader cdir;
        cdir.setRootDir("/");

        if(!cdir.changeDir(dir))
                return files;

        cdir.setNiceDir(1);

        vector<DirReader::directory_t> sfiles = cdir.getRegFiles();

        for(int i = 0; i < sfiles.size() ; i++)
        {
                DirReader::directory_t t_dir = sfiles[ i ];
		DEBUG_STREAM("AdvancedReadNode::getFileList file: " << t_dir.name << endl);
                files.push_back(t_dir.name);
        }

        return files;

  }

  bool AdvancedReadNode::fillMetaInformation(const string& name) {

    file = name;
   
    if (stat(file.c_str(), &statbuf)) {
        ERROR_STREAM("AdvancedReadNode: fillMetaInformation, could not lstat " << file << endl ); 
        return false;
    }

    
    // printf("On device: major %d/minor %d    Inode number: %ld\n",
    //       major(statbuf.st_dev), minor(statbuf.st_dev), statbuf.st_ino);

    // printf("Size     : %-10ld         Type: %07o       "
    //       "Permissions: %05o\n", statbuf.st_size, 
    //       statbuf.st_mode & S_IFMT, statbuf.st_mode & ~(S_IFMT));
    // printf("Owner    : %d                Group: %d"
    //       "          Number of links: %d\n", 
    //       statbuf.st_uid, statbuf.st_gid, statbuf.st_nlink);


    DEBUG_STREAM(" The file size is: " << statbuf.st_size << endl);

    filled = true;

    return filled;
  }

  string AdvancedReadNode::getExtractInformation(string keyword) {

    string output,value="";
    char readbuf[300];
    FILE *pipein_fp;

    string command = "extract -p "+keyword+"  '"+file+"'";

    if (( pipein_fp = popen(command.c_str(), "r")) == NULL)
    {
      return "";
    }

    while(fgets(readbuf,300,pipein_fp))
    {
       output.append(readbuf);
    }
    
    if( output.length() > 2 )
      value.assign(output,keyword.length()+3,output.length()-keyword.length()-4);

    return value;
  }

  string AdvancedReadNode::timeString(time_t t) {

    char buffer[50];
    struct tm * local;

    local = localtime(&t);

    // The size of buffer is 50 the same that second parameter of strftime.
    strftime(buffer, 50, "%c", local);

    string buf(buffer);
    return buf;
  }

  string AdvancedReadNode::getAccessTime() {

    if(filled)  
      return timeString(statbuf.st_atime);
    else
      return "";

  }

  long AdvancedReadNode::getNumAccessTime() {
    if(filled)
      return statbuf.st_atime;
    else return 0;
  }

  string AdvancedReadNode::getModifyTime() {

    if(filled)
      return timeString(statbuf.st_mtime);
    else
      return "";

  }

  long AdvancedReadNode::getNumModifyTime() {
    return statbuf.st_mtime;
  }

  string AdvancedReadNode::getCreationTime() {

    if(filled)
      return timeString(statbuf.st_ctime);
    else
      return "";

  }

  long AdvancedReadNode::getNumCreationTime() {
    if(filled)
      return statbuf.st_ctime;
    else
      return 0;
  }
 
  string AdvancedReadNode::getAlbum() {
    return getExtractInformation("album");
  }

  string AdvancedReadNode::getTitle() {
    return getExtractInformation("title");
  }

  string AdvancedReadNode::getArtist() {
    return getExtractInformation("artist");
  }

  string AdvancedReadNode::getGenre() {
    return getExtractInformation("genre");
  }

  string AdvancedReadNode::getDate() {
    return getExtractInformation("date");
  }

  bool AdvancedReadNode::fillAVInfo()
  {
    char readbuf[300];
    FILE *pipein_fp;
    videof = "";
    audiof = "";
    string output;

    // El parametro --raw en avinfo se obtine una salida mucho mas procesable
    // por si interesa obtener mejor los parametros. 
    string command = "avinfo '"+file+"'";

    if (( pipein_fp = popen(command.c_str(), "r")) == NULL)
    {
      return false;
    }

    while(fgets(readbuf,300,pipein_fp))
    {
       output.append(readbuf);
       if( output.length() > 2 )
       {
         if( output.substr(0,5) == "video" )
           videof.assign(output,7,output.length()-8);
         if( output.substr(0,5) == "audio" )
           audiof.assign(output,7,output.length()-8);
       }
       output = "";
    }

  }

  string AdvancedReadNode::getAudioFormat() {
    if(fillAVInfo())
      return audiof;
    else return "";
  }

  string AdvancedReadNode::getVideoFormat() {
    if(fillAVInfo())
      return videof;
    else return "";
  }

  string AdvancedReadNode::getMimeType() {

    //file not return the mime type of .mpeg files
    string extension;
    string::size_type extension_pos = file.rfind(".");
    if(extension_pos != string::npos && extension_pos != file.length() - 1) {
      extension = file.substr(extension_pos+1);
      for (unsigned int i = 0; i < extension.length(); i++) {
        extension[i] = tolower(extension[i]);
      }
    }
    if(extension == "mpeg" || extension == "mpg" || extension == "MPEG" || extension == "MPG" )
	return "video/mpeg";

    if(extension == "mp3" || extension == "MP3")
        return "audio/x-mp3";

    //Copy from GraphURL.cpp
    //file works better than extract to get the mime type.

    std::FILE* read_fp;
    char buffer[256];
    int chars_read;
    string command;
    command = "/usr/bin/file --mime '" + file +"'";

    string mime_type = "";

    memset(buffer, '\0', sizeof(buffer));
    read_fp = popen(command.c_str(), "r");
    if(read_fp != NULL) {
      chars_read = fread(buffer, sizeof(char), 255, read_fp);
      if(chars_read > 0) {

        string tmp(buffer);
        string::size_type pos = tmp.rfind(file, 0);

        mime_type = tmp.substr(pos+file.size()+2, tmp.size()-pos-file.size()-3);
      }
      pclose(read_fp);
    }

    return mime_type;
  
  }

  int AdvancedReadNode::getMode() {
    if(filled)
      return statbuf.st_mode;
    else return 0;
  }

  bool AdvancedReadNode::getIsDir() {
    return S_ISDIR(statbuf.st_mode);
  }

  bool AdvancedReadNode::hasFileSize() {
    return filled;
  }

  void AdvancedReadNode::getFileSize(u_int64_t& bytes) {
    bytes = statbuf.st_size;
  }  

}
