/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include <cctype>
#include <iostream>
#include <string>
#include <cstdio>
#include <iomanip.h>
#include <stdlib.h>
#include <getopt.h>

// general includes
#include "nmm/base/connect.hpp"
#include "nmm/base/ProxyObject.hpp"


// include used interfaces
#include "nmm/interfaces/device/audio/IAudioDevice.hpp"
#include "nmm/interfaces/device/cdda/ICDDA.hpp"
#include "nmm/interfaces/device/cdda/ICDDB.hpp"
#include "nmm/interfaces/file/ITrack.hpp"

// include the registry
#include "nmm/base/registry/ClientRegistry.hpp"
#include "nmm/base/ProxyApplication.hpp"

// import namespace
using namespace NMM;


class CDDAExample {
private:
  bool is_playing;
  
  // all nodes (resources are owned here)
  vector<INode*> nodes;
  
  // specific nodes
  INode* audio_play;
  INode* cd;
  
  // other interfaces
  ICDDA_var cdda;
  ICDDB_var cddb;
  IAudioDevice_var audio_device;
  
  // the NMM application
  NMMApplication* app;
  ClientRegistry* registry;
  
  // node requests
  vector<NodeDescription> request;
  
  // options
  string device;
  bool use_cddb;
  bool verbose;
  
public:
  CDDAExample(int argc, char* argv[]) {
    is_playing = false;
    
    use_cddb = false;
    verbose = false;
    device = "/dev/cdrom";
    
    app = NULL;
    registry = NULL;
    
    parseCommandLine(argc, argv);
    
    app = ProxyApplication::getApplication(argc, argv);
    
    registry = &app->getRegistry();
  }
  
  ~CDDAExample() {
    cleanup();
  }
  
  virtual Result receiveEvent(const Event& e ) { 
    cerr << "receive Event with key " << e.getKey() << endl; 
    return SUCCESS; 
  }
  
protected:
  Result eventToc(string& toc) { 
    cerr << "Got Event >>    toc    << with Value :\n" 
         << toc << endl;
    cerr << endl 
         << "Press:" << endl 
         << "\t'+' for next Track" << endl
         << "\t'-' for previous Track" << endl
         << "\t'f' to seek 5 seconds forward" << endl
         << "\t'F' to seek 30 seconds forward" << endl
         << "\t'r' to seek 5 seconds backward" << endl
         << "\t'R' to seek 30 seconds backward" << endl
         << "\t'p' to pause playing" << endl
         << "\t'c' to continue playing after stop" << endl
         << "\t'q' for quit." << endl
         << "\t [0..9] for tracknummer (0 for 10)" << endl 
	 << "(don't forget to press [RETURN] as this little example doesn't use any high order input tools!)" << endl
         << endl;
    return SUCCESS; 
  }
  
  Result finished() { 
    cleanup();
    exit(0);
    return SUCCESS; 
  }
  
  Result startTrack(const string& name) { 
    cerr << "Got Event >>   start   << with Value \"" 
         << name << "\"" << endl; 
    return SUCCESS; 
  }
  
  Result endTrack() { 
    cerr << "Got Event >>    end    <<" << endl;
    return SUCCESS; 
  }
  
  Result eventTruncated(string& name) { 
    cerr << "Got Event >> truncated << with Value \"" 
         << name << "\"" << endl; 
    return SUCCESS; 
  }
  
  Result eventSeeked(string& name) { 
    cerr << "Got Event >>  seeked   << with Value \"" 
         << name << "\"" << endl; 
    return SUCCESS; 
  }
 
private: 
  void seek(int) {
//     cd->stop(); // !!!!!!!!! Reihenfolge ist wichtig, erst cd, dann snd (geht beides, mlohse)
//     snd->stop();
    
//     Time t;
//     Interval i;
//     i.sec = seconds;
//     long num_bytes = 0;
    
//     snd->sendSetEvent(Event("SEEK", new TInValue<Interval>(i), new TInValue<Time>(t), new TInValue<long>(num_bytes)));
    
//     //   cout << "SEEK " << i.sec << " RETURN " 
//     //        <<  seek_result
//     //        << endl;
    
//     snd->start();
//     cd->start();
  }
  
  void setTrack(unsigned int no) {
    if (1 > no) {
      return;
    }
    
    audio_play->stop();
    cd->stop();
    
    Result switchtrack = FAILURE;
    try {
      switchtrack = cdda->setTrack(no);
    }
    catch(...) {
      cout << endl << "Could not switch to Track #" << no << endl;
    }
    
    audio_play->flush();
    if (FAILURE != switchtrack) {
      cout << "Play now Track #" << no << endl;
    }
    
    audio_play->start();
    cd->start();
  }
  
  void setNextTrack() {
    int curr_track = cdda->getTrack();
    int total_tracks = cdda->getTotalTracks();
    if (curr_track >= total_tracks) {
      return;
    }
    
    unsigned int no = 0;
    
    cd->stop();
    audio_play->stop();
    audio_play->flush();
    
    try {
      no = curr_track+1;
      cdda->setTrack(no);
    }
    catch(...) {
      cout << endl << "Could not switch to Track #" << no << endl;
      cdda->setTrack(1);
    }
    
    GLOBAL_DEBUG_STREAM("Play now Track #" << no << endl); 
    audio_play->start();
    cd->start();
  }


  void setPrevTrack() {
    int curr_track = cdda->getTrack();
    if (1 >= curr_track) {
      return;
    }
    
    int no = 0;
    cd->stop();
    audio_play->stop();
    audio_play->flush();

    try {
      no = curr_track-1;
      cdda->setTrack(no);
    }
    catch(...) {
      GLOBAL_ERROR_STREAM("Could not switch to Track #" << no << endl);
      try {
        int total_tracks = cdda->getTotalTracks();
        cdda->setTrack(total_tracks);
      } catch(...) {
        GLOBAL_ERROR_STREAM("Could not switch to last track of the cd." << endl);
      }
    }
  
    GLOBAL_DEBUG_STREAM("Play now Track #" << no  << endl);
    audio_play->start();  
    cd->start();
  }
  
  void setStop() {
    GLOBAL_DEBUG_STREAM( "Stopping CD ... " << is_playing << endl);
    
    if (is_playing) {
      audio_play->stop();
      cout << "Pausing ..." << endl;
      is_playing = false;
    }
  }
  
  void setContinue() {
    if (!is_playing) {
      audio_play->start();
      cout << "Continuing ..." << endl;
      is_playing = true;
    }
  }
  
  void disableAllMessages() {
    NamedObject::getGlobalInstance().setErrorStream(NULL, NamedObject::ALL_LEVELS);
    NamedObject::getGlobalInstance().setDebugStream(NULL, NamedObject::ALL_LEVELS);
    NamedObject::getGlobalInstance().setWarningStream(NULL, NamedObject::ALL_LEVELS);
    NamedObject::getGlobalInstance().setMessageStream(NULL, NamedObject::ALL_LEVELS);
  }
  
  INode* requestNode(const NodeDescription& request) {
    list<Response> response = registry->initRequest(request);
    if (response.empty()) { return NULL; }
    else { return registry->requestNode(response.front()); }    
  }
  
  Result requestNodes() {
    for (unsigned int i=0; i<request.size(); ++i) {
      INode* node = requestNode(request[i]);
      if (node == NULL) { return FAILURE; }
      nodes.push_back(node);
    }
    
    return SUCCESS;
  }
  
  void startNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      nodes[i]->start();
    }
  }
  
  void stopNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      if (nodes[i]->isStarted()) {
        nodes[i]->stop();
      }
    }
  }
  
  void flushNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      nodes[i]->flush();
    }
  }
  
  void deactivateNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      if (nodes[i]->isActivated()) {
        nodes[i]->deactivate();
      }
    }
  }
  
  void deinitOutputNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      if (nodes[i]->isOutputInitialized()) {
        nodes[i]->deinitOutput();
      }
    }
  }
  
  void deinitNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      if (nodes[i]->isInitialized()) {
        nodes[i]->deinit();
      }
    }
  }
  
  void releaseNodes() {
    for (unsigned int i=0; i<nodes.size(); ++i) {
      registry->releaseNode(*nodes[i]);
      delete nodes[i];
    }
    
    nodes.clear();
  }
  
  void cleanup() {
    GLOBAL_DEBUG_STREAM("Stopping nodes..." << endl);
    stopNodes();
    GLOBAL_DEBUG_STREAM("Flushing nodes..." << endl);
    flushNodes();
    GLOBAL_DEBUG_STREAM("Deactivating nodes..." << endl);
    deactivateNodes();
    GLOBAL_DEBUG_STREAM("Output deinitializing nodes..." << endl);
    deinitOutputNodes();
    GLOBAL_DEBUG_STREAM("Deinitializing nodes..." << endl);
    deinitNodes();
    GLOBAL_DEBUG_STREAM("Releasing nodes..." << endl);
    releaseNodes();
    registry->releaseTransaction();
  }
  
public:
  int run() {
    // disable messages
    if (! verbose) {
      disableAllMessages();
    }
    
    // load plugins
    GLOBAL_DEBUG_STREAM("Loading plugins..." << endl);
    loadPlugins();
    
    // request nodes
    GLOBAL_DEBUG_STREAM("Requesting nodes..." << endl);
    registry->initTransaction();
    initNodeRequests();
    if (FAILURE == requestNodes()) return 1;
    registry->releaseTransaction();
    
    // init nodes
    GLOBAL_DEBUG_STREAM("Initializing nodes..." << endl);
    initNodes();
    
    // register event listeners
    registerEvents();
    
    // build and activate graph
    GLOBAL_DEBUG_STREAM("Connecting and activating nodes..." << endl);
    buildGraph();
    
    // start all nodes
    GLOBAL_DEBUG_STREAM("Starting nodes..." << endl);
    startNodes();
    
    int result = doRun();
    cleanup();
    return result;
  }
  
protected:
  void loadPlugins() {
    IPluginManager& pm = app->getPluginManager();
    pm.loadLibrary("libnmmcdda.so");
    pm.loadLibrary("libnmmaudio.so");
  }
  
  void usage(void) {
    cerr << "USAGE:" << endl;
    cerr << "  cddaplay [options]" << endl;
    cerr << endl;
    cerr << "OPTIONS:" << endl;
    cerr << " -c --cddb     : use CDDB-Server for track names" << endl;
    cerr << " -v --verbose  : enable debug output" << endl;
    cerr << " -d --device   : set CDROM device" << endl;
    exit(1);
  }
  
  void parseCommandLine(int argc, char* argv[]) {
    const char *opt_string = "cikvd";
    struct option options [] = {
      {"cddb",0,NULL,'c'},
      {"verbose",0,NULL,'v'},
      {"help",0,NULL,'h'},
      {"device",1,NULL,'d'},
      {NULL,0,NULL,0}
    };
    int ch,long_option_index;
    
    while((ch=getopt_long(argc,argv,opt_string,options,&long_option_index))!=EOF){
      // which option did we get?
      switch (ch) {
      case 'v':
        verbose = true;
        break;
      case 'c':
        use_cddb = true;
        break;
      case 'd':
        if (optarg) {
          device = optarg;
          break;
        } else {
          usage();
        }
      case 'h':
        usage();
        break;
      default:
        usage();
      }
    }
  }
  
  void initNodeRequests() {
    NodeDescription desc;
    
    desc.setNodeName("CDDANode");
    request.push_back(desc);
    desc.setNodeName("PlaybackNode");
    request.push_back(desc);
  }
  
  void initNodes() {
    // init cd player
    cd = nodes[0];
    ICDDA_var temp_cdda(cd->getParentObject()->getInterface<ICDDA>());
    cdda = temp_cdda;
    cdda->setDevice(device);
    cd->init();    
    
    if (use_cddb) {
      ICDDB_var temp_cddb = ICDDB_var(cd->getParentObject()->getInterface<ICDDB>());
      cddb = temp_cddb;
      cddb->setCDDBServer("freedb.freedb.org", 8880);
      cddb->setCDDB(true);
    }
    
    cdda->setRepeat(false);
    
    // init playback node
    audio_play = nodes[1];
    IAudioDevice_var temp_audio_device(audio_play->getParentObject()->getInterface<IAudioDevice>());
    audio_device = temp_audio_device;
    audio_device->setDevice("/dev/dsp");
    audio_play->init();
  }
  
  void buildGraph() {
    cd->initOutput();
    
    // connect cd to playback
    connect(cd, audio_play);
    
    cd->activate();
    
    audio_play->initOutput();
    audio_play->activate();
  }
  
  void registerEvents() {
    EventDispatcher& dispatcher = audio_play->getParentObject()->getEventDispatcher();
    dispatcher.registerEventListener(ITrack::finished_event,
                                     new TEDObject0<CDDAExample>(this, &CDDAExample::finished)); 
    dispatcher.registerEventListener(ITrack::startTrack_event,
                                     new TEDObject1<CDDAExample, string, Result(CDDAExample::*)(const string&)>(this, &CDDAExample::startTrack)); 
    dispatcher.registerEventListener(ITrack::endTrack_event,
                                     new TEDObject0<CDDAExample>(this, &CDDAExample::endTrack)); 
    dispatcher.registerEventListener("toc",
                                     new TEDObject1<CDDAExample, string>(this, &CDDAExample::eventToc)); 
    dispatcher.registerEventListener("truncated",
                                     new TEDObject1<CDDAExample, string>(this, &CDDAExample::eventTruncated  ));
    dispatcher.registerEventListener("seeked",
                                     new TEDObject1<CDDAExample, string>(this, &CDDAExample::eventSeeked  ));
  }
  
  int doRun() {
    /* --- dance --- */
    bool finished = false;
    char buffer = 0;
    unsigned int c;
    while (!finished) {
      cin >> buffer;
      
      switch (buffer) {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        c = ((unsigned int) buffer) - 48;
        if (0 == c) {
          c = 10;
        }
        setTrack(c);
        break;
      case '+':
        setNextTrack();
        break;
      case '-':
        setPrevTrack();
        break;
      case 'f':
        seek(5);
        break;
      case 'r':
        seek(-5);
        break;
      case 'F':
        seek(30);
        break;
      case 'R':
        seek(-30);
        break;
      case 'p':
        setStop();
        break;
      case 'c':
        setContinue();
        break;
      case 'q':
        finished = true;
        break;
      default:
        break;
      }
    }
    
    return 0;
  }
};

int main(int argc, char* argv[]) {
  try {
    CDDAExample example(argc, argv);    
    return example.run();
  } catch(Exception e) {
    cerr << e << endl;
  } catch(...) {
    cerr << "Something went wrong..." << endl;
    return 1;
  }
}
