/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *                     
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include <iostream>
#include <unistd.h>

#include "nmm/base/connect.hpp"
#include "nmm/base/ProxyObject.hpp"

#include "nmm/interfaces/file/IFileHandler.hpp"
#include "nmm/interfaces/file/ISinkFileHandler.hpp"
#include "nmm/interfaces/file/ITrack.hpp"

#include "nmm/base/ProxyApplication.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"

  

using namespace NMM;

/* Listener class, catch events from the nodes */
class Listener2 {
private:
  bool finished;
public:
  Listener2() { finished = false; }
  virtual Result receiveEvent(const Event&) { return SUCCESS; }  
  Result setfinishedflag() {finished = true; return SUCCESS; }
  bool getfinishedflag() { return finished;}
};

void usage(void) {
  cerr << "USAGE:" << endl;
  cerr << "  mp32wav <file>" << endl;
  cerr << endl;
  cerr << "OPTIONS:" << endl;
  cerr << " <none>" << endl;
  cerr << endl;
  cerr << "ATTENTION:" << endl;
  cerr << " At the moment the wav file have 44100 Hz 16Bit, stereo" << endl;
  cerr << endl;  
  exit(1);
}

int main(int argc, char ** argv) {
  Listener2* l = new Listener2(); 
  NMMApplication* app = ProxyApplication::getApplication(argc, argv);
  INode_var read;
  INode_var dec;
  INode_var write;

  try {
    //   NamedObject::getGlobalInstance().setWarningStream(NULL, NamedObject::ALL_LEVELS);
    //   NamedObject::getGlobalInstance().setMessageStream(NULL, NamedObject::ALL_LEVELS);
    //   NMM::NamedObject::getGlobalInstance().setErrorStream(NULL, NamedObject::ALL_LEVELS);
    //   NMM::NamedObject::getGlobalInstance().setDebugStream(NULL, NamedObject::ALL_LEVELS);
    //   NMM::NamedObject::getGlobalInstance().setWarningStream(NULL, NamedObject::ALL_LEVELS);
    //   NMM::NamedObject::getGlobalInstance().setMessageStream(NULL, NamedObject::ALL_LEVELS);

      string filename;
      if (2 > argc) usage(); else filename=argv[1];
      
      NodeDescription req1;
      NodeDescription req2;
      NodeDescription req3;
      req1.setNodeName("GenericReadNode");
      req2.setNodeName("MPEGAudioDecodeNode");
      req3.setNodeName("WavWriteNode");

      ClientRegistry& registry = app->getRegistry();
      
      list<Response> response1 = registry.initRequest(req1); 
      list<Response> response2 = registry.initRequest(req2); 
      list<Response> response3 = registry.initRequest(req3);

      if (response1.empty() || response2.empty() || response3.empty() ) {
	GLOBAL_ERROR_STREAM("Size of 1. response " << response1.size() << endl);
	GLOBAL_ERROR_STREAM("Size of 2. response " << response2.size() << endl);
	GLOBAL_ERROR_STREAM("Size of 3. response " << response3.size() << endl);
	throw Exception("A required node is not available");
      }

      read.reset(registry.requestNode(response1.front()));
      dec.reset(registry.requestNode(response2.front()));
      write.reset(registry.requestNode(response3.front()));

      IFileHandler_var iread(read->getParentObject()->getCheckedInterface<IFileHandler>());
      ISinkFileHandler_var iwrite(write->getParentObject()->getCheckedInterface<ISinkFileHandler>());
      
      iread->setFilename(filename);
      //iwrite->setFilename(filename);
      write->getParentObject()->registerEventListener(ITrack::endTrack_event, new TEDObject0<Listener2>(l, &Listener2::setfinishedflag));
      
      read->init(); dec->init(); write->init();
      read->initOutput(); connect(read,dec); read->activate();
      dec->initOutput(); connect(dec,write); dec->activate();
      write->initOutput(); write->activate();
      read->start(); dec->start(); write->start();

      while (!l->getfinishedflag()) {sleep(1);}
      
      read->stop(); dec->stop(), write->stop();
      read->deactivate(); dec->deactivate(); write->deactivate();
      read->deinitOutput(); dec->deinitOutput(); write->deinitOutput();
      read->deinit(); dec->deinit(); write->deinit();

      registry.releaseNode(*read);
      registry.releaseNode(*dec);
      registry.releaseNode(*write);
  }

  catch(Exception e) {
    if (read.get())
      app->getRegistry().releaseNode(*read);
    if (dec.get())
      app->getRegistry().releaseNode(*dec);
    if (write.get())
      app->getRegistry().releaseNode(*write);
    cerr << e << endl;
    return 1;
  }

  catch(...) {
    if (read.get())
      app->getRegistry().releaseNode(*read);
    if (dec.get())
      app->getRegistry().releaseNode(*dec);
    if (write.get())
      app->getRegistry().releaseNode(*write);
    cerr << "Something went wrong..." << endl;
    return 1;
  }

  return 0;
}
