/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *                    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "nmm/base/connect.hpp"
#include "nmm/plugins/file/GenericReadNode.hpp"
#include "nmm/plugins/audio/mpegdec/MPEGAudioDecodeNode.hpp"
#include "nmm/plugins/audio/PlaybackNode.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>


using namespace NMM;

void usage( char *name ) {
  printf("\nUsage:\n");
  printf("%s mp3-file\n", name);
  printf("Example: %s nmm2/resources/audio/test.mp3 \n\n", name);
  exit(0);
};

int main(int argc, char ** argv) 
{
  
  try {
    int version = 1;
    
    char filename[256];
    
    if ( argc < 2 ) usage( argv[0] );
    
    strcpy( filename, argv[1] ); 
    
    if( argc == 3) {
      if(!strcmp(argv[2], "run")) version = 1;
      if(!strcmp(argv[2], "process")) version = 2;
      if(!strcmp(argv[2], "trigger")) version = 3;
      if(!strcmp(argv[2], "old")) version = 4;
    }
    
    GenericReadNode *readfile = new GenericReadNode();
    INode_var ireadfile(readfile->getInterface<INode>());
    IFileHandler_var ifile(readfile->getCheckedInterface<IFileHandler>());
    ifile->setFilename(argv[1]);
    readfile->init();
    
    //readfile->SetBufferSize( 2047 );
    
    MPEGAudioDecodeNode *mp3dec = new MPEGAudioDecodeNode("MP3DECODE", StreamQueue::MODE_SUSPEND, 5);
    INode_var imp3dec(mp3dec->getCheckedInterface<INode>());
    mp3dec->init();
    
    PlaybackNode *audio_play = new PlaybackNode("PLAYBACK", StreamQueue::MODE_SUSPEND, 5);
    INode_var iaudio_play(audio_play->getInterface<INode>());
    audio_play->init();
    
    readfile->initOutput();
    connect(ireadfile, imp3dec);
    mp3dec->initOutput();
    connect(imp3dec, iaudio_play);
    audio_play->initOutput();
  
    if(version != 4) {
      readfile->setThreadOwner(false);
      mp3dec->setThreadOwner(false);
      audio_play->setThreadOwner(false);
    }

    readfile->activate();
    mp3dec->activate();
    audio_play->activate();
  
  
    readfile->start();
    mp3dec->start();
    audio_play->start();
  
    Message * buffer = 0;
    unsigned steps = 50; // approx. 0:31 min

    // run
    if(version == 1) {
      while(true) {
	if(steps-- == 0) {
	  return 0;
	}

	readfile->run(1);

	mp3dec->run(1);
	audio_play->run(1);
	
	while(mp3dec->getWorkingFlag()) {
	  mp3dec->run(1);
	  audio_play->run(1);
	}

      }
    }

    // process
    if(version == 2) {
      while(true) {
	if(steps-- == 0) {
	  return 0;
	}

	buffer = readfile->triggerProcessBuffer(0);
	if(!buffer) continue;
	
	if(buffer->getType() == Message::BUFFER) {
	  buffer = mp3dec->triggerProcessBuffer(static_cast<Buffer*>(buffer));
	  if(!buffer) continue;
	}
	else {
	  continue; // no cevent handling for now
	}
	if(buffer->getType() == Message::BUFFER) {
	  audio_play->triggerProcessBuffer(static_cast<Buffer*>(buffer));
	}
	else {
	  continue; // no cevent handling for now
	}

	while(mp3dec->getWorkingFlag()) {
	  buffer = mp3dec->triggerProcessBuffer(0);
	  if(!buffer) continue;

	  if(buffer->getType() == Message::BUFFER) {
	    audio_play->triggerProcessBuffer(static_cast<Buffer*>(buffer));
	  }
	}
      }
    }

    // trigger
    if(version == 3) {
      while(true) {
	if(steps-- == 0) {
	  return 0;
	}

	buffer = readfile->triggerProcessBuffer(0);
	if(!buffer) continue;
	readfile->triggerSendMessage(buffer);
	buffer = mp3dec->triggerDequeueMessage();

	if(buffer->getType() == Message::BUFFER) {
	  buffer = mp3dec->triggerProcessBuffer(static_cast<Buffer*>(buffer));
	  if(!buffer) continue;
	  mp3dec->triggerSendMessage(buffer);
	  buffer = audio_play->triggerDequeueMessage();
	}
	else {
	  continue; // no cevent handling for now
	}

	if(buffer->getType() == Message::BUFFER) {
	  audio_play->triggerProcessBuffer(static_cast<Buffer*>(buffer));
	}
	else {
	  continue; // no cevent handling for now
	}

	while(mp3dec->getWorkingFlag()) {
	  buffer = mp3dec->triggerProcessBuffer(0);
	  if(!buffer) continue;
	  mp3dec->triggerSendMessage(buffer);
	  buffer = audio_play->triggerDequeueMessage();

	  if(buffer->getType() == Message::BUFFER) {
	    buffer = audio_play->triggerProcessBuffer(static_cast<Buffer*>(buffer));
	  }
	}
      }
    }

    if(version == 4) {
      while(true) {
	sleep(31); // approx. 50
	return 0;
      }
    }

  }
  catch(Exception e) {
    NODE_ERROR_STREAM( e.getParent(), e << endl);
    return 1;
  }
  catch(...) { 
    GLOBAL_ERROR_STREAM("Something went wrong...");
    return 1;
  }

  return 0;
}
