/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_CONVERTER_NODE_HPP
#define NMM_CONVERTER_NODE_HPP

#include "GenericProcessorNode.hpp"

namespace NMM{

  /** This class is base-class for all converter-objects and defines the interface
   */
  class ConverterObject {
  public :
    /** destructor
     */
    virtual ~ConverterObject() {};

    /** Abstrcat converter-method. Each ConverterObject muyst implement this method
     */
    virtual Buffer* convert(Buffer*) = 0;
  };

  template<class T>
  class TConverterObject : ConverterObject{
    typedef Buffer* (T::*CMethod)(Buffer*);
  public:
    TConverterObject(T* node, CMethod method) {this->node = node; this->method = method;}
    virtual Buffer* convert(Buffer* buffer) { return (node->*method)(buffer);}
  private:
    CMethod method;
    T* node;
  };

}
#endif

#ifndef NMM_GENERICBIDIRCONVERTERNODE_HPP
#define NMM_GENERICBIDIRCONVERTERNODE_HPP

#include "GenericProcessorNode.hpp"

namespace NMM{
  /** \class GenericConverterNode
   * \ingroup GenericNode
   * \brief To do
   */
  class GenericConverterNode : public GenericProcessorNode {

    typedef Buffer* (*Converter)(Buffer* source_buffer, const GenericConverterNode*);

  public:

    GenericConverterNode(const char* name = "GenericConverterNode",
			 const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE, 
			 const int size = StreamQueue::DEFAULT_SIZE)
      : GenericProcessorNode(name, queue_mode, size)
    {
      setType(INode::CONVERTER);
    }

    virtual ~GenericConverterNode() { }

    /** \fn addConverter(NMM::Format* i_format, NMM::Format* o_format , Converter converter)
     * \brief This method adds a static Converter method
     * \param i_format : Pointer to the input format
     * \param o_format : Pointer to the output format
     * \param	converter : Function pointer to the converter funcion
     */
    void addConverter(NMM::Format* i_format, NMM::Format* o_format , Converter c);

    /** \fn addConverter(NMM::Format* i_format, NMM::Format* o_format , ConverterObject* )
     * \brief This method adds a Converter-object.
     * \param i_format : Pointer to the input format
     * \param o_format : Pointer to the output format
     * \param	converter : Function pointer to the converter funcion
     */
    void addConverter(NMM::Format* i_format, NMM::Format* o_format, ConverterObject* );

    /*
     */
    virtual void setConverter(Converter c) {converter = c;}
    virtual void activate()
      throw (const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&);
  protected:

    virtual Message *processBuffer(Buffer *in_buffer);


  private:
	
    Converter converter;

    map<pair<const NMM::Format*,const NMM::Format*>, Converter > converter_map;
    map<pair<const NMM::Format*,const NMM::Format*>, ConverterObject* > object_converter_map;
  };
}
#endif

