/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_INTERFACE_FACTORY_HPP
#define NMM_INTERFACE_FACTORY_HPP

#include <map>
#include "nmm/utils/NamedObject.hpp"
#include "nmm/utils/Singleton.hpp"
#include "nmm/base/serialize/Value.hpp"

namespace NMM {

class Interface;
class NMMObject;

class IFactoryObject {
public:
  virtual IFactoryObject*  copy()                          const = 0;
  virtual Interface*       createInstance(NMMObject&) const = 0;
};


/** This class is obsolete and will be replaces with the ValueVactory!!!!
  */
class InterfaceFactory : public NamedObject {
  friend class CreateUsingNew<InterfaceFactory>;
public:
  void registerInterface(const IFactoryObject&, const string& interface_type);
  Interface* createInterface(const string& interface_type, NMMObject&);
  bool isInterfaceAvailable(const string& interface_type) const;
private:
  InterfaceFactory() : NamedObject("InterfaceFactory") {}
  ~InterfaceFactory();
private:
  typedef map<string, IFactoryObject*>::iterator InternalIt;
  typedef map<string, IFactoryObject*>::const_iterator CInternalIt;

  map<string, IFactoryObject*> __factory_map;
};


typedef Singleton<InterfaceFactory, CreateUsingNew> SInterfaceFactory;

template <class T>
class TIFactoryObject : public IFactoryObject {
public:
  TIFactoryObject(const string& type_name, const string& t_namespace = "NMM") : __type_name(type_name), __t_namespace(t_namespace), __value_register(type_name, t_namespace)          { SInterfaceFactory::getInstance().registerInterface(*this, __type_name); }
  virtual ~TIFactoryObject() {};
  virtual IFactoryObject* copy()                                       const { return new TIFactoryObject<T>(*this); }
  virtual Interface*          createInstance(NMMObject& receiver)      const { return new T(receiver); }
private:
  string __type_name;
  string __t_namespace;
  TConstructor<T> __value_register;
};




}

#endif
