/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_OBJECT_ACCEPTOR_HPP
#define NMM_OBJECT_ACCEPTOR_HPP

#include "nmm/interfaces/net/IObjectAcceptorImpl.hpp"

namespace NMM {

class IConnectionManager;
class TSAddress;  

/** This class is base class for all Acceptors. An Acceptor is used to accept a connection
  * that is established from an connector. The Transport-Strategies are used as connector.
  * If a connection is established the ObjectAcceptor must create a corresponding Transport Strategy
  */
class ObjectAcceptor : public IObjectAcceptorImpl {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  ObjectAcceptor();

/** Destructor
  */
  virtual ~ObjectAcceptor() {}
//@}

protected:
/** @name Connection methods
  * These methods can be access through the IObjectAcceptor.
  */
//@{
/** This method connects the Transport Strategy to the NMMObject with given NMMReference
  */
  virtual void connectToObject(const NMMReference& reference)
     throw(const BindingException&);

/** This method connects the Transport Strategy to the NMMObject with given service name
  */
  virtual void connectToService(const string& service)
    throw(const BindingException&);

/** This method returns a list of all NMMObjects that can be accessed through an service name.
  */
  virtual list<string >  getServiceList() const;
//@}
};

  
class TSAddressOwner {
public:
  virtual const TSAddress& getLocalAddress() const throw() = 0;
};

}

#endif


