/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_QUEUEDOUTPUTJACK_HPP_
#define _NMM_QUEUEDOUTPUTJACK_HPP_

#include "OutputJack.hpp"
#include "StreamQueue.hpp"

namespace NMM {

  class Message;
  class InputJack;

  /**
   * This is a very simple "pushing" input jack, which simply pushes
   * all incoming buffers along to its parent node.
   */
  class QueuedOutputJack : public OutputJack {

  public:

    /**
     * The main constructor. This constructor only sets the parent,
     * not the partner.
     * \param parent the parent node.
     * \param jack_group the jack group this jack belongs to
     */
    QueuedOutputJack(Node* node, 		    
		    const string& jack_tag = "default",
		    const string& name = "QueuedOutputJack",
		    const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
		    const unsigned int size = StreamQueue::DEFAULT_SIZE);

    /**
     * The destructor.
     */
    virtual ~QueuedOutputJack();

    /**
     * Connect to a partner jack.
     * \param jack partner jack.
     */
    virtual void connectTo(const InputJack* in_jack) const
      throw(const BindingException& );

    virtual Result receiveMessage(Message* message, const IMessageSender* jack = 0);

    virtual Result sendMessage(Message *message);

    /**
     * This function returns the next buffer or null if stream_queue is empty. 
     */
    inline Message* dequeueMessage() { return stream_queue->dequeueMessage(); }

    /** 
     * Returns the size of the stream_queue 
     */
    inline unsigned int getQueueSize() { return stream_queue->getSize(); }

    /** 
     * Flushes the stream_queue. 
     */
    Result flush();

    // methods for accessing the StreamQueue
    unsigned int getSize() const;
    unsigned int getMaxSize() const;
    Result setMaxSize(unsigned int size);
    StreamQueue::Mode getMode() const;
    Result setMode(StreamQueue::Mode queue_mode);

    // reimpemented from Jack
    virtual void receiveSignal(Signal sig);

  protected:
    virtual  void setTransportStrategy(TransportStrategy* strategy);
    /**
     * Disconnect from partner jack.
     * \returns SUCCESS if the disconnection was successful, FAILURE otherwise.
     */
    virtual void disconnect();

  protected:

    StreamQueue *stream_queue;    ///< The input stream_queue for this stream.

  };

}

#endif // _NMM_QUEUEDINPUTJACK_HPP_
