/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "ReferenceCount.hpp"

namespace NMM 
{

  ReferenceCount::ReferenceCount() : __counter(0)
  {
  }

  ReferenceCount::~ReferenceCount() 
  {
  }

  void ReferenceCount::increaseCount(const unsigned int& i)
  {
    MutexGuard mg(__mutex);
    __counter += i;
  }

  void ReferenceCount::decreaseCount(const unsigned int& i)
  {
    MutexGuard mg(__mutex);
    if(__counter < i) {
      GLOBAL_ERROR_STREAM("ReferenceCount is " << __counter << " and to small to decrease by " << i << ", setting to 0");
      __counter = 0;
    }
    else {
      __counter -= i;
    }
  }

  unsigned int ReferenceCount::getCount()
  {
    return __counter;
  }

}
