/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_FORMATEXCEPTION_HPP
#define _NMM_FORMATEXCEPTION_HPP

#include "Parameter.hpp"
#include "nmm/base/BaseExceptions.hpp"

namespace NMM
{

  class FormatException  : public Exception
  {
    // baseclass
  public:

    const char * m_message;

    FormatException(const char * message = 0);
    virtual ~FormatException(){};
    
    virtual void showException();
  };


  class UsergraphException: public FormatException
  {
  public:
    UsergraphException(const char * message);
    ~UsergraphException(){};
    virtual void showException();
  };

//  class UsergraphExceptionNoPath : public UsergraphException
//  {
//    user_node_list m_list;
//    const char * m_begin;
//    const char * m_end;
//
//  public:
//    UsergraphExceptionNoPath(user_node_list & list);
//    UsergraphExceptionNoPath(const char * begin, const char * end);
//    UsergraphExceptionNoPath(const char * destination);
//    ~UsergraphExceptionNoPath(){};
//    virtual void showException();
//  };
  
  class XmlErrorException : public FormatException
  {
  public:
    const char * m_file_name;

    XmlErrorException(const char * file_name = 0);
    virtual ~XmlErrorException(){};
    
    virtual void showException();
  };


  class GraphManagerException : public FormatException
  {
  public:
    const char * m_message;

    GraphManagerException(const char * message);
    virtual ~GraphManagerException(){};

    virtual void showException();
  };

  class GraphManagerExceptionDeviceNotExist : public GraphManagerException
  {
  public:
    const char * m_device_name;
    
    GraphManagerExceptionDeviceNotExist(const char * device_name);
    virtual ~GraphManagerExceptionDeviceNotExist(){};
    
    virtual void showException();
  };


  class ParameterException : public FormatException
  {
  public:
    const char * m_parameter_name;
    Parameter * m_parameter;

    ParameterException(const char * message,const char *parameter_name, Parameter * Parameter);
    ParameterException(const char * parameter_name, Parameter * parameter);
    ParameterException(const char * parameter_name);
    ParameterException(const char * message,const char * parameter_name);

    virtual ~ParameterException(){};

    virtual void showException();
  };

  class ParameterExceptionNotExist : public ParameterException
  {
  public:
    ParameterExceptionNotExist(const char * parameter_name);
    virtual ~ParameterExceptionNotExist(){};

    virtual void showException();
  };

  class ParameterExceptionWeightRangeError : public ParameterException
  {
  public:
    float m_weight;
    
    ParameterExceptionWeightRangeError(const char * parameter_name, float weight);
    virtual ~ParameterExceptionWeightRangeError(){};
    
    virtual void showException();
  };

  class ParameterExceptionFalseType : public ParameterException
  {
  public:
    Parameter * m_parameter_false;
    Parameter::PType m_original;
    Parameter::PType m_fail;
    
    ParameterExceptionFalseType(const char * parameter_name);
    ParameterExceptionFalseType(const char * parameter_name, Parameter * parameter_false);
    ParameterExceptionFalseType(const char * parameter_name, Parameter::PType first, Parameter::PType second);
    virtual ~ParameterExceptionFalseType(){};
    
    virtual void showException();
  };
}


#endif
