/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_IPARAMETER_HPP
#define _NMM_IPARAMETER_HPP

#include "Parameter.hpp"

namespace NMM 
{

  class OValueStream;
  class IValueStream;
  /**
     The class IParameter is derived from the parameter-class and stores 
     32-Bit Integer-Values.
  **/

  class IParameter : public Parameter 
  {

  private:
    //! The STL-List wich stores the INT32-values.
    map <int32_t, float> m_value_list;
    
    //! Iterator for this datatyp.
    mutable map<int32_t,float>::const_iterator m_value_counter;
    mutable map<int32_t,float>::const_iterator m_pair_counter;
    
    //! The default value
    int32_t m_default_value;
    
  public:
    /**
       The constructur of the class.
       \param string pName: the name of the parameter.
    **/
    
    IParameter(const string & pName = "");
    
    /**
       The constructor adds directly pValue to the Value(-List).
       \param string pName: the name of the parameter.
       \param void *pValue: a value.
       \param float pWeight: a weighting factor
    **/

    IParameter(string pName, int32_t pValue, float pWeight = DEFAULT_WEIGHT);
    
    /**
       The default destructor
    **/
    virtual ~IParameter();
    
    virtual void addValue(void * pValue);
    
    virtual void addValue(void * pValue, float pWeight = DEFAULT_WEIGHT);
    
    virtual void addValue(int32_t pValue, float weight = DEFAULT_WEIGHT);

    virtual void showParameter () const;
    
    virtual ostream& showParameter(ostream&) const;
    
    virtual ostream& showParameterVerbose(ostream&) const;

    virtual bool isValueIncluded(void * pValue) const;

    virtual bool isValueIncluded(int32_t value) const;

    virtual void * nextValue() const;
    
    /**
       Returns the next pair which consists of 
       an int32_t and a float, representing the value
       and the weight.
       \return INT32_Iterator: the next pair.
    **/
    virtual map<int32_t,float>::const_iterator nextPair() const;
    
    virtual Parameter *duplicate() const;
    
    virtual float getWeight(void *pValue) const;
    
    virtual bool isParameterIdentical(const Parameter *param) const;

    virtual bool isParameterMatching(const Parameter *param) const;
    
    virtual uint getNumberOfValues() const;
    
    virtual Parameter *getIntersection(const Parameter *reference) const;

    virtual void * getFirstValue() const;

    virtual float getStandardizedWeight() const;

    virtual void setDefaultValue(int32_t param);

    virtual int32_t getDefaultValue() const;

    virtual int32_t getLowestValue() const;

    virtual int32_t getHighestValue() const;

    virtual const int32_t_list getIntValueList() const;

    void fromStream(IValueStream& );
    void toStream(OValueStream& ) const;
  };
}
#endif
