/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "Parameter.hpp"
#include <iostream>

namespace NMM {

  Parameter::Parameter(const string & pName) 
  {
    m_parameter_name   = pName;
    m_parameter_weight = DEFAULT_WEIGHT;
  }
  
  Parameter::~Parameter()
  {
  }

  void Parameter::setParameterName(const string & param)
  {
    m_parameter_name = param;
  }

  const string & Parameter::getParameterName() const 
  {
    return m_parameter_name;
  }

  Parameter::PType Parameter::getParameterType() const 
  {
    return m_parameter_type;
  }

  Parameter * Parameter::duplicate() const
  {
    return NULL;
  }

  void Parameter::showParameterName() const
  {
    GLOBAL_DEBUG_STREAM2( m_parameter_name << endl);
  }

  void Parameter::setParameterWeight(float weight)
  {
    m_parameter_weight = weight;
  }

  float Parameter::getParameterWeight() const
  {
    return m_parameter_weight;
  }
  
  bool Parameter::isValueIncluded(int32_t /*value*/) const
  {
    return false;
  }

  bool Parameter::isValueIncluded(float /*value*/) const
  {
    return true;
  }

  bool Parameter::isValueIncluded(const string & /*value*/) const
  {
    return false;
  }

  bool Parameter::isValueIncluded(const Format * /*value*/) const
  {
    return false;
  }

  bool Parameter::isValueIncluded(void */*lValue*/, void */*hValue*/) const
  {
    return false;
  }

  bool Parameter::isValueIncluded(float /*lValue*/, float /*hValue*/) const
  {
    return false;
  }

  bool Parameter::isValueIncluded(int32_t /*lValue*/, int32_t /*hValue*/) const
  {
    return false;
  }

  bool Parameter::isRangeMatching(float /*lValue*/, float /*hValue*/) const
  {
    return false;
  }

  bool Parameter::isRangeMatching(int32_t /*lValue*/, int32_t /*hValue*/) const
  {
    return false;
  }

  ostream & operator << (ostream & os, const Parameter & format)
  {
    return format.showParameter(os);
  }
}
