/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_UDPGENERIC_HPP
#define NMM_UDPGENERIC_HPP

#include "nmm/base/NMMObject.hpp"
#include "nmm/utils/net/SockDGram.hpp"
#include "nmm/interfaces/net/IUDPControlImpl.hpp"

namespace NMM {
  
  /** Generic UDP socket.
   * This class manages a datagram socket and implements an interface for setting protocol specific options.
   */
  class UDPGeneric
    : public IUDPControlImpl
  {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Constructor
     */
    UDPGeneric();

    /** Destructor
     */
    virtual ~UDPGeneric();
    //@}


    /** @name UDPGeneric specifc methods
     */
    //@{
    /** This method returns the Sock that is used to read/write the data
     */
    virtual       Sock& getDatagramSocket()        { return __sock; }

    /** This method returns the Sock that is used to read/write the data
     */
    virtual const Sock& getDatagramSocket()  const { return __sock; }
    //@}
    
    
    /** @name IIPControl interface
     */
    //@{
    virtual void test() {}
    //@}
    
  protected:

  private:
    void createException(int) const;
  private:
    SockDGram      __sock;
    
  };

}
#endif
