/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef NMM_LTDll_HPP
#define NMM_LTDll_HPP

#include "nmm/base/BaseExceptions.hpp"

#include <string>
#include <ltdl.h>

namespace NMM {

/** This a C++ wrapper class for the ldl library
  */
class LTDll {
public:
/** Constructor
  */
  LTDll();

/** Destructor. Calls also close
  *\sa close
  */
  ~LTDll();

/** Loads a shared library.
  * \throw Exception if an error occurs.
  */
  void open(const string& name, int flag = 0)
    throw (const Exception& );

/** Close a loaderd shared library.
  * \throw Exception if an error occur.
  */
  void close()
    throw (const Exception& );

/** resolves a symbol with name 'symbol'
  * \return pointer to the given symbol, 0 otherwise
  * \sa getError
  */
  void* resolve(const string& symbol);

/** returns the last error message occured
  * \return string with the error message
  */
  inline  const string& getError() const  {return error;}

/** returns the name of the current library
  * \return string with the library name
  */
  inline  const string& getName()  const  {return name;}

/** This method returns the current handle
  * \return current handle to the shared library
  */
  inline  const void* getHandle()  const  {return handle;}
private:
  lt_dlhandle  handle; ///current handle pointer
  string error; ///latest error message
  string name;  ///current library name. is set when open is called

private: //disabled methods
  LTDll(const LTDll&);
  LTDll& operator=(const LTDll&);
  LTDll& operator==(const LTDll&);
  bool __is_valid;                          ///< is true if lt_dlinit() successfully returnd inside the constructor, else false
};

}//end namespace
#endif
