/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "NodeFactoryStructs.hpp"
#include "nmm/utils/utils.hpp"
namespace NMM {


NodeFactoryID::NodeFactoryID(const string& fac_name)
  : SerializedValue(TypeInfo("NodeFactoryID", "NMM"))
{
  __hostname = NMM::getHostname();
  __fac_name = fac_name;
  registerAttributes();
}

NodeFactoryID::NodeFactoryID(const NodeFactoryID& f_id)
  : SerializedValue(f_id)
{
  registerAttributes();
  *this = f_id;
}

void NodeFactoryID::registerAttributes() {
  insertArgument(&__hostname);
  insertArgument(&__fac_name);
}

NodeFactoryID& NodeFactoryID::operator=(const NodeFactoryID& id) {
  SerializedValue::operator=(id);

  __hostname = id.__hostname;
  __fac_name = id.__fac_name;
  return *this;
}


bool NodeFactoryID::operator==(const NodeFactoryID& id) const {
  if (getFactoryName() != id.getFactoryName())
    return false;

  return true;
}

FactoryTicket::FactoryTicket()
  : SerializedValue(TypeInfo("FactoryTicket", "NMM")), __is_valid(false)
{
  registerAttributes();
}

FactoryTicket::FactoryTicket(const FactoryTicket& ft)
  : SerializedValue(ft)
{
  registerAttributes();
  *this = ft;
}

FactoryTicket& FactoryTicket::operator=(const FactoryTicket& ft){
  SerializedValue::operator=(ft);

  /* Begin> assign the values in this struct */
  __config_event = ft.__config_event;
  __factory_id   = ft.__factory_id;
  __node_id      = ft.__node_id;
  __is_valid     = ft.__is_valid;
  /* End< */

  /* If you copy a ticket the old one is invalid  */
  ft.__is_valid = false;
  return *this;
}

void FactoryTicket::registerAttributes() {
  /* Begin> register the attributes */
  insertArgument(&__factory_id);
  insertArgument(&__node_id);
  insertArgument(&__is_valid);
  insertArgument(&__config_event);
  /* End< */
}

/* TConstructor to register both classes in the ValueFactory */
TConstructor<NodeFactoryID>    __nmm__nodefactory_id("NodeFactoryID", "NMM");
TConstructor<FactoryTicket>    __nmm__factory_ticket_id("FactoryTicket", "NMM");

}//end namespace

