/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger  <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
 * USA
 */

#include "RegistryMemento.hpp"

namespace NMM {
  MementoStruct::MementoStruct()
    : SerializedValue("MementoStruct", "NMM")
  {
    registerAttributes();
  }

  MementoStruct::MementoStruct(const MementoStruct& ms)
    : SerializedValue( ms )
  {
    registerAttributes();
    *this = ms;
  }

  void MementoStruct::registerAttributes()
  {
    registerAttribute( &__node_description );
    registerAttribute( &__object_id );
    registerAttribute( &__max_instance_count );
    registerAttribute( &__plugin_index );
  }

  RegistryMemento::RegistryMemento()
    : SerializedValue("RegistryMemento", "NMM")
  {
    registerAttributes();
  }

  RegistryMemento::RegistryMemento( const RegistryMemento& rm )
    : SerializedValue( rm )
  {
    registerAttributes();
    *this = rm;
  }

  void RegistryMemento::registerAttributes() {
    registerAttribute( &__plugin );
    registerAttribute( &__registry_id );
    registerAttribute( &__memento_list );
  }
  
  OValueStream& operator<<(OValueStream& ov, const list<MementoStruct>& string_list) {
    return writeList<MementoStruct>(ov, string_list, "MementoStruct");
  }

  IValueStream& operator>>(IValueStream& ov, list<MementoStruct>& string_list) {
    return readList<MementoStruct>(ov, string_list, "MementoStruct");
  }
  
}//end namespace

