/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse        <mlohse@cs.uni-sb.de>
 *                    Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_SESSION_HPP
#define NMM_SESSION_HPP

#include "nmm/interfaces/base/registry/ISessionImpl.hpp"

namespace NMM {

  class Session : public ISessionImpl {
  public:
    Session(const GraphDescription& gd);
    virtual ~Session();

  protected:
    /**
     * Returns the graph description of this sesssion.
     */
    virtual GraphDescription* getGraphDescription() const;

    /**
     * Returns the composite node of this sesssion.
     */
    virtual ICompositeNode* getCompositeNode() const;

    /**
     * Returns the source synchronizer of this sesssion.
     */
    virtual ISourceSynchronizer* getSourceSynchronizer() const;

    /**
     * Returns the sink synchronizer of this sesssion.
     */
    virtual ISinkSynchronizer* getSinkSynchronizer() const;

    /**
     * Sets the sink synchronizer for the session.
     */
    virtual void setSinkSynchronizer(const ISinkSynchronizer& synchronizer);

    /**
     * Sets the source synchronizer for the session.
     */
    virtual void setSourceSynchronizer(const ISourceSynchronizer& synchronizer);

    /**
     * Sets the graph description for the session.
     */
    virtual void setGraphDescription(const GraphDescription& gd);

    /**
     * Sets the composite node for the session.
     */
    virtual void setCompositeNode(const ICompositeNode& comp_node);

    virtual GraphDescription* getMasterGraph();

    virtual void setMasterGraph(const GraphDescription& master_graph);

    virtual void realizeGraph();

    virtual void activateGraph();

    virtual void startGraph();

    virtual void stopGraph();
  private:
    void increaseCount(const GraphDescription& gd);
    void decreaseCount(const GraphDescription& gd);
    void releaseNodes();
  private:
    GraphDescription_var    __gd;
    GraphDescription_var    __master_graph;
    ICompositeNode_var      __comp_node;
    ISourceSynchronizer_var __source_synchronizer;
    ISinkSynchronizer_var   __sink_synchronizer;
  };

  typedef auto_ptr<Session> Session_var;

}

#endif
