/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_RMINDEXVERTEX_HPP_
#define _NMM_RMINDEXVERTEX_HPP_

#include "RMVertex.hpp"
#include "nmm/base/StdNamedObject.hpp"
#include <map>
#include <string>

namespace NMM {

/**
 * This class implemets an index to allow faster access to vertices
 */
 
class RMIndexVertex : public StdNamedObject {
    
    string d_name; // dimension name
      
    //  value  pointer to indices
    map<QualityParameter, RMIndexVertex*> index;
    map<QualityParameter, RMVertex*> vertices;
    
    bool leaf; // if true the map contains pointers to RMVertices, if false the map contains pointers to RMIndexVertices
    
  public:
  
    RMIndexVertex(string dim_name,bool isleaf = false);
    
    RMVertex* getVertex(QualityParameter value); // returns a vertex with the specified value if there is one (in the actual dimension) or 0 if not
    RMVertex* getLowerVertex(QualityParameter value); // returns a vertex with the next lower value than specified (in the actual dimension) or 0 if there is no such vertex
    RMVertex* getHigherVertex(QualityParameter value); // returns a vertex with the next higher value than specified (in the actual dimension) or 0 if there is no such vertex
    
    RMIndexVertex* getNextIndex(const QualityParameter value); // go one dimension down (using value for the actual dimension) returns 0 if failure 
    RMIndexVertex* getLowerIndex(QualityParameter value); // returns an index with the next lower value than specified (in the actual dimension) or 0 if there is no such index or we are on leaf-level
    RMIndexVertex* getHigherIndex(QualityParameter value); // returns an index with the next higher value than specified (in the actual dimension) or 0 if there is no such index or we are on leaf-level
    RMIndexVertex* getLowerIndex(QualityParameter value, QualityParameter&  lower_value); // returns an index with the next lower value than specified (in the actual dimension) or 0 if there is no such index or we are on leaf-level. lower_value contains the next lower value
    RMIndexVertex* getHigherIndex(QualityParameter value, QualityParameter&  higher_value); // returns an index with the next higher value than specified (in the actual dimension) or 0 if there is no such index or we are on leaf-level. higher_value contains the next higher value
    
    Result addLeaf(QualityParameter value,RMVertex* vertex);
    Result addIndex(QualityParameter value,RMIndexVertex* vertex);
    Result eraseIndex(QualityParameter value);
    
    void deleteVertices();
    
    const bool isIncluded(QualityParameter value) const {
      if (leaf) {
        if (vertices.find(value) == vertices.end())
          return false;
        else
          return true;
      } else {
        if (index.find(value) == index.end())
          return false;
        else
          return true;
      }
    }
    const bool isLeaf() const { return leaf; }
    const string name() const { return d_name; }
    void setLeaf() { leaf = true; }
    
    
    // debug methods
    
    void printmap();
};

}


#endif
