/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_RESOURCES_HPP_
#define _NMM_RESOURCES_HPP_

#include "../serialize/ComplexValue.hpp"
#include <list>

namespace NMM {
  class Resources : public NMM::SerializedValue {

      TValue<float> max_processing_time;   // maximum processing time for 1 second of data
      TValue<float> avg_bandwidth;         // average bandwidth in bytes/sec only used for serialization
      TValue<float> max_bandwidth;         // maximum bandwidth in bytes/sec
      TValue<float> min_processing_time;   // minimum processing time for 1 second of data
      TValue<float> avg_processing_time;   // average processing time for 1 second of data only used for serialization
      TValue<float> min_bandwidth;         // minimum Sbandwidth in bytes/sec
      list<float> processing_time; // processing times for 1 second of data
      list<float> bandwidth;       // bandwidth in bytes/sec

      const float getAverage(const list<float>&) const;

      void registerValues();
      void updateAttributes();
      void prepareAttributes();

    public:

      Resources();
      Resources(const Resources&);
      ~Resources();

      const float getMaximumProcessingTime() const; // the maximum of the obseved processing times
      const float getAverageProcessingTime() const; // the average of the obseved processing times
      const float getMinimumProcessingTime() const; // the minimum of the obseved processing times

      const float getMaximumBandwidth() const; // the maximum of the obseved bandwith requirements
      const float getAverageBandwidth() const; // the average of the obseved bandwith requirements
      const float getMinimumBandwidth() const; // the minimum of the obseved bandwith requirements

      void setProcessingTime(float); // set a measurement point
      void setBandwidth(float);       // set a measurement point
  };
}
#endif
