/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_StdIStream_HPP_
#define _NMM_StdIStream_HPP_

#include "ValueStream.hpp"

#include <string>

namespace NMM {
/** This class is a wrapper class for all c++ OStreams. All values that are written, are forward
  * to the c++ ostream
  */
class StdOStream :public OStream {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  StdOStream(std::ostream*) ;

/** Destructor
  */
  virtual ~StdOStream() ;
//@}

/** @name Operator
  */
//@{
/** This method writes an u_int16 i to the stream and returns a reference to the stream.
  */ 
  virtual StdOStream& operator<<(const u_int16_t&)      ;

/** This method writes an int16 to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const int16_t&   )     ;

/** This method writes an u_int32 to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const u_int32_t&)      ;

/** This method writes an int32 to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const int32_t&   )     ;

/** This method writes an u_int64 i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const u_int64_t& )     ;

/** This method writes an int64 i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const int64_t& )       ;

/** This method writes a bool i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const bool& )          ;

/** This method writes an unsigned long i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const unsigned long&)  ;

/** This method writes a long i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const long& )          ;

/** This method writes a double i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const double& )        ;

/** This method writes a float i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const float& )         ;

/** This method writes a char i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const char& )          ;

/** This method writes a unsigned char i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const unsigned char&)  ;

/** This method writes a string i to the stream and returns a reference to the stream.
  */
  virtual StdOStream& operator<<(const string&)         ;

/** This method must be called if no other values are written.
  */
  virtual void writeData();
//@}

/** @name Write char-array
  */
//@{
/** This method writes an char buffer of given size to the stream.
  * \param buffer Poitner to the char array.
  * \param size Size of the buffer.
  */
  virtual void writeBuffer(const char* buffer, unsigned int size);

/** This method writes an unsigned char buffer of given size to the stream.
  * \param buffer Poitner to the unsigned char array.
  * \param size Size of the buffer.
  */
  virtual void writeBuffer(const unsigned char* buffer, unsigned int size);
//@}
private:
  std::ostream * __stream;           ///< pointer to the ostream
  char __fin;                        ///< fin flag to seperate the values
};


class StdIStream : public IStream {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  StdIStream(std::istream* = 0);
//@}

/** @name Operator
  */
//@{
/** This method reads an u_int16_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(u_int16_t&)       ;

/** This method reads an int16_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(int16_t&)         ;

/** This method reads an u_int32_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(u_int32_t&  )     ;

/** This method reads an int32_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(int32_t&  )       ;

/** This method reads an u_int64_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(u_int64_t&  )     ;

/** This method reads an int64_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(int64_t&  )       ;

/** This method reads a bool i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(bool&  )          ;

/** This method reads an unsigned long i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(unsigned long&)   ;

/** This method reads a long from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(long& )           ;

/** This method reads a double i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(double& )         ;

/** This method reads a float i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(float& )          ;

/** This method reads a char i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(char& )           ;

/** This method reads an u_int16_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(unsigned char&)   ;

/** This method reads an u_int16_t i from the stream and returns a reference to the stream.
  */
  virtual  StdIStream& operator>>(string&)          ;
//@}

/** @name Read char-array
  */
//@{
/** This method reads size chars and store they in the given buffer.
  * \param buffer The buffer to store the read char's. The minimum size of the array must be size
  * \param size The size of char's that should be read
  */
  virtual void readBuffer(char*, unsigned int size)           ;

/** This method reads size unsigned chars and store they in the given buffer.
  * \param buffer The buffer to store the read unsigned char's. The minimum size of the array must be size
  * \param size The size of char's that should be read
  */
  virtual void readBuffer(unsigned char* buffer, unsigned int size);
//@}

  /** Returns true if the stream does not contain any data
    */
  virtual bool isEmpty() const;
private:
  std::istream *__stream;     ///< the used istream
};



}

#endif // _NMM_StdIStream_HPP_


