/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "Timestamp.hpp"
#include "nmm/base/serialize/Value.hpp"

namespace NMM {

  ostream& operator<< (ostream& stream, const Timestamp& time) 
  {
    return stream << "Counter: "     << time.stream_counter
		  << ", created at " << time.sync_time
                  << ", valid "      << time.is_valid; 
  };

  OValueStream& operator<<(OValueStream& ov, const Timestamp& ts) {
    ov.beginType("Timestamp", "NMM");
    ov << ts.sync_time;
    ov << ts.stream_counter;
    ov << ts.is_valid;
    ov << ts.is_set;
    ov.endType();

    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, Timestamp& ts) {
    if (iv.getCurrentType() != "Timestamp" && iv.getCurrentNamespace() != "NMM") {
      GLOBAL_ERROR_STREAM("Invalid type assignment");
      iv.nextElement();
      return iv;
    }

    if (!iv.goDown()) {
      GLOBAL_ERROR_STREAM("Timstamp has no data");
      iv.nextElement();
      return iv;
    }

    iv >> ts.sync_time;
    iv >> ts.stream_counter;
    iv >> ts.is_valid;
    iv >> ts.is_set;

    iv.goUp();

    return iv;
  }

  TValueConstructor<Timestamp> timestamp_prototype("Timestamp", "NMM");
}

