/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _NMM_TIMESTAMP_HPP_
#define _NMM_TIMESTAMP_HPP_

#include "Types.hpp"

namespace NMM {
class IValueStream;
class OValueStream;

  //! The Timestamp contains time information about one message.
  /*!
   * With timestamps it is possible to send time information for each 
   * data buffer within the stream. 
   */
  struct Timestamp {
    Timestamp() : sync_time(0), stream_counter(0), is_valid(false), is_set(false) {}
    Time sync_time;          //! This time is used for synchronization.
    long stream_counter;     //! The number of the buffer in the stream.
    bool is_valid;           //! True if the timestamp has been set correctly.
    bool is_set;             //! True if the timestamp was set (also true for invalid timestamps)
  };

  //! Returns true if a is less than b.
  inline bool operator<(const Timestamp& a, const Timestamp& b)
  {
    return (a.sync_time < b.sync_time);
  };

  //! Returns true if a is greater than b.
  inline bool operator>(const Timestamp& a, const Timestamp& b)
  {
    return (a.sync_time > b.sync_time);
  };

  //! Returns true if a is less or equal than b.
  inline bool operator<=(const Timestamp& a, const Timestamp& b)
  {
    return (a.sync_time <= b.sync_time);
  };

  //! Returns true if a is greater or equal than b.
  inline bool operator>=(const Timestamp& a, const Timestamp& b)
  {
    return (a.sync_time >= b.sync_time);
  };

  //! The output operator of Timestamp.
  std::ostream& operator<< (std::ostream& stream, const Timestamp& time);

  OValueStream& operator<<(OValueStream& ov, const Timestamp& ts);

  IValueStream& operator>>(IValueStream& iv, Timestamp& ts);
}

#endif // _NMM_TIMESTAMP_HPP_

