
#ifndef NMM_IEchoCancellation_HPP
#define NMM_IEchoCancellation_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IEchoCancellation : virtual public Interface { 
  public:
    IEchoCancellation() {  setTypeInfo(TypeInfo(IEchoCancellation::IEchoCancellation_type, "NMM")); };

    IEchoCancellation(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IEchoCancellation::IEchoCancellation_type, "NMM")); setEventReceiver(parent); };

    virtual ~IEchoCancellation() {};

    static const char* IEchoCancellation_type;

    virtual  string getInterfaceType() const { return IEchoCancellation_type; }

    Interface* copy() const { return new IEchoCancellation(*this); }
    static const char* setMin_xf_event;
    static const char* setGeigelThreshold_event;
    static const char* setNLPAttenuation_event;
    static const char* setBufferShifting_event;
    static const char* setSampleShifting_event;
    static const char* setECEnabled_event;
    static const char* setESEnabled_event;
    static const char* setForwardEnabled_event;
    static const char* setDoBeep_event;
    static const char* increaseBufferShifting_event;
    static const char* decreaseBufferShifting_event;
    static const char* calculateBufferDelay_event;
    static const char* setDumpFile_event;
    static const char* setCalcDelayMode_event;
    static const char* calcDelayFinished_event;
    /**
     * minimum energy in xf. Range: M70dB_PCM to M50dB_PCM. 
     * Should be equal to microphone ambient Noise level 
     */

    virtual void setMin_xf(const float& min_xf)
      throw(const Exception&);
    /**
     * Double Talk Detector Speaker/Microphone Threshold. Range <=1
     * Large value (M0dB) is good for Single-Talk Echo cancellation, 
     * small value (M12dB) is good for Doulbe-Talk AEC
     */

    virtual void setGeigelThreshold(const float& geigelthreshold)
      throw(const Exception&);
    /**
     * for Non Linear Processor. Range >0 to 1. Large value (M0dB) is good
     * for Double-Talk, small value (M12dB) is good for Single-Talk
     */

    virtual void setNLPAttenuation(const float& nlpattenuation)
      throw(const Exception&);
    /**
     * Sets the constant value of buffers by which the audio streams are shifted against each other. 
     */

    virtual void setBufferShifting(const int& shifting_const)
      throw(const Exception&);
    /**
     * Sets the constant value of samples by which the audio streams are shifted against each other. 
     */

    virtual void setSampleShifting(const int& shifting_const)
      throw(const Exception&);
    /**
     * Enables/Disables Echo Cancellation 
     */

    virtual void setECEnabled(const bool& ece)
      throw(const Exception&);
    /**
     * Enables/Disables Echo Surpression
     */

    virtual void setESEnabled(const bool& ese)
      throw(const Exception&);
    /**
     * Node just forwards incoming buffer without changing them
     */

    virtual void setForwardEnabled(const bool& fe)
      throw(const Exception&);
    /**
     * Lets Node send a beep, which can be used do estimate delay.
     */

    virtual void setDoBeep(const bool& dB)
      throw(const Exception&);
    /**
     * Increases constant for buffer shifting in EchoCancellationNode
     */

    virtual void increaseBufferShifting()
      throw(const Exception&);
    /**
     * Decrease constant for buffer shifting in EchoCancellationNode
     */

    virtual void decreaseBufferShifting()
      throw(const Exception&);
    /**
     * Recalculate Buffer offset between speaker and mic stream in EchoCancellationNode.
     * Test offset for values between start and start plus buffer_range.
     */

    virtual int calculateBufferDelay(const int& backwards, const int& forwards)
      throw(const Exception&);
    /**
     * Sets filename to dump audiostreams to.
     */

    virtual void setDumpFile(const string& filename)
      throw(const Exception&);
    /**
     * Set EchoCancellationNode to this special mode to calculate echo delay.
     */

    virtual void setCalcDelayMode(const bool& enable)
      throw(const Exception&);
    /**
     * Used to notify application of finished delay calculation.
     * Argument is equal to calculated delay.
     */

    virtual void calcDelayFinished(const int& samples_delay)
      throw(const Exception&);


  };

  typedef auto_ptr<IEchoCancellation> IEchoCancellation_var;




}

#endif
