
#ifndef NMM_ITransportStrategyConfigImpl_HPP
#define NMM_ITransportStrategyConfigImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITransportStrategyConfig.hpp"


namespace NMM
{
  /** Transport strategy configuration interface.
   * This interface is for transport strategies that need special configuration and have only limited
   * support for sending and receiving NMM messages. It provides methods for configuring the transport
   * strategy and querying for its capabilities.
   */

  class ITransportStrategyConfigImpl  : public virtual NMMObject { 
  public:
    ITransportStrategyConfigImpl() { addInterface(new ITransportStrategyConfig(*this)); registerEvents(); };

    virtual ~ITransportStrategyConfigImpl() {};

  protected:
    /** This method configures the transport strategy for an input jack.
     * Some protocols need special configuration that depends on the stream format and whether data is sent or
     * received. This method configures transport strategies for such special protocols for use with input jacks.
     * The method must be called again whenever the stream format changes.
     * @param format Stream format
     */

    virtual void configureForInputJack(const Format& format) = 0;
    /** This method configures the transport strategy for an output jack.
     * Some protocols need special configuration that depends on the stream format and whether data is sent or
     * received. This method configures transport strategies for such special protocols for use with output jacks.
     * The method must be called again whenever the stream format changes.
     * @param format Stream format
     */

    virtual void configureForOutputJack(const Format& format) = 0;


  private:
    void registerEvents();
  };

}

#endif
