
#ifndef NMM_IResourceCollectorImpl_HPP
#define NMM_IResourceCollectorImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IResourceCollector.hpp"


namespace NMM
{

  class IResourceCollectorImpl  : public virtual NMMObject { 
  public:
    IResourceCollectorImpl() { addInterface(new IResourceCollector(*this)); registerEvents(); };

    virtual ~IResourceCollectorImpl() {};

  protected:
    /**
     * activate resource-management
     */

    virtual Result enableResourceProbing() = 0;
    /**
     * deactivate resource-management
     */

    virtual Result disableResourceProbing() = 0;
    /**
     * set the number of buffers to be processed between
     * the resource probes
     */

    virtual void setBufferProbeInterval(const unsigned int& value) = 0;
    /**
     * restrict the cpu usage of a node
     */

    virtual void restrictCPUUsage(const float& level) = 0;
    /**
     * returns how long the node used the cpu
     */

    virtual Interval getUsedCPUTime() = 0;
    /**
     * returns how long the node was started
     */

    virtual Interval getUsedTime() = 0;
    /**
     * returns the size of largest incoming buffer
     */

    virtual unsigned long getMaxReceivedBufferSize() = 0;
    /**
     * returns how many percent of the cputime the nodes thread used
     */

    virtual unsigned int getCPUUsage(const bool& since_start = false) = 0;
    /**
     * tell the node on what cpu it is running
     */

    virtual Result setCPU(const string& ci) = 0;
    /**
     * load the serialized mesh
     */

    virtual Result loadMesh(const string& s_mesh) = 0;

    virtual Result loadMeshes(const string& format_id, const string& s_mesh) = 0;
    /**
     * get the serialized mesh
     */

    virtual Result getMesh(string& s_mesh) = 0;
    /**
     * save the measured points to a file readable by HnSRTree-2.0
     */

    virtual Result savePoints(const string& filename) = 0;
    /**
     * compute estimated cpu usage of a certain format
     */

    virtual float getUsage(const Format& description) = 0;
    /**
     * get the estimated cpu usage for the actual connection format
     */

    virtual float getActualUsage() = 0;
    /**
     * get the estimated bandwidth for the actual connection format
     */

    virtual float getActualBandwidth() = 0;
   /**
     * get the size of an arriving buffer
     */

    virtual float getActualBufferSize() = 0;
    /**
     * return (average) measured cpu usage per processed buffer
     */

    virtual float getCPUperBuffer() = 0;
    /**
     * return the vertex that corresponds to the given format
     */

    virtual Result getVertex(const Format& description, RMVertex& vertex) = 0;
    /**
     * update resource-usage of an already existing vertex
     */

    virtual Result updateVertex(const Format& description, const float& cpu_usage, const float& bandwidth) = 0;
    /**
     * add a new vertex
     */

    virtual Result addVertex(const Format& description, const float& cpu_usage, const float& bandwidth) = 0;
    /**
     * setup a new mesh
     */

    virtual Result setupMesh(const Format& dimensions) = 0;
    /**
     * get the actually used format
     */

    virtual bool getActualFormat(Format& ret_format) = 0;


  private:
    void registerEvents();
  };

}

#endif
