
#ifndef NMM_IMultiAudioVideoSynchronizerImpl_HPP
#define NMM_IMultiAudioVideoSynchronizerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IMultiAudioVideoSynchronizer.hpp"

#include "IMultiAudioSinkSynchronizerImpl.hpp"
#include "IMultiVideoSinkSynchronizerImpl.hpp"

namespace NMM
{
  /**
   * Audio-video sink synchronizer.
   * Interface for a synchronizer which performs inter-stream synchronization between
   * multiple audio and video sinks relative to a master audio sink.
   */

  class IMultiAudioVideoSynchronizerImpl  : virtual public ISinkSynchronizerImpl, virtual public IMultiAudioSinkSynchronizerImpl, virtual public IMultiVideoSinkSynchronizerImpl { 
  public:
    IMultiAudioVideoSynchronizerImpl() { addInterface(new IMultiAudioVideoSynchronizer(*this)); registerEvents(); };

    virtual ~IMultiAudioVideoSynchronizerImpl() {};

  protected:
    /**
     * Set master audio sink.
     * Selects an audio sink as master audio sink. If this method is not called, the first
     * added audio sink becomes the master.
     * @param id audio sink id
     */

    virtual Result setMasterAudioSink(const int& id) = 0;
    /**
     * Set skew of an audio sink.
     * Sets the skew of an audio sink relative to the master audio sink.
     * @param id sink id
     * @param skew skew of this sink relative to master
     */

    virtual Result setAudioSkew(const int& id, const Interval& skew) = 0;
    /**
     * Set skew of a video sink.
     * Sets the skew of a video sink relative to the master audio sink.
     * @param id sink id
     * @param skew skew of this sink relative to master
     */

    virtual Result setVideoSkew(const int& id, const Interval& skew) = 0;


  private:
    void registerEvents();
  };

}

#endif
