
#ifndef NMM_ISinkSynchronizer_HPP
#define NMM_ISinkSynchronizer_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "ISynchronizer.hpp"
#include "ISinkController.hpp"

namespace NMM
{

  class ISinkSynchronizer : virtual public ISynchronizer { 
  public:
    ISinkSynchronizer() {  setTypeInfo(TypeInfo(ISinkSynchronizer::ISinkSynchronizer_type, "NMM")); };

    ISinkSynchronizer(NMMObject& parent) : ISynchronizer(parent) {  setTypeInfo(TypeInfo(ISinkSynchronizer::ISinkSynchronizer_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISinkSynchronizer() {};

    static const char* ISinkSynchronizer_type;

    virtual  string getInterfaceType() const { return ISinkSynchronizer_type; }

    Interface* copy() const { return new ISinkSynchronizer(*this); }
    static const char* setAverageLatency_event;
    static const char* setFirstBufferInTimeAverageLatency_event;
    static const char* removeSinkController_event;
    /**
     * Set average latency.
     * This method is used by the sink controllers and should not be called 
     * directly.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their incoming buffers.
     * @param average_latency The average_latency of incoming buffers computed
     *                        by the Controller.
     * @param sink sink ID
     */

    virtual void setAverageLatency(const Interval& value, const int& sink)
      throw(const Exception&);
    /**
     * Set average latency for first buffer in time after syncReset.
     * This method is used by the sink controllers and should not be called 
     * directly.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their first buffer in time after syncReset. 
     * @param average_latency The average_latency of incoming buffers computed
     *                        by the Controller.
     * @param sink sink ID
     * @param first_ts the timestamp of the first buffer in time
     * @return SUCCESS if value is good enough compared to theo_latency; FAILURE otherwise
     */

    virtual Result setFirstBufferInTimeAverageLatency(const Interval& value, const int& sink, const Timestamp& first_ts)
      throw(const Exception&);
    /**
     * Remove a sink.
     * Deletes the sink with the specified interface.
     * @param controller sink controller
     */

    virtual Result removeSinkController(const ISinkController* controller)
      throw(const Exception&);


  };

  typedef auto_ptr<ISinkSynchronizer> ISinkSynchronizer_var;




}

#endif
