
#include "ISwitch.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ISwitch::switchOutputs()
      throw(const Exception&)
  {
    Event event("ISwitch::switchOutputs");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISwitch::flushOutputs(const Timestamp& ts)
      throw(const Exception&)
  {
    TInValue<Timestamp > ts_value(ts, false);
    Event event("ISwitch::flushOutputs", &ts_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISwitch::setSwitchOutputsEnabled(const bool& enable)
      throw(const Exception&)
  {
    TInValue<bool > enable_value(enable, false);
    Event event("ISwitch::setSwitchOutputsEnabled", &enable_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ISwitch::getSwitchOutputsEnabled()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ISwitch::getSwitchOutputsEnabled");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISwitch::ISwitch_type = "ISwitch";
  TIFactoryObject<ISwitch > __ISwitch_prototype("ISwitch");


  const char* ISwitch::switchOutputs_event = "ISwitch::switchOutputs";
  const char* ISwitch::flushOutputs_event = "ISwitch::flushOutputs";
  const char* ISwitch::setSwitchOutputsEnabled_event = "ISwitch::setSwitchOutputsEnabled";
  const char* ISwitch::getSwitchOutputsEnabled_event = "ISwitch::getSwitchOutputsEnabled";


}


