
#include "ISynchronizer.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ISynchronizer::pause()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ISynchronizer::pause");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISynchronizer::wakeup()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ISynchronizer::wakeup");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISynchronizer::reset(const int& sink)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > sink_value(sink, false);
    Event event("ISynchronizer::reset", &sink_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISynchronizer::ISynchronizer_type = "ISynchronizer";
  TIFactoryObject<ISynchronizer > __ISynchronizer_prototype("ISynchronizer");


  const char* ISynchronizer::pause_event = "ISynchronizer::pause";
  const char* ISynchronizer::wakeup_event = "ISynchronizer::wakeup";
  const char* ISynchronizer::reset_event = "ISynchronizer::reset";


}


