
#include "IDevice.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IDevice::setDevice(const string& value) /* limited(INode::CONSTRUCTED) */ 
      throw(const Exception&, const IllegalStateException&, const DeviceNotFoundException&, const WrongDeviceException&)
  {
    TInValue<string > value_value(value, false);
    Event event("IDevice::setDevice", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(DeviceNotFoundException* ex = dynamic_cast<DeviceNotFoundException*>(event.getException())) {
          throw DeviceNotFoundException(*ex);
        }
        if(WrongDeviceException* ex = dynamic_cast<WrongDeviceException*>(event.getException())) {
          throw WrongDeviceException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  string IDevice::getDevice() const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IDevice::getDevice");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IDevice::IDevice_type = "IDevice";
  TIFactoryObject<IDevice > __IDevice_prototype("IDevice");


  const char* IDevice::setDevice_event = "IDevice::setDevice";
  const char* IDevice::getDevice_event = "IDevice::getDevice";


}


