
#include "ISeekable.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  bool ISeekable::hasPercentSeek() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ISeekable::hasPercentSeek");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISeekable::seekPercent(const Rational& percent)
      throw(const Exception&)
  {
    TInValue<Rational > percent_value(percent, false);
    Event event("ISeekable::seekPercent", &percent_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISeekable::seekPercentTo(const Rational& percent)
      throw(const Exception&)
  {
    TInValue<Rational > percent_value(percent, false);
    Event event("ISeekable::seekPercentTo", &percent_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ISeekable::hasBytesSeek() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ISeekable::hasBytesSeek");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISeekable::seekBytes(const int64_t& offset)
      throw(const Exception&)
  {
    TInValue<int64_t > offset_value(offset, false);
    Event event("ISeekable::seekBytes", &offset_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISeekable::seekBytesTo(const u_int64_t& bytes)
      throw(const Exception&)
  {
    TInValue<u_int64_t > bytes_value(bytes, false);
    Event event("ISeekable::seekBytesTo", &bytes_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ISeekable::hasTimeSeek() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ISeekable::hasTimeSeek");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISeekable::seekTime(const Interval& interval)
      throw(const Exception&)
  {
    TInValue<Interval > interval_value(interval, false);
    Event event("ISeekable::seekTime", &interval_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISeekable::seekTimeTo(const Time& time)
      throw(const Exception&)
  {
    TInValue<Time > time_value(time, false);
    Event event("ISeekable::seekTimeTo", &time_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ISeekable::ISeekable_type = "ISeekable";
  TIFactoryObject<ISeekable > __ISeekable_prototype("ISeekable");


  const char* ISeekable::hasPercentSeek_event = "ISeekable::hasPercentSeek";
  const char* ISeekable::seekPercent_event = "ISeekable::seekPercent";
  const char* ISeekable::seekPercentTo_event = "ISeekable::seekPercentTo";
  const char* ISeekable::hasBytesSeek_event = "ISeekable::hasBytesSeek";
  const char* ISeekable::seekBytes_event = "ISeekable::seekBytes";
  const char* ISeekable::seekBytesTo_event = "ISeekable::seekBytesTo";
  const char* ISeekable::hasTimeSeek_event = "ISeekable::hasTimeSeek";
  const char* ISeekable::seekTime_event = "ISeekable::seekTime";
  const char* ISeekable::seekTimeTo_event = "ISeekable::seekTimeTo";


}


