
#ifndef NMM_ISeekableImpl_HPP
#define NMM_ISeekableImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ISeekable.hpp"


namespace NMM
{

  class ISeekableImpl  : public virtual NMMObject { 
  public:
    ISeekableImpl() { addInterface(new ISeekable(*this)); registerEvents(); };

    virtual ~ISeekableImpl() {};

  protected:
    /**
     * this method can be used to test if this interface supports
     * seeking by percentage.
     * \returns true iff percantage seeking is supported.
     */

    virtual bool hasPercentSeek() const = 0;
    /**
     * this method seeks relative from the current position in
     * percent.
     * \param percent The offset to seek in a Rational from -1 to 1.
     */

    virtual void seekPercent(const Rational& percent) = 0;
    /**
     * this method seeks absolute to the given position in percent.
     * \param percent The position to seek to in a Rational from 0 to 1.
     */

    virtual void seekPercentTo(const Rational& percent) = 0;
    /**
     * this method can be used to test if this interface supports
     * seeking by bytes.
     * \returns true iff bytewise seeking is supported.
     */

    virtual bool hasBytesSeek() const = 0;
    /**
     * this method seeks relative from the current position in
     * bytes.
     * \param offset The offset to seek.
     */

    virtual void seekBytes(const int64_t& offset) = 0;
    /**
     * this method seeks absolute to the given position in percent.
     * \param bytes The position to seek to.
     */

    virtual void seekBytesTo(const u_int64_t& bytes) = 0;
    /**
     * this method can be used to test if this interface supports
     * seeking by Time.
     * \returns true iff seeking by Time is supported.
     */

    virtual bool hasTimeSeek() const = 0;
    /**
     * this method seeks relative from the current position in
     * Time.
     * \param interval The Interval to seek.
     */

    virtual void seekTime(const Interval& interval) = 0;
    /**
     * this method seeks absolute to the given position in Time.
     * \param time The Time to seek to.
     */

    virtual void seekTimeTo(const Time& time) = 0;


  private:
    void registerEvents();
  };

}

#endif
