
#include "ITrack.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ITrack::startTrack(const string& filename) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > filename_value(filename, false);
    Event event("ITrack::startTrack", &filename_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITrack::endTrack() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITrack::endTrack");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITrack::finished() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITrack::finished");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITrack::ITrack_type = "ITrack";
  TIFactoryObject<ITrack > __ITrack_prototype("ITrack");


  const char* ITrack::startTrack_event = "ITrack::startTrack";
  Event* ITrack::create_startTrack(const string& filename)
  {
    TInValue<string >* filename_value = new TInValue<string >();
    filename_value->setValue(filename);

    return new Event("ITrack::startTrack", filename_value);
  }

  const char* ITrack::endTrack_event = "ITrack::endTrack";
  Event* ITrack::create_endTrack()
  {

    return new Event("ITrack::endTrack");
  }

  const char* ITrack::finished_event = "ITrack::finished";
  Event* ITrack::create_finished()
  {

    return new Event("ITrack::finished");
  }



}


