
#ifndef NMM_ITimeProgress_HPP
#define NMM_ITimeProgress_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   *  @brief Progress sender interface
   *
   *  A node implementing ITimeProgress can send time progress information,
   *  e.g. to inform the application about the current playback time
   *  of an audio or video file.
   */

  class ITimeProgress : virtual public Interface { 
  public:
    ITimeProgress() {  setTypeInfo(TypeInfo(ITimeProgress::ITimeProgress_type, "NMM")); };

    ITimeProgress(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITimeProgress::ITimeProgress_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITimeProgress() {};

    static const char* ITimeProgress_type;

    virtual  string getInterfaceType() const { return ITimeProgress_type; }

    Interface* copy() const { return new ITimeProgress(*this); }
    static const char* setProgressUpdateInterval_event;
    static const char* setProgressUpdateEnabled_event;
    /**
     *  @brief Sets the time interval after which progress events are sent (default 1s).
     *
     *  @param The time interval after which a time progress event is sent.
     */

    virtual void setProgressUpdateInterval(const Interval& update_interval)
      throw(const Exception&);
    /**
     *  @brief Enable or disable sending progress information
     *  @param True to enable sending progress information
     *
     *  This method enables or disables sending of progress information. If enables, this node will send progress
     *  events at regular intervals.
     */

    virtual void setProgressUpdateEnabled(const bool& enable_update)
      throw(const Exception&);


  };

  typedef auto_ptr<ITimeProgress> ITimeProgress_var;




}

#endif
