/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        chwellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "ChannelDescription.hpp"

namespace NMM{
  ChannelDescription::ChannelDescription()
    :NMM::SerializedValue("ChannelDescription", "NMM")
  {
    registerValues();
  }

  ChannelDescription::ChannelDescription(const ChannelDescription& cd)
    :NMM::SerializedValue("ChannelDescription", "NMM")
  {
    registerValues();
    *this = cd;
  }

  ChannelDescription::~ChannelDescription(){}

  void ChannelDescription::setID(string channelid)
  {
    id = channelid;
  }

  string ChannelDescription::getID() const  {
    return id.getRValue();
  }
  string ChannelDescription::getID(){
    return id.getRValue();
  }

  void ChannelDescription::addChannelName(string name)
  {
    (names.getRValue()).push_back(name);
    (names.getRValue()).sort();
    (names.getRValue()).unique();                       
  }
  void ChannelDescription::setChannelNames(list<string> namelist)
  {
    names = namelist;
  }

  list<string> ChannelDescription::getChannelNames() const {
    return names.getRValue();
  }
  list<string> ChannelDescription::getChannelNames() {
    return names.getRValue();
  }

  ChannelDescription* ChannelDescription::copy(){
    return new ChannelDescription(*this);
  }

  void ChannelDescription::registerValues(){
    insertArgument(&id);
    insertArgument(&names);
  }


  OValueStream& operator<<(OValueStream& os, const list<ChannelDescription>& cd){
    return writeList<ChannelDescription>(os, cd, "ChannelDescription");
  }
  IValueStream& operator>>(IValueStream& is, list<ChannelDescription>& cd){
    return readList<ChannelDescription>(is, cd, "ChannelDescription");
  }

  TConstructor<ChannelDescription> nmm_cd("ChannelDescription", "NMM");
  TValueConstructor<list<ChannelDescription> > nmm_cd_list("list_ChannelDescription","std");


  OValueStream& operator<<(OValueStream& os, const list<ChannelDescriptionList>& cl){
    return writeList<ChannelDescriptionList>(os, cl, "ChannelDescriptionList");
  }
  IValueStream& operator>>(IValueStream& is, list<ChannelDescriptionList>& cl){
    return readList<ChannelDescriptionList>(is, cl, "CHannelDescriptionList");
  }
  
  TConstructor<ChannelDescriptionList> nmm_cl("ChannelDescriptionList", "NMM");
  TValueConstructor<list<ChannelDescriptionList> > nmm_cl_list("list_ChannelDescriptionList","NMM");

}
