/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        chwellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef CHANNELDESCRIPTION_HPP
#define CHANNELDESCRIPTION_HPP CHANNELDESCRIPTION_HPP

#include <iostream>
#include <list>
#include <string>

#include "nmm/base/serialize/ComplexValue.hpp"


namespace NMM{

  class ChannelDescription : public SerializedValue
  {
  private:
    NMM::TValue<string> id;
    NMM::TValue<list<string> > names;

  public:
    
    ChannelDescription();
    ChannelDescription(const ChannelDescription&);
    ~ChannelDescription();

    /**
     *Set the ID for this channel
     *@param channel-ID
     */
    void setID (string channelid);
    /**
     *Get the channel-ID of this channel
     *@return string channel-ID
     */
    string getID() const;
    /**
     *Get the channel-ID of this channel
     *@return string channel-ID
     */
    string getID();

    /**
     *Adds a Channel-name for this channel
     *@param channel-name
     */
    void addChannelName(string name);
    /**
     *Sets a list of channel-names
     *@param list of channel-names
     */
    void setChannelNames(list<string> namelist);

    /**
     *Get all the names set for this channel
     *@return list<string> channel-names
     */
    list<string> getChannelNames() const;
    /**
     *Get all the names set for this channel
     *@return list<string> channel-names
     */
    list<string> getChannelNames();
    
    //Interface SerialzedValue
    ChannelDescription* copy();

  private:
    void registerValues();

  };

  OValueStream& operator<<(OValueStream& os, const list<ChannelDescription>& cd);
  IValueStream& operator>>(IValueStream& is, list<ChannelDescription>& cd);



  class ChannelDescriptionList : public SerializedValue{
  private:
    NMM::TValue<list<ChannelDescription> > internal_list;

  public:
    ChannelDescriptionList() : SerializedValue("ChannelDescriptionList", "NMM"){registerAttributes();}
    ChannelDescriptionList(const ChannelDescriptionList &pl): SerializedValue("ChannelDescriptionList", "NMM"){*this=pl; registerAttributes();}
    ChannelDescriptionList(list<ChannelDescription> l): SerializedValue("ChannelDescriptionList", "NMM"){internal_list = l;registerAttributes();}
  private:
    void registerAttributes(){insertArgument(&internal_list);}

  public:
    list<ChannelDescription> getList() const {return internal_list.getRValue();}
    unsigned int size() const {return internal_list.getRValue().size();}
    void pus_back(ChannelDescription cd){internal_list.getRValue().push_back(cd);}
    ChannelDescription back() const {return internal_list.getRValue().back();}
    void pop_back() {internal_list.getRValue().pop_back();}
  };

  OValueStream& operator<<(OValueStream& os, const list<ChannelDescriptionList>& pl);
  IValueStream& operator>>(IValueStream& is, list<ChannelDescriptionList>&pl);

}

#endif
