
#ifndef NMM_ITVTimerImpl_HPP
#define NMM_ITVTimerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITVTimer.hpp"

#include "nmm/interfaces/misc/tvguide/ITVGuideImpl.hpp"
#include "nmm/interfaces/base/registry/IServerRegistryImpl.hpp"

namespace NMM
{

  class ITVTimerImpl  : public virtual NMMObject { 
  public:
    ITVTimerImpl() { addInterface(new ITVTimer(*this)); registerEvents(); };

    virtual ~ITVTimerImpl() {};

  protected:
    /**
       Program Timer using TvTimerEntry 
    */

    virtual Result programTimer(const TVTimerEntry& entry) 
      throw(const TimerAlreadyProgrammedException&) = 0;
    /**
       ChangeTimer using TvTimerEntry 
    */

    virtual Result changeTimer(const TVTimerEntry& newEntry) 
      throw(const ActionNotAllowedException&) = 0;
    /**
       Delete Timer with ID i 
    */

    virtual Result deleteTimer(const int& i, const string& user = "") 
      throw(const ActionNotAllowedException&) = 0;

    virtual list<TVTimerEntry >  getTimers(const string& day = "", const string& channel = "") = 0;
    /**
       Get the timer with the specified ID
    */

    virtual TVTimerEntry getTimer(const unsigned int& id) = 0;
    /**
       Get all timers with status SLEEPING
    */

    virtual list<TVTimerEntry >  getSleepingTimers() = 0;
    /**
       Get all timers with status RECORDING
    */

    virtual list<TVTimerEntry >  getRecordingTimers() = 0;
    /**
       Get all timers with status RECORDED or RECORDED_WITH_ERRORS
    */

    virtual list<TVTimerEntry >  getRecordedTimers() = 0;
    /**
       Get all timers witch Conflicts
     */

    virtual list<TVTimerEntry >  getConflictTimers() = 0;
    /**
       Get all timers witch Conflicts with the given timer
     */

    virtual list<TVTimerEntry >  getConflictsWithTimer(const TVTimerEntry& entry) = 0;
    /**
       Load timers from XML-File
    */

    virtual Result loadTimer(const string& filename) = 0;
    /**
       Save timers to XMLFile
    */

    virtual Result saveTimer(const string& filename) = 0;
    /**
       Set the pointer to the TVGuide
    */

    virtual void setTVGuide(const ITVGuide* itvg) = 0;
    /**
       Get the pointer to the TVGuide
    */

    virtual ITVGuide* getTVGuide() = 0;
    /**
       Set the directory where the recording should be stored
    */

    virtual void setDirectory(const string& dir) = 0;
    /**
       Get the directory where the recordings are stored
    */

    virtual string getDirectory() = 0;

    virtual TVChannelList getChannelConfig() = 0;


  private:
    void registerEvents();
  };

}

#endif
