
#include "ITCPControl.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ITCPControl::enableNoDelay(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("ITCPControl::enableNoDelay", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ITCPControl::isDelayEnabled() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITCPControl::isDelayEnabled");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::setMaxSegment(const int& value)
      throw(const Exception&)
  {
    TInValue<int > value_value(value, false);
    Event event("ITCPControl::setMaxSegment", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int ITCPControl::getMaxSegment() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ITCPControl::getMaxSegment");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::enablePartialFrames(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("ITCPControl::enablePartialFrames", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ITCPControl::isPartialFramesEnabled()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITCPControl::isPartialFramesEnabled");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int ITCPControl::getUnreadQueueCount() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ITCPControl::getUnreadQueueCount");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool ITCPControl::allUrgentDataSend() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITCPControl::allUrgentDataSend");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int ITCPControl::getUnsendQueueCount() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ITCPControl::getUnsendQueueCount");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::enableKeepAlive(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("ITCPControl::enableKeepAlive", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ITCPControl::getKeepAlive() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITCPControl::getKeepAlive");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::setRecvBufferSize(const int& value)
      throw(const Exception&)
  {
    TInValue<int > value_value(value, false);
    Event event("ITCPControl::setRecvBufferSize", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int ITCPControl::getRecvBufferSize() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ITCPControl::getRecvBufferSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::setSendBufferSize(const int& value) /* in-stream */ 
      throw(const Exception&)
  {
    TInValue<int > value_value(value, false);
    Event event("ITCPControl::setSendBufferSize", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int ITCPControl::getSendBufferSize() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ITCPControl::getSendBufferSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::setPriority(const int& value)
      throw(const Exception&)
  {
    TInValue<int > value_value(value, false);
    Event event("ITCPControl::setPriority", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int ITCPControl::getPriority() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ITCPControl::getPriority");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITCPControl::ITCPControl_type = "ITCPControl";
  TIFactoryObject<ITCPControl > __ITCPControl_prototype("ITCPControl");


  const char* ITCPControl::enableNoDelay_event = "ITCPControl::enableNoDelay";
  const char* ITCPControl::isDelayEnabled_event = "ITCPControl::isDelayEnabled";
  const char* ITCPControl::setMaxSegment_event = "ITCPControl::setMaxSegment";
  const char* ITCPControl::getMaxSegment_event = "ITCPControl::getMaxSegment";
  const char* ITCPControl::enablePartialFrames_event = "ITCPControl::enablePartialFrames";
  const char* ITCPControl::isPartialFramesEnabled_event = "ITCPControl::isPartialFramesEnabled";
  const char* ITCPControl::getUnreadQueueCount_event = "ITCPControl::getUnreadQueueCount";
  const char* ITCPControl::allUrgentDataSend_event = "ITCPControl::allUrgentDataSend";
  const char* ITCPControl::getUnsendQueueCount_event = "ITCPControl::getUnsendQueueCount";
  const char* ITCPControl::enableKeepAlive_event = "ITCPControl::enableKeepAlive";
  const char* ITCPControl::getKeepAlive_event = "ITCPControl::getKeepAlive";
  const char* ITCPControl::setRecvBufferSize_event = "ITCPControl::setRecvBufferSize";
  const char* ITCPControl::getRecvBufferSize_event = "ITCPControl::getRecvBufferSize";
  const char* ITCPControl::setSendBufferSize_event = "ITCPControl::setSendBufferSize";
  Event* ITCPControl::create_setSendBufferSize(const int& value)
  {
    TInValue<int >* value_value = new TInValue<int >();
    value_value->setValue(value);

    return new Event("ITCPControl::setSendBufferSize", value_value);
  }

  const char* ITCPControl::getSendBufferSize_event = "ITCPControl::getSendBufferSize";
  const char* ITCPControl::setPriority_event = "ITCPControl::setPriority";
  const char* ITCPControl::getPriority_event = "ITCPControl::getPriority";


}


