
#ifndef NMM_ITCPControl_HPP
#define NMM_ITCPControl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ITCPControl : virtual public Interface { 
  public:
    ITCPControl() {  setTypeInfo(TypeInfo(ITCPControl::ITCPControl_type, "NMM")); };

    ITCPControl(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITCPControl::ITCPControl_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITCPControl() {};

    static const char* ITCPControl_type;

    virtual  string getInterfaceType() const { return ITCPControl_type; }

    Interface* copy() const { return new ITCPControl(*this); }
    static const char* enableNoDelay_event;
    static const char* isDelayEnabled_event;
    static const char* setMaxSegment_event;
    static const char* getMaxSegment_event;
    static const char* enablePartialFrames_event;
    static const char* isPartialFramesEnabled_event;
    static const char* getUnreadQueueCount_event;
    static const char* allUrgentDataSend_event;
    static const char* getUnsendQueueCount_event;
    static const char* enableKeepAlive_event;
    static const char* getKeepAlive_event;
    static const char* setRecvBufferSize_event;
    static const char* getRecvBufferSize_event;
    static const char* setSendBufferSize_event;
    static Event* create_setSendBufferSize(const int& value);
    static const char* getSendBufferSize_event;
    static const char* setPriority_event;
    static const char* getPriority_event;

    virtual void enableNoDelay(const bool& value)
      throw(const Exception&);

    virtual bool isDelayEnabled() const
      throw(const Exception&);

    virtual void setMaxSegment(const int& value)
      throw(const Exception&);

    virtual int getMaxSegment() const
      throw(const Exception&);

    virtual void enablePartialFrames(const bool& value)
      throw(const Exception&);

    virtual bool isPartialFramesEnabled()
      throw(const Exception&);

    virtual int getUnreadQueueCount() const
      throw(const Exception&);

    virtual bool allUrgentDataSend() const
      throw(const Exception&);

    virtual int getUnsendQueueCount() const
      throw(const Exception&);

    virtual void enableKeepAlive(const bool& value)
      throw(const Exception&);

    virtual bool getKeepAlive() const
      throw(const Exception&);

    virtual void setRecvBufferSize(const int& value)
      throw(const Exception&);

    virtual int getRecvBufferSize() const
      throw(const Exception&);
  /* instream (protected) and non-instream (public) */    
    virtual void setSendBufferSize(const int& value) /* in-stream */ 
      throw(const Exception&);
  /* non-instream (public) */    
    virtual int getSendBufferSize() const
      throw(const Exception&);

    virtual void setPriority(const int& value)
      throw(const Exception&);

    virtual int getPriority() const
      throw(const Exception&);


  };

  typedef auto_ptr<ITCPControl> ITCPControl_var;




}

#endif
