
#include "IVNC.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IVNC::setTolerance(const float& tolerance)
      throw(const Exception&)
  {
    TInValue<float > tolerance_value(tolerance, false);
    Event event("IVNC::setTolerance", &tolerance_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVNC::setInterval(const float& interval)
      throw(const Exception&)
  {
    TInValue<float > interval_value(interval, false);
    Event event("IVNC::setInterval", &interval_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVNC::setShared(const bool& unshared)
      throw(const Exception&)
  {
    TInValue<bool > unshared_value(unshared, false);
    Event event("IVNC::setShared", &unshared_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float IVNC::getTolerance() const
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IVNC::getTolerance");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float IVNC::getInterval() const
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("IVNC::getInterval");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IVNC::setPasswd(const string& passwd)
      throw(const Exception&)
  {
    TInValue<string > passwd_value(passwd, false);
    Event event("IVNC::setPasswd", &passwd_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVNC::setPasswdPath(const string& passwdpath)
      throw(const Exception&)
  {
    TInValue<string > passwdpath_value(passwdpath, false);
    Event event("IVNC::setPasswdPath", &passwdpath_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVNC::setMode(const IVNC::Mode& mode)
      throw(const Exception&)
  {
    TInValue<IVNC::Mode > mode_value(mode, false);
    Event event("IVNC::setMode", &mode_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool IVNC::isShared() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IVNC::isShared");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IVNC::sendKeyEvent(const unsigned long& key, const int& down)
      throw(const Exception&)
  {
    TInValue<unsigned long > key_value(key, false);
    TInValue<int > down_value(down, false);
    Event event("IVNC::sendKeyEvent", &key_value, &down_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVNC::sendPointerEvent(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttonmask_value(buttonmask, false);
    Event event("IVNC::sendPointerEvent", &x_value, &y_value, &buttonmask_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVNC::sendFrameBufferUpdateRequest()
      throw(const Exception&)
  {
    Event event("IVNC::sendFrameBufferUpdateRequest");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IVNC::IVNC_type = "IVNC";
  TIFactoryObject<IVNC > __IVNC_prototype("IVNC");
  TValueConstructor< IVNC::Mode > __IVNC_Mode_prototype(TypeInfo("IVNC::Mode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IVNC::Mode& t)
  {
    ov.beginType(TypeInfo("IVNC::Mode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IVNC::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IVNC::Mode", "NMM")) {
      throw InvalidTypeException(TypeInfo("IVNC::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IVNC::setTolerance_event = "IVNC::setTolerance";
  const char* IVNC::setInterval_event = "IVNC::setInterval";
  const char* IVNC::setShared_event = "IVNC::setShared";
  const char* IVNC::getTolerance_event = "IVNC::getTolerance";
  const char* IVNC::getInterval_event = "IVNC::getInterval";
  const char* IVNC::setPasswd_event = "IVNC::setPasswd";
  const char* IVNC::setPasswdPath_event = "IVNC::setPasswdPath";
  const char* IVNC::setMode_event = "IVNC::setMode";
  const char* IVNC::isShared_event = "IVNC::isShared";
  const char* IVNC::sendKeyEvent_event = "IVNC::sendKeyEvent";
  const char* IVNC::sendPointerEvent_event = "IVNC::sendPointerEvent";
  const char* IVNC::sendFrameBufferUpdateRequest_event = "IVNC::sendFrameBufferUpdateRequest";


}


