
#ifndef NMM_IByteCounterImpl_HPP
#define NMM_IByteCounterImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IByteCounter.hpp"


namespace NMM
{
  /**
   * Interface for counting the bytes send/received by a transport strategy.
   */

  class IByteCounterImpl  : public virtual NMMObject { 
  public:
    IByteCounterImpl() { addInterface(new IByteCounter(*this)); registerEvents(); };

    virtual ~IByteCounterImpl() {};

  protected:
    /**
     * Get number of bytes send.
     * Returns the current value of the sending byte counter.
     */

    virtual int getSendingCounter() = 0;
    /**
     * Get number of bytes received.
     * Returns the current value of the receiving byte counter.
     */

    virtual int getReceivingCounter() = 0;
    /**
     * Reset the sending byte counter.
     * Resets the sending counter to zero bytes processed.
     */

    virtual void resetSendingCounter() = 0;
    /**
     * Reset the receiving byte counter.
     * Resets the sending counter to zero bytes processed.
     */

    virtual void resetReceivingCounter() = 0;


  private:
    void registerEvents();
  };

}

#endif
