
#include "IDisplayEvents.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IDisplayEvents::keyPressed(const string& key) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > key_value(key, false);
    Event event("IDisplayEvents::keyPressed", &key_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::keyReleased(const string& key) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > key_value(key, false);
    Event event("IDisplayEvents::keyReleased", &key_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::keysym_keyPressed(const unsigned long& keysym) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned long > keysym_value(keysym, false);
    Event event("IDisplayEvents::keysym_keyPressed", &keysym_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::keysym_keyReleased(const unsigned long& keysym) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned long > keysym_value(keysym, false);
    Event event("IDisplayEvents::keysym_keyReleased", &keysym_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::mouseButtonPressed(const int& button, const int& x, const int& y, const int& buttons, const bool& shift) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > button_value(button, false);
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttons_value(buttons, false);
    TInValue<bool > shift_value(shift, false);
    Event event("IDisplayEvents::mouseButtonPressed", &button_value, &x_value, &y_value, &buttons_value, &shift_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::mouseButtonReleased(const int& button, const int& x, const int& y, const int& buttons, const bool& shift) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > button_value(button, false);
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttons_value(buttons, false);
    TInValue<bool > shift_value(shift, false);
    Event event("IDisplayEvents::mouseButtonReleased", &button_value, &x_value, &y_value, &buttons_value, &shift_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::mouseMoved(const int& x, const int& y, const int& buttons, const bool& shift) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttons_value(buttons, false);
    TInValue<bool > shift_value(shift, false);
    Event event("IDisplayEvents::mouseMoved", &x_value, &y_value, &buttons_value, &shift_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::windowExposed() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IDisplayEvents::windowExposed");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::windowClosed() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IDisplayEvents::windowClosed");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::windowResized(const int& xsize, const int& ysize) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > xsize_value(xsize, false);
    TInValue<int > ysize_value(ysize, false);
    Event event("IDisplayEvents::windowResized", &xsize_value, &ysize_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IDisplayEvents::IDisplayEvents_type = "IDisplayEvents";
  TIFactoryObject<IDisplayEvents > __IDisplayEvents_prototype("IDisplayEvents");
  TValueConstructor< IDisplayEvents::MouseButtons > __IDisplayEvents_MouseButtons_prototype(TypeInfo("IDisplayEvents::MouseButtons", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IDisplayEvents::MouseButtons& t)
  {
    ov.beginType(TypeInfo("IDisplayEvents::MouseButtons", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IDisplayEvents::MouseButtons& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IDisplayEvents::MouseButtons", "NMM")) {
      throw InvalidTypeException(TypeInfo("IDisplayEvents::MouseButtons", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IDisplayEvents::keyPressed_event = "IDisplayEvents::keyPressed";
  Event* IDisplayEvents::create_keyPressed(const string& key)
  {
    TInValue<string >* key_value = new TInValue<string >();
    key_value->setValue(key);

    return new Event("IDisplayEvents::keyPressed", key_value);
  }

  const char* IDisplayEvents::keyReleased_event = "IDisplayEvents::keyReleased";
  Event* IDisplayEvents::create_keyReleased(const string& key)
  {
    TInValue<string >* key_value = new TInValue<string >();
    key_value->setValue(key);

    return new Event("IDisplayEvents::keyReleased", key_value);
  }

  const char* IDisplayEvents::keysym_keyPressed_event = "IDisplayEvents::keysym_keyPressed";
  Event* IDisplayEvents::create_keysym_keyPressed(const unsigned long& keysym)
  {
    TInValue<unsigned long >* keysym_value = new TInValue<unsigned long >();
    keysym_value->setValue(keysym);

    return new Event("IDisplayEvents::keysym_keyPressed", keysym_value);
  }

  const char* IDisplayEvents::keysym_keyReleased_event = "IDisplayEvents::keysym_keyReleased";
  Event* IDisplayEvents::create_keysym_keyReleased(const unsigned long& keysym)
  {
    TInValue<unsigned long >* keysym_value = new TInValue<unsigned long >();
    keysym_value->setValue(keysym);

    return new Event("IDisplayEvents::keysym_keyReleased", keysym_value);
  }

  const char* IDisplayEvents::mouseButtonPressed_event = "IDisplayEvents::mouseButtonPressed";
  Event* IDisplayEvents::create_mouseButtonPressed(const int& button, const int& x, const int& y, const int& buttons, const bool& shift)
  {
    TInValue<int >* button_value = new TInValue<int >();
    button_value->setValue(button);
    TInValue<int >* x_value = new TInValue<int >();
    x_value->setValue(x);
    TInValue<int >* y_value = new TInValue<int >();
    y_value->setValue(y);
    TInValue<int >* buttons_value = new TInValue<int >();
    buttons_value->setValue(buttons);
    TInValue<bool >* shift_value = new TInValue<bool >();
    shift_value->setValue(shift);

    return new Event("IDisplayEvents::mouseButtonPressed", button_value, x_value, y_value, buttons_value, shift_value);
  }

  const char* IDisplayEvents::mouseButtonReleased_event = "IDisplayEvents::mouseButtonReleased";
  Event* IDisplayEvents::create_mouseButtonReleased(const int& button, const int& x, const int& y, const int& buttons, const bool& shift)
  {
    TInValue<int >* button_value = new TInValue<int >();
    button_value->setValue(button);
    TInValue<int >* x_value = new TInValue<int >();
    x_value->setValue(x);
    TInValue<int >* y_value = new TInValue<int >();
    y_value->setValue(y);
    TInValue<int >* buttons_value = new TInValue<int >();
    buttons_value->setValue(buttons);
    TInValue<bool >* shift_value = new TInValue<bool >();
    shift_value->setValue(shift);

    return new Event("IDisplayEvents::mouseButtonReleased", button_value, x_value, y_value, buttons_value, shift_value);
  }

  const char* IDisplayEvents::mouseMoved_event = "IDisplayEvents::mouseMoved";
  Event* IDisplayEvents::create_mouseMoved(const int& x, const int& y, const int& buttons, const bool& shift)
  {
    TInValue<int >* x_value = new TInValue<int >();
    x_value->setValue(x);
    TInValue<int >* y_value = new TInValue<int >();
    y_value->setValue(y);
    TInValue<int >* buttons_value = new TInValue<int >();
    buttons_value->setValue(buttons);
    TInValue<bool >* shift_value = new TInValue<bool >();
    shift_value->setValue(shift);

    return new Event("IDisplayEvents::mouseMoved", x_value, y_value, buttons_value, shift_value);
  }

  const char* IDisplayEvents::windowExposed_event = "IDisplayEvents::windowExposed";
  Event* IDisplayEvents::create_windowExposed()
  {

    return new Event("IDisplayEvents::windowExposed");
  }

  const char* IDisplayEvents::windowClosed_event = "IDisplayEvents::windowClosed";
  Event* IDisplayEvents::create_windowClosed()
  {

    return new Event("IDisplayEvents::windowClosed");
  }

  const char* IDisplayEvents::windowResized_event = "IDisplayEvents::windowResized";
  Event* IDisplayEvents::create_windowResized(const int& xsize, const int& ysize)
  {
    TInValue<int >* xsize_value = new TInValue<int >();
    xsize_value->setValue(xsize);
    TInValue<int >* ysize_value = new TInValue<int >();
    ysize_value->setValue(ysize);

    return new Event("IDisplayEvents::windowResized", xsize_value, ysize_value);
  }



}


