
#ifndef NMM_IMouseCursor_HPP
#define NMM_IMouseCursor_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Interface for setting mouse cursor properties.
   */

  class IMouseCursor : virtual public Interface { 
  public:
    IMouseCursor() {  setTypeInfo(TypeInfo(IMouseCursor::IMouseCursor_type, "NMM")); };

    IMouseCursor(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IMouseCursor::IMouseCursor_type, "NMM")); setEventReceiver(parent); };

    virtual ~IMouseCursor() {};

    static const char* IMouseCursor_type;

    virtual  string getInterfaceType() const { return IMouseCursor_type; }

    Interface* copy() const { return new IMouseCursor(*this); }
    static const char* useDotCursor_event;
    static const char* useStandardCursor_event;
    static const char* useTriangleCursor_event;
    /**
     * Set dot mouse cursor in window.
     * @param value true for dot mouse cursor, false for standard mouse cursor
     */

    virtual void useDotCursor(const bool& value)
      throw(const Exception&);
    /**
     * Use standard X mouse cursor in window.
     */

    virtual void useStandardCursor()
      throw(const Exception&);
    /**
     * Set cursor to pixmap specified in src with mask msk.
     * @param src the cursor is set to this pixmap values
     * @param msk the mask for the cursor
     */

    virtual void useTriangleCursor(const bool& value)
      throw(const Exception&);


  };

  typedef auto_ptr<IMouseCursor> IMouseCursor_var;




}

#endif
