
#ifndef NMM_IVideoDisplay_HPP
#define NMM_IVideoDisplay_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{
  /**
   * General display interface.
   * This interface provides all general display methods.
   */

  class IVideoDisplay : virtual public Interface { 
  public:
    IVideoDisplay() {  setTypeInfo(TypeInfo(IVideoDisplay::IVideoDisplay_type, "NMM")); };

    IVideoDisplay(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IVideoDisplay::IVideoDisplay_type, "NMM")); setEventReceiver(parent); };

    virtual ~IVideoDisplay() {};

    static const char* IVideoDisplay_type;

    virtual  string getInterfaceType() const { return IVideoDisplay_type; }

    Interface* copy() const { return new IVideoDisplay(*this); }
    static const char* setFullscreen_event;
    static const char* getFullscreen_event;
    static const char* hasFullscreen_event;
    static const char* isFullscreenOnly_event;
    static const char* makeScreenshot_event;
    static const char* setLeftWindowAlignment_event;
    static const char* setRightWindowAlignment_event;
    static const char* setCenterWindowAlignment_event;
    /**
     * Set fullscreen mode.
     * @param value true for fullscreen, false for windowed display mode
     */

    virtual void setFullscreen(const bool& value)
      throw(const Exception&);
    /**
     * Get fullscreen mode.
     * @return true if the display is in fullscreen mode, false if it is in windowed mode
     */

    virtual bool getFullscreen() const
      throw(const Exception&);
    /**
     * test if display has fullscreen support.
     * @return true if fullscreen mode can be turned on, false otherwise
     */

    virtual bool hasFullscreen() const
      throw(const Exception&);
    /**
     * Test if display is fullscreen only.
     * @return true if fullscren mode is the default and cannot be turned off.
     */

    virtual bool isFullscreenOnly() const
      throw(const Exception&);

    virtual bool makeScreenshot(const string& filename) const
      throw(const Exception&);
    /**
     * Set window alignment.
     */

    virtual void setLeftWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);

    virtual void setRightWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);

    virtual void setCenterWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);


  };

  typedef auto_ptr<IVideoDisplay> IVideoDisplay_var;




}

#endif
