
#include "IMagickManipulationNode.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IMagickManipulationNode::setBorderColor(const double& r, const double& g, const double& b)
      throw(const Exception&)
  {
    TInValue<double > r_value(r, false);
    TInValue<double > g_value(g, false);
    TInValue<double > b_value(b, false);
    Event event("IMagickManipulationNode::setBorderColor", &r_value, &g_value, &b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::clearManipulators()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::clearManipulators");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::adaptiveThreshold(const unsigned int& width, const unsigned int& height, const unsigned int& offset /* = 0 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > width_value(width, false);
    TInValue<unsigned int > height_value(height, false);
    TInValue<unsigned int > offset_value(offset, false);
    Event event("IMagickManipulationNode::adaptiveThreshold", &width_value, &height_value, &offset_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::addNoise(const string& noise_type)
      throw(const Exception&)
  {
    TInValue<string > noise_type_value(noise_type, false);
    Event event("IMagickManipulationNode::addNoise", &noise_type_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::annotate(const string& text, const string& bounding_area, const string& gravity, const double& degrees)
      throw(const Exception&)
  {
    TInValue<string > text_value(text, false);
    TInValue<string > bounding_area_value(bounding_area, false);
    TInValue<string > gravity_value(gravity, false);
    TInValue<double > degrees_value(degrees, false);
    Event event("IMagickManipulationNode::annotate", &text_value, &bounding_area_value, &gravity_value, &degrees_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::blur(const double& radius /* = 1 */, const double& sigma /* = 0.5 */)
      throw(const Exception&)
  {
    TInValue<double > radius_value(radius, false);
    TInValue<double > sigma_value(sigma, false);
    Event event("IMagickManipulationNode::blur", &radius_value, &sigma_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::border(const string& geometry /* = "6x6+0+0" */)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::border", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::channel(const string& layer)
      throw(const Exception&)
  {
    TInValue<string > layer_value(layer, false);
    Event event("IMagickManipulationNode::channel", &layer_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::charcoal(const double& radius /* = 1 */, const double& sigma /* = 0.5 */)
      throw(const Exception&)
  {
    TInValue<double > radius_value(radius, false);
    TInValue<double > sigma_value(sigma, false);
    Event event("IMagickManipulationNode::charcoal", &radius_value, &sigma_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::chop(const string& geometry)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::chop", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::colorize(const unsigned int& opacity_red, const unsigned int& opacity_green, const unsigned int& opacity_blue, const double& pen_color_r, const double& pen_color_g, const double& pen_color_b)
      throw(const Exception&)
  {
    TInValue<unsigned int > opacity_red_value(opacity_red, false);
    TInValue<unsigned int > opacity_green_value(opacity_green, false);
    TInValue<unsigned int > opacity_blue_value(opacity_blue, false);
    TInValue<double > pen_color_r_value(pen_color_r, false);
    TInValue<double > pen_color_g_value(pen_color_g, false);
    TInValue<double > pen_color_b_value(pen_color_b, false);
    Event event("IMagickManipulationNode::colorize", &opacity_red_value, &opacity_green_value, &opacity_blue_value, &pen_color_r_value, &pen_color_g_value, &pen_color_b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::comment(const string& comment)
      throw(const Exception&)
  {
    TInValue<string > comment_value(comment, false);
    Event event("IMagickManipulationNode::comment", &comment_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::contrast(const unsigned int& sharpen)
      throw(const Exception&)
  {
    TInValue<unsigned int > sharpen_value(sharpen, false);
    Event event("IMagickManipulationNode::contrast", &sharpen_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::crop(const string& geometry)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::crop", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::cycleColormap(const int& amount)
      throw(const Exception&)
  {
    TInValue<int > amount_value(amount, false);
    Event event("IMagickManipulationNode::cycleColormap", &amount_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::despeckle()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::despeckle");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::edge(const unsigned int& radius /* = 0.0 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > radius_value(radius, false);
    Event event("IMagickManipulationNode::edge", &radius_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::emboss(const double& radius /* = 1 */, const double& sigma /* = 0.5 */)
      throw(const Exception&)
  {
    TInValue<double > radius_value(radius, false);
    TInValue<double > sigma_value(sigma, false);
    Event event("IMagickManipulationNode::emboss", &radius_value, &sigma_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::enhance()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::enhance");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::equalize()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::equalize");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::erase()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::erase");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::flip()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::flip");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::floodFillColor(const unsigned int& x, const unsigned int& y, const double& fill_color_r, const double& fill_color_g, const double& fill_color_b)
      throw(const Exception&)
  {
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    TInValue<double > fill_color_r_value(fill_color_r, false);
    TInValue<double > fill_color_g_value(fill_color_g, false);
    TInValue<double > fill_color_b_value(fill_color_b, false);
    Event event("IMagickManipulationNode::floodFillColor", &x_value, &y_value, &fill_color_r_value, &fill_color_g_value, &fill_color_b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::floodFillOpacity(const long& x, const long& y, const unsigned int& opacity, const string& method)
      throw(const Exception&)
  {
    TInValue<long > x_value(x, false);
    TInValue<long > y_value(y, false);
    TInValue<unsigned int > opacity_value(opacity, false);
    TInValue<string > method_value(method, false);
    Event event("IMagickManipulationNode::floodFillOpacity", &x_value, &y_value, &opacity_value, &method_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::flop()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::flop");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::frame(const unsigned int& width, const unsigned int& height, const int& x, const int& y, const int& inner_bevel /* = 0 */, const int& outer_bevel /* = 0 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > width_value(width, false);
    TInValue<unsigned int > height_value(height, false);
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > inner_bevel_value(inner_bevel, false);
    TInValue<int > outer_bevel_value(outer_bevel, false);
    Event event("IMagickManipulationNode::frame", &width_value, &height_value, &x_value, &y_value, &inner_bevel_value, &outer_bevel_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::gamma(const double& gamma_red, const double& gamma_green, const double& gamma_blue)
      throw(const Exception&)
  {
    TInValue<double > gamma_red_value(gamma_red, false);
    TInValue<double > gamma_green_value(gamma_green, false);
    TInValue<double > gamma_blue_value(gamma_blue, false);
    Event event("IMagickManipulationNode::gamma", &gamma_red_value, &gamma_green_value, &gamma_blue_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::gaussianBlur(const double& width, const double& sigma)
      throw(const Exception&)
  {
    TInValue<double > width_value(width, false);
    TInValue<double > sigma_value(sigma, false);
    Event event("IMagickManipulationNode::gaussianBlur", &width_value, &sigma_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::implode(const double& factor)
      throw(const Exception&)
  {
    TInValue<double > factor_value(factor, false);
    Event event("IMagickManipulationNode::implode", &factor_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::label(const string& name)
      throw(const Exception&)
  {
    TInValue<string > name_value(name, false);
    Event event("IMagickManipulationNode::label", &name_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::magnify()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::magnify");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::matteFloodfill(const double& target_r, const double& target_g, const double& target_b, const unsigned int& opacity, const int& x, const int& y, const string& method)
      throw(const Exception&)
  {
    TInValue<double > target_r_value(target_r, false);
    TInValue<double > target_g_value(target_g, false);
    TInValue<double > target_b_value(target_b, false);
    TInValue<unsigned int > opacity_value(opacity, false);
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<string > method_value(method, false);
    Event event("IMagickManipulationNode::matteFloodfill", &target_r_value, &target_g_value, &target_b_value, &opacity_value, &x_value, &y_value, &method_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::medianFilter(const double& radius /* = 0.0 */)
      throw(const Exception&)
  {
    TInValue<double > radius_value(radius, false);
    Event event("IMagickManipulationNode::medianFilter", &radius_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::minify()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::minify");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::modulate(const double& brightness, const double& saturation, const double& hue)
      throw(const Exception&)
  {
    TInValue<double > brightness_value(brightness, false);
    TInValue<double > saturation_value(saturation, false);
    TInValue<double > hue_value(hue, false);
    Event event("IMagickManipulationNode::modulate", &brightness_value, &saturation_value, &hue_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::negate()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::negate");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::normalize()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::normalize");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::oilPaint(const unsigned int& radius /* = 3 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > radius_value(radius, false);
    Event event("IMagickManipulationNode::oilPaint", &radius_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::opacity(const unsigned int& opacity)
      throw(const Exception&)
  {
    TInValue<unsigned int > opacity_value(opacity, false);
    Event event("IMagickManipulationNode::opacity", &opacity_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::opaque(const double& opaque_color_r, const double& opaque_color_g, const double& opaque_color_b, const double& pen_color_r, const double& pen_color_g, const double& pen_color_b)
      throw(const Exception&)
  {
    TInValue<double > opaque_color_r_value(opaque_color_r, false);
    TInValue<double > opaque_color_g_value(opaque_color_g, false);
    TInValue<double > opaque_color_b_value(opaque_color_b, false);
    TInValue<double > pen_color_r_value(pen_color_r, false);
    TInValue<double > pen_color_g_value(pen_color_g, false);
    TInValue<double > pen_color_b_value(pen_color_b, false);
    Event event("IMagickManipulationNode::opaque", &opaque_color_r_value, &opaque_color_g_value, &opaque_color_b_value, &pen_color_r_value, &pen_color_g_value, &pen_color_b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::quantize(const bool& measureError /* = false */)
      throw(const Exception&)
  {
    TInValue<bool > measureError_value(measureError, false);
    Event event("IMagickManipulationNode::quantize", &measureError_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::raise(const string& geometry, const bool& raisedFlag /* = false */)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    TInValue<bool > raisedFlag_value(raisedFlag, false);
    Event event("IMagickManipulationNode::raise", &geometry_value, &raisedFlag_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::reduceNoise(const unsigned int& order)
      throw(const Exception&)
  {
    TInValue<unsigned int > order_value(order, false);
    Event event("IMagickManipulationNode::reduceNoise", &order_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::roll(const int& columns, const int& rows)
      throw(const Exception&)
  {
    TInValue<int > columns_value(columns, false);
    TInValue<int > rows_value(rows, false);
    Event event("IMagickManipulationNode::roll", &columns_value, &rows_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::rotate(const double& degrees)
      throw(const Exception&)
  {
    TInValue<double > degrees_value(degrees, false);
    Event event("IMagickManipulationNode::rotate", &degrees_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::sample(const string& geometry)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::sample", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::scale(const string& geometry)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::scale", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::segment(const double& cluster_threshold /* = 1.0 */, const double& smoothing_threshold /* = 1.5 */)
      throw(const Exception&)
  {
    TInValue<double > cluster_threshold_value(cluster_threshold, false);
    TInValue<double > smoothing_threshold_value(smoothing_threshold, false);
    Event event("IMagickManipulationNode::segment", &cluster_threshold_value, &smoothing_threshold_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::shade(const double& azimuth /* = 30 */, const double& elevation /* = 30 */, const bool& color_shading /* = false */)
      throw(const Exception&)
  {
    TInValue<double > azimuth_value(azimuth, false);
    TInValue<double > elevation_value(elevation, false);
    TInValue<bool > color_shading_value(color_shading, false);
    Event event("IMagickManipulationNode::shade", &azimuth_value, &elevation_value, &color_shading_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::sharpen(const double& radius /* = 1 */, const double& sigma /* = 0.5 */)
      throw(const Exception&)
  {
    TInValue<double > radius_value(radius, false);
    TInValue<double > sigma_value(sigma, false);
    Event event("IMagickManipulationNode::sharpen", &radius_value, &sigma_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::shave(const string& geometry)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::shave", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::shear(const double& x_shear_angle, const double& y_shear_angle)
      throw(const Exception&)
  {
    TInValue<double > x_shear_angle_value(x_shear_angle, false);
    TInValue<double > y_shear_angle_value(y_shear_angle, false);
    Event event("IMagickManipulationNode::shear", &x_shear_angle_value, &y_shear_angle_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::solarize(const double& factor /* = 50.0 */)
      throw(const Exception&)
  {
    TInValue<double > factor_value(factor, false);
    Event event("IMagickManipulationNode::solarize", &factor_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::spread(const unsigned int& amount /* = 3 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > amount_value(amount, false);
    Event event("IMagickManipulationNode::spread", &amount_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::swirl(const double& degrees)
      throw(const Exception&)
  {
    TInValue<double > degrees_value(degrees, false);
    Event event("IMagickManipulationNode::swirl", &degrees_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::threshold(const double& threshold)
      throw(const Exception&)
  {
    TInValue<double > threshold_value(threshold, false);
    Event event("IMagickManipulationNode::threshold", &threshold_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::transform(const string& image_geometry, const string& crop_geometry)
      throw(const Exception&)
  {
    TInValue<string > image_geometry_value(image_geometry, false);
    TInValue<string > crop_geometry_value(crop_geometry, false);
    Event event("IMagickManipulationNode::transform", &image_geometry_value, &crop_geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::transparent(const double& color_r, const double& color_g, const double& color_b)
      throw(const Exception&)
  {
    TInValue<double > color_r_value(color_r, false);
    TInValue<double > color_g_value(color_g, false);
    TInValue<double > color_b_value(color_b, false);
    Event event("IMagickManipulationNode::transparent", &color_r_value, &color_g_value, &color_b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::trim()
      throw(const Exception&)
  {
    Event event("IMagickManipulationNode::trim");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::unsharpmask(const double& radius, const double& sigma, const double& amount, const double& threshold)
      throw(const Exception&)
  {
    TInValue<double > radius_value(radius, false);
    TInValue<double > sigma_value(sigma, false);
    TInValue<double > amount_value(amount, false);
    TInValue<double > threshold_value(threshold, false);
    Event event("IMagickManipulationNode::unsharpmask", &radius_value, &sigma_value, &amount_value, &threshold_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::wave(const double& amplitude /* = 25.0 */, const double& wavelength /* = 150.0 */)
      throw(const Exception&)
  {
    TInValue<double > amplitude_value(amplitude, false);
    TInValue<double > wavelength_value(wavelength, false);
    Event event("IMagickManipulationNode::wave", &amplitude_value, &wavelength_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::zoom(const string& geometry)
      throw(const Exception&)
  {
    TInValue<string > geometry_value(geometry, false);
    Event event("IMagickManipulationNode::zoom", &geometry_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IMagickManipulationNode::IMagickManipulationNode_type = "IMagickManipulationNode";
  TIFactoryObject<IMagickManipulationNode > __IMagickManipulationNode_prototype("IMagickManipulationNode");


  const char* IMagickManipulationNode::setBorderColor_event = "IMagickManipulationNode::setBorderColor";
  const char* IMagickManipulationNode::clearManipulators_event = "IMagickManipulationNode::clearManipulators";
  const char* IMagickManipulationNode::adaptiveThreshold_event = "IMagickManipulationNode::adaptiveThreshold";
  const char* IMagickManipulationNode::addNoise_event = "IMagickManipulationNode::addNoise";
  const char* IMagickManipulationNode::annotate_event = "IMagickManipulationNode::annotate";
  const char* IMagickManipulationNode::blur_event = "IMagickManipulationNode::blur";
  const char* IMagickManipulationNode::border_event = "IMagickManipulationNode::border";
  const char* IMagickManipulationNode::channel_event = "IMagickManipulationNode::channel";
  const char* IMagickManipulationNode::charcoal_event = "IMagickManipulationNode::charcoal";
  const char* IMagickManipulationNode::chop_event = "IMagickManipulationNode::chop";
  const char* IMagickManipulationNode::colorize_event = "IMagickManipulationNode::colorize";
  const char* IMagickManipulationNode::comment_event = "IMagickManipulationNode::comment";
  const char* IMagickManipulationNode::contrast_event = "IMagickManipulationNode::contrast";
  const char* IMagickManipulationNode::crop_event = "IMagickManipulationNode::crop";
  const char* IMagickManipulationNode::cycleColormap_event = "IMagickManipulationNode::cycleColormap";
  const char* IMagickManipulationNode::despeckle_event = "IMagickManipulationNode::despeckle";
  const char* IMagickManipulationNode::edge_event = "IMagickManipulationNode::edge";
  const char* IMagickManipulationNode::emboss_event = "IMagickManipulationNode::emboss";
  const char* IMagickManipulationNode::enhance_event = "IMagickManipulationNode::enhance";
  const char* IMagickManipulationNode::equalize_event = "IMagickManipulationNode::equalize";
  const char* IMagickManipulationNode::erase_event = "IMagickManipulationNode::erase";
  const char* IMagickManipulationNode::flip_event = "IMagickManipulationNode::flip";
  const char* IMagickManipulationNode::floodFillColor_event = "IMagickManipulationNode::floodFillColor";
  const char* IMagickManipulationNode::floodFillOpacity_event = "IMagickManipulationNode::floodFillOpacity";
  const char* IMagickManipulationNode::flop_event = "IMagickManipulationNode::flop";
  const char* IMagickManipulationNode::frame_event = "IMagickManipulationNode::frame";
  const char* IMagickManipulationNode::gamma_event = "IMagickManipulationNode::gamma";
  const char* IMagickManipulationNode::gaussianBlur_event = "IMagickManipulationNode::gaussianBlur";
  const char* IMagickManipulationNode::implode_event = "IMagickManipulationNode::implode";
  const char* IMagickManipulationNode::label_event = "IMagickManipulationNode::label";
  const char* IMagickManipulationNode::magnify_event = "IMagickManipulationNode::magnify";
  const char* IMagickManipulationNode::matteFloodfill_event = "IMagickManipulationNode::matteFloodfill";
  const char* IMagickManipulationNode::medianFilter_event = "IMagickManipulationNode::medianFilter";
  const char* IMagickManipulationNode::minify_event = "IMagickManipulationNode::minify";
  const char* IMagickManipulationNode::modulate_event = "IMagickManipulationNode::modulate";
  const char* IMagickManipulationNode::negate_event = "IMagickManipulationNode::negate";
  const char* IMagickManipulationNode::normalize_event = "IMagickManipulationNode::normalize";
  const char* IMagickManipulationNode::oilPaint_event = "IMagickManipulationNode::oilPaint";
  const char* IMagickManipulationNode::opacity_event = "IMagickManipulationNode::opacity";
  const char* IMagickManipulationNode::opaque_event = "IMagickManipulationNode::opaque";
  const char* IMagickManipulationNode::quantize_event = "IMagickManipulationNode::quantize";
  const char* IMagickManipulationNode::raise_event = "IMagickManipulationNode::raise";
  const char* IMagickManipulationNode::reduceNoise_event = "IMagickManipulationNode::reduceNoise";
  const char* IMagickManipulationNode::roll_event = "IMagickManipulationNode::roll";
  const char* IMagickManipulationNode::rotate_event = "IMagickManipulationNode::rotate";
  const char* IMagickManipulationNode::sample_event = "IMagickManipulationNode::sample";
  const char* IMagickManipulationNode::scale_event = "IMagickManipulationNode::scale";
  const char* IMagickManipulationNode::segment_event = "IMagickManipulationNode::segment";
  const char* IMagickManipulationNode::shade_event = "IMagickManipulationNode::shade";
  const char* IMagickManipulationNode::sharpen_event = "IMagickManipulationNode::sharpen";
  const char* IMagickManipulationNode::shave_event = "IMagickManipulationNode::shave";
  const char* IMagickManipulationNode::shear_event = "IMagickManipulationNode::shear";
  const char* IMagickManipulationNode::solarize_event = "IMagickManipulationNode::solarize";
  const char* IMagickManipulationNode::spread_event = "IMagickManipulationNode::spread";
  const char* IMagickManipulationNode::swirl_event = "IMagickManipulationNode::swirl";
  const char* IMagickManipulationNode::threshold_event = "IMagickManipulationNode::threshold";
  const char* IMagickManipulationNode::transform_event = "IMagickManipulationNode::transform";
  const char* IMagickManipulationNode::transparent_event = "IMagickManipulationNode::transparent";
  const char* IMagickManipulationNode::trim_event = "IMagickManipulationNode::trim";
  const char* IMagickManipulationNode::unsharpmask_event = "IMagickManipulationNode::unsharpmask";
  const char* IMagickManipulationNode::wave_event = "IMagickManipulationNode::wave";
  const char* IMagickManipulationNode::zoom_event = "IMagickManipulationNode::zoom";


}


