
#ifndef NMM_IMagickManipulationNodeImpl_HPP
#define NMM_IMagickManipulationNodeImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IMagickManipulationNode.hpp"


namespace NMM
{

  class IMagickManipulationNodeImpl  : public virtual NMMObject { 
  public:
    IMagickManipulationNodeImpl() { addInterface(new IMagickManipulationNode(*this)); registerEvents(); };

    virtual ~IMagickManipulationNodeImpl() {};

  protected:

    virtual void setBorderColor(const double& r, const double& g, const double& b) = 0;

    virtual void clearManipulators() = 0;

    virtual void adaptiveThreshold(const unsigned int& width, const unsigned int& height, const unsigned int& offset = 0) = 0;

    virtual void addNoise(const string& noise_type) = 0;

    virtual void annotate(const string& text, const string& bounding_area, const string& gravity, const double& degrees) = 0;

    virtual void blur(const double& radius = 1, const double& sigma = 0.5) = 0;

    virtual void border(const string& geometry = "6x6+0+0") = 0;

    virtual void channel(const string& layer) = 0;

    virtual void charcoal(const double& radius = 1, const double& sigma = 0.5) = 0;

    virtual void chop(const string& geometry) = 0;

    virtual void colorize(const unsigned int& opacity_red, const unsigned int& opacity_green, const unsigned int& opacity_blue, const double& pen_color_r, const double& pen_color_g, const double& pen_color_b) = 0;

    virtual void comment(const string& comment) = 0;

    virtual void contrast(const unsigned int& sharpen) = 0;

    virtual void crop(const string& geometry) = 0;

    virtual void cycleColormap(const int& amount) = 0;

    virtual void despeckle() = 0;

    virtual void edge(const unsigned int& radius = 0.0) = 0;

    virtual void emboss(const double& radius = 1, const double& sigma = 0.5) = 0;

    virtual void enhance() = 0;

    virtual void equalize() = 0;

    virtual void erase() = 0;

    virtual void flip() = 0;

    virtual void floodFillColor(const unsigned int& x, const unsigned int& y, const double& fill_color_r, const double& fill_color_g, const double& fill_color_b) = 0;

    virtual void floodFillOpacity(const long& x, const long& y, const unsigned int& opacity, const string& method) = 0;

    virtual void flop() = 0;

    virtual void frame(const unsigned int& width, const unsigned int& height, const int& x, const int& y, const int& inner_bevel = 0, const int& outer_bevel = 0) = 0;

    virtual void gamma(const double& gamma_red, const double& gamma_green, const double& gamma_blue) = 0;

    virtual void gaussianBlur(const double& width, const double& sigma) = 0;

    virtual void implode(const double& factor) = 0;

    virtual void label(const string& name) = 0;

    virtual void magnify() = 0;

    virtual void matteFloodfill(const double& target_r, const double& target_g, const double& target_b, const unsigned int& opacity, const int& x, const int& y, const string& method) = 0;

    virtual void medianFilter(const double& radius = 0.0) = 0;

    virtual void minify() = 0;

    virtual void modulate(const double& brightness, const double& saturation, const double& hue) = 0;

    virtual void negate() = 0;

    virtual void normalize() = 0;

    virtual void oilPaint(const unsigned int& radius = 3) = 0;

    virtual void opacity(const unsigned int& opacity) = 0;

    virtual void opaque(const double& opaque_color_r, const double& opaque_color_g, const double& opaque_color_b, const double& pen_color_r, const double& pen_color_g, const double& pen_color_b) = 0;

    virtual void quantize(const bool& measureError = false) = 0;

    virtual void raise(const string& geometry, const bool& raisedFlag = false) = 0;

    virtual void reduceNoise(const unsigned int& order) = 0;

    virtual void roll(const int& columns, const int& rows) = 0;

    virtual void rotate(const double& degrees) = 0;

    virtual void sample(const string& geometry) = 0;

    virtual void scale(const string& geometry) = 0;

    virtual void segment(const double& cluster_threshold = 1.0, const double& smoothing_threshold = 1.5) = 0;

    virtual void shade(const double& azimuth = 30, const double& elevation = 30, const bool& color_shading = false) = 0;

    virtual void sharpen(const double& radius = 1, const double& sigma = 0.5) = 0;

    virtual void shave(const string& geometry) = 0;

    virtual void shear(const double& x_shear_angle, const double& y_shear_angle) = 0;

    virtual void solarize(const double& factor = 50.0) = 0;

    virtual void spread(const unsigned int& amount = 3) = 0;

    virtual void swirl(const double& degrees) = 0;

    virtual void threshold(const double& threshold) = 0;

    virtual void transform(const string& image_geometry, const string& crop_geometry) = 0;

    virtual void transparent(const double& color_r, const double& color_g, const double& color_b) = 0;

    virtual void trim() = 0;

    virtual void unsharpmask(const double& radius, const double& sigma, const double& amount, const double& threshold) = 0;

    virtual void wave(const double& amplitude = 25.0, const double& wavelength = 150.0) = 0;

    virtual void zoom(const string& geometry) = 0;


  private:
    void registerEvents();
  };

}

#endif
