
#include "IOSDOverlay.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IOSDOverlay::addText(const string& id, const string& text, const size_t& xoffset, const size_t& yoffset)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > text_value(text, false);
    TInValue<size_t > xoffset_value(xoffset, false);
    TInValue<size_t > yoffset_value(yoffset, false);
    Event event("IOSDOverlay::addText", &id_value, &text_value, &xoffset_value, &yoffset_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::printText(const string& id, const string& text, const size_t& xoffset, const size_t& yoffset, const int& seconds)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > text_value(text, false);
    TInValue<size_t > xoffset_value(xoffset, false);
    TInValue<size_t > yoffset_value(yoffset, false);
    TInValue<int > seconds_value(seconds, false);
    Event event("IOSDOverlay::printText", &id_value, &text_value, &xoffset_value, &yoffset_value, &seconds_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::removeText(const string& id)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    Event event("IOSDOverlay::removeText", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::addProgessBar(const string& id, const int& value, const int& x_offset, const int& y_offset)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<int > value_value(value, false);
    TInValue<int > x_offset_value(x_offset, false);
    TInValue<int > y_offset_value(y_offset, false);
    Event event("IOSDOverlay::addProgessBar", &id_value, &value_value, &x_offset_value, &y_offset_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::addImage(const string& id, const string& url, const int& x_offset, const int& y_offset)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > url_value(url, false);
    TInValue<int > x_offset_value(x_offset, false);
    TInValue<int > y_offset_value(y_offset, false);
    Event event("IOSDOverlay::addImage", &id_value, &url_value, &x_offset_value, &y_offset_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::setItemOffset(const string& id, const int& x_offset, const int& y_offset)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<int > x_offset_value(x_offset, false);
    TInValue<int > y_offset_value(y_offset, false);
    Event event("IOSDOverlay::setItemOffset", &id_value, &x_offset_value, &y_offset_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::setItemText(const string& id, const string& text)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > text_value(text, false);
    Event event("IOSDOverlay::setItemText", &id_value, &text_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::setItemEnabled(const string& id, const bool& enable)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<bool > enable_value(enable, false);
    Event event("IOSDOverlay::setItemEnabled", &id_value, &enable_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::disableItems()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDOverlay::disableItems");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::doPause(const bool& pause)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<bool > pause_value(pause, false);
    Event event("IOSDOverlay::doPause", &pause_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDOverlay::update()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDOverlay::update");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOSDOverlay::IOSDOverlay_type = "IOSDOverlay";
  TIFactoryObject<IOSDOverlay > __IOSDOverlay_prototype("IOSDOverlay");


  const char* IOSDOverlay::addText_event = "IOSDOverlay::addText";
  const char* IOSDOverlay::printText_event = "IOSDOverlay::printText";
  const char* IOSDOverlay::removeText_event = "IOSDOverlay::removeText";
  const char* IOSDOverlay::addProgessBar_event = "IOSDOverlay::addProgessBar";
  const char* IOSDOverlay::addImage_event = "IOSDOverlay::addImage";
  const char* IOSDOverlay::setItemOffset_event = "IOSDOverlay::setItemOffset";
  const char* IOSDOverlay::setItemText_event = "IOSDOverlay::setItemText";
  const char* IOSDOverlay::setItemEnabled_event = "IOSDOverlay::setItemEnabled";
  const char* IOSDOverlay::disableItems_event = "IOSDOverlay::disableItems";
  const char* IOSDOverlay::doPause_event = "IOSDOverlay::doPause";
  const char* IOSDOverlay::update_event = "IOSDOverlay::update";


}


