
#ifndef NMM_IBitmapReaderImpl_HPP
#define NMM_IBitmapReaderImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IBitmapReader.hpp"


namespace NMM
{

  class IBitmapReaderImpl  : public virtual NMMObject { 
  public:
    IBitmapReaderImpl() { addInterface(new IBitmapReader(*this)); registerEvents(); };

    virtual ~IBitmapReaderImpl() {};

  protected:
    /**
     * loads a (PNG) image that can be accessed by the given identifier
     * @param id identifier for accessing image later
     * @param filename filename of the (png) image
     * @param f color format that should be used (RGBA, YV12, YV12A)
     * @returns true, if bitmap could be read
     * Note: an image that is already saved under the given id, will be deleted before automatically
     */

    virtual bool loadImage(const string& id, const string& filename, const IBitmapReader::Format& f = IBitmapReader::RGBA) = 0;
    /**
     * Get width and height of an image.
     * @param id identifier of the image
     * @param w width of the image
     * @param h width of the image
     */

    virtual void getImage(const string& id, unsigned int& w, unsigned int& h) = 0;
    /**
     * Creates an image with the given color.
     * @param id identifier for accessing image later
     * @param width width of the bitmap
     * @param height height of the bitmap
     * @param color color of the image (RGBA)
     * @param f color format that should be used (RGBA, YV12, YV12A)
     */

    virtual bool createImage(const string& id, const unsigned int& width, const unsigned int& height, const unsigned int& color1, const unsigned int& color2, const unsigned int& color3, const IBitmapReader::Format& f = IBitmapReader::RGBA) = 0;
    /** Deletes image with the given id */

    virtual void freeImage(const string& id) = 0;
    /** Deletes all read images */

    virtual void freeAll() = 0;


  private:
    void registerEvents();
  };

}

#endif
