
#include "IOSDManager.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IOSDManager::loadFont(const string& id, const string& xmlfilename, const string& fontname)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > xmlfilename_value(xmlfilename, false);
    TInValue<string > fontname_value(fontname, false);
    Event event("IOSDManager::loadFont", &id_value, &xmlfilename_value, &fontname_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  IOSDManager::getFontList() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("IOSDManager::getFontList");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::update()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::update");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWidget* IOSDManager::getWidget(const int& id)
      throw(const Exception&)
  {
    TOutValue<IWidget* > return_value;
    TInValue<int > id_value(id, false);
    Event event("IOSDManager::getWidget", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWindow* IOSDManager::getWindow(const int& id)
      throw(const Exception&)
  {
    TOutValue<IWindow* > return_value;
    TInValue<int > id_value(id, false);
    Event event("IOSDManager::getWindow", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWindow* IOSDManager::createWindow()
      throw(const Exception&)
  {
    TOutValue<IWindow* > return_value;
    Event event("IOSDManager::createWindow");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::deleteWindow(const IWindow& window)
      throw(const Exception&)
  {
    TInValue<IWindow > window_value(window, false);
    Event event("IOSDManager::deleteWindow", &window_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::deleteWidget(const IWidget& widget)
      throw(const Exception&)
  {
    TInValue<IWidget > widget_value(widget, false);
    Event event("IOSDManager::deleteWidget", &widget_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result IOSDManager::displayMessage(const string& header, const string& text, const string& label1, const string& label2 /* = "" */, const string& label3 /* = "" */, const unsigned int& sel /* = 0 */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > header_value(header, false);
    TInValue<string > text_value(text, false);
    TInValue<string > label1_value(label1, false);
    TInValue<string > label2_value(label2, false);
    TInValue<string > label3_value(label3, false);
    TInValue<unsigned int > sel_value(sel, false);
    Event event("IOSDManager::displayMessage", &header_value, &text_value, &label1_value, &label2_value, &label3_value, &sel_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::deleteMessage()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::deleteMessage");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::displayTime(const string& header /* = "" */, const unsigned int& x /* = 500 */, const unsigned int& y /* = 410 */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > header_value(header, false);
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    Event event("IOSDManager::displayTime", &header_value, &x_value, &y_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::createTime(const bool& ct)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<bool > ct_value(ct, false);
    Event event("IOSDManager::createTime", &ct_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::resetTime()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::resetTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideTime()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::hideTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::setTime(const u_int64_t& time)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<u_int64_t > time_value(time, false);
    Event event("IOSDManager::setTime", &time_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  u_int64_t IOSDManager::getTime()
      throw(const Exception&)
  {
    TOutValue<u_int64_t > return_value;
    Event event("IOSDManager::getTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::displaySymbol(const string& symbol, const unsigned int& x /* = 600 */, const unsigned int& y /* = 50 */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > symbol_value(symbol, false);
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    Event event("IOSDManager::displaySymbol", &symbol_value, &x_value, &y_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideSymbol()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::hideSymbol");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::displayProgressBar(const string& header /* = "" */, const unsigned int& x /* = 50 */, const unsigned int& y /* = 470 */, const unsigned int& width /* = 620 */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > header_value(header, false);
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    TInValue<unsigned int > width_value(width, false);
    Event event("IOSDManager::displayProgressBar", &header_value, &x_value, &y_value, &width_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideProgressBar()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::hideProgressBar");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setMaxProgress(const u_int64_t& max)
      throw(const Exception&)
  {
    TInValue<u_int64_t > max_value(max, false);
    Event event("IOSDManager::setMaxProgress", &max_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::setProgress(const u_int64_t& progress)
      throw(const Exception&)
  {
    TInValue<u_int64_t > progress_value(progress, false);
    Event event("IOSDManager::setProgress", &progress_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::updateProgress()
      throw(const Exception&)
  {
    Event event("IOSDManager::updateProgress");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result IOSDManager::displayColorBar(const unsigned int& x /* = 80 */, const unsigned int& y /* = 480 */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    Event event("IOSDManager::displayColorBar", &x_value, &y_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideColorBar()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::hideColorBar");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setColorBarText(const string& red, const string& green, const string& yellow, const string& blue)
      throw(const Exception&)
  {
    TInValue<string > red_value(red, false);
    TInValue<string > green_value(green, false);
    TInValue<string > yellow_value(yellow, false);
    TInValue<string > blue_value(blue, false);
    Event event("IOSDManager::setColorBarText", &red_value, &green_value, &yellow_value, &blue_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::setRedColorBarText(const string& text)
      throw(const Exception&)
  {
    TInValue<string > text_value(text, false);
    Event event("IOSDManager::setRedColorBarText", &text_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::setGreenColorBarText(const string& text)
      throw(const Exception&)
  {
    TInValue<string > text_value(text, false);
    Event event("IOSDManager::setGreenColorBarText", &text_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::setYellowColorBarText(const string& text)
      throw(const Exception&)
  {
    TInValue<string > text_value(text, false);
    Event event("IOSDManager::setYellowColorBarText", &text_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::setBlueColorBarText(const string& text)
      throw(const Exception&)
  {
    TInValue<string > text_value(text, false);
    Event event("IOSDManager::setBlueColorBarText", &text_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string IOSDManager::getRedColorBarText()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IOSDManager::getRedColorBarText");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::getGreenColorBarText()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IOSDManager::getGreenColorBarText");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::getYellowColorBarText()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IOSDManager::getYellowColorBarText");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::getBlueColorBarText()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IOSDManager::getBlueColorBarText");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::messageLeft()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::messageLeft");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::messageRight()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IOSDManager::messageRight");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::messageID()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IOSDManager::messageID");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setPause(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("IOSDManager::setPause", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IOSDManager::sendPauseEvent()
      throw(const Exception&)
  {
    Event event("IOSDManager::sendPauseEvent");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IBitmapReader* IOSDManager::getBitmapReader()
      throw(const Exception&)
  {
    TOutValue<IBitmapReader* > return_value;
    Event event("IOSDManager::getBitmapReader");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::checkCharDimension(const string& id, const unsigned char& c, unsigned int& x, unsigned int& y) const
      throw(const Exception&)
  {
    TInValue<string > id_value(id, false);
    TInValue<unsigned char > c_value(c, false);
    TOutValue<unsigned int > x_value(x);
    TOutValue<unsigned int > y_value(y);
    Event event("IOSDManager::checkCharDimension", &id_value, &c_value, &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  unsigned int IOSDManager::getMaxWidth(const string& id) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IOSDManager::getMaxWidth", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IOSDManager::getMaxHeight(const string& id) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IOSDManager::getMaxHeight", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setLineSpace(const string& id, const unsigned int& space)
      throw(const Exception&)
  {
    TInValue<string > id_value(id, false);
    TInValue<unsigned int > space_value(space, false);
    Event event("IOSDManager::setLineSpace", &id_value, &space_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IOSDManager::getLineSpace(const string& id) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<string > id_value(id, false);
    Event event("IOSDManager::getLineSpace", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOSDManager::IOSDManager_type = "IOSDManager";
  TIFactoryObject<IOSDManager > __IOSDManager_prototype("IOSDManager");


  const char* IOSDManager::loadFont_event = "IOSDManager::loadFont";
  const char* IOSDManager::getFontList_event = "IOSDManager::getFontList";
  const char* IOSDManager::update_event = "IOSDManager::update";
  const char* IOSDManager::getWidget_event = "IOSDManager::getWidget";
  const char* IOSDManager::getWindow_event = "IOSDManager::getWindow";
  const char* IOSDManager::createWindow_event = "IOSDManager::createWindow";
  const char* IOSDManager::deleteWindow_event = "IOSDManager::deleteWindow";
  const char* IOSDManager::deleteWidget_event = "IOSDManager::deleteWidget";
  const char* IOSDManager::displayMessage_event = "IOSDManager::displayMessage";
  const char* IOSDManager::deleteMessage_event = "IOSDManager::deleteMessage";
  const char* IOSDManager::displayTime_event = "IOSDManager::displayTime";
  const char* IOSDManager::createTime_event = "IOSDManager::createTime";
  const char* IOSDManager::resetTime_event = "IOSDManager::resetTime";
  const char* IOSDManager::hideTime_event = "IOSDManager::hideTime";
  const char* IOSDManager::setTime_event = "IOSDManager::setTime";
  const char* IOSDManager::getTime_event = "IOSDManager::getTime";
  const char* IOSDManager::displaySymbol_event = "IOSDManager::displaySymbol";
  const char* IOSDManager::hideSymbol_event = "IOSDManager::hideSymbol";
  const char* IOSDManager::displayProgressBar_event = "IOSDManager::displayProgressBar";
  const char* IOSDManager::hideProgressBar_event = "IOSDManager::hideProgressBar";
  const char* IOSDManager::setMaxProgress_event = "IOSDManager::setMaxProgress";
  const char* IOSDManager::setProgress_event = "IOSDManager::setProgress";
  const char* IOSDManager::updateProgress_event = "IOSDManager::updateProgress";
  const char* IOSDManager::displayColorBar_event = "IOSDManager::displayColorBar";
  const char* IOSDManager::hideColorBar_event = "IOSDManager::hideColorBar";
  const char* IOSDManager::setColorBarText_event = "IOSDManager::setColorBarText";
  const char* IOSDManager::setRedColorBarText_event = "IOSDManager::setRedColorBarText";
  const char* IOSDManager::setGreenColorBarText_event = "IOSDManager::setGreenColorBarText";
  const char* IOSDManager::setYellowColorBarText_event = "IOSDManager::setYellowColorBarText";
  const char* IOSDManager::setBlueColorBarText_event = "IOSDManager::setBlueColorBarText";
  const char* IOSDManager::getRedColorBarText_event = "IOSDManager::getRedColorBarText";
  const char* IOSDManager::getGreenColorBarText_event = "IOSDManager::getGreenColorBarText";
  const char* IOSDManager::getYellowColorBarText_event = "IOSDManager::getYellowColorBarText";
  const char* IOSDManager::getBlueColorBarText_event = "IOSDManager::getBlueColorBarText";
  const char* IOSDManager::messageLeft_event = "IOSDManager::messageLeft";
  const char* IOSDManager::messageRight_event = "IOSDManager::messageRight";
  const char* IOSDManager::messageID_event = "IOSDManager::messageID";
  const char* IOSDManager::setPause_event = "IOSDManager::setPause";
  const char* IOSDManager::sendPauseEvent_event = "IOSDManager::sendPauseEvent";
  const char* IOSDManager::getBitmapReader_event = "IOSDManager::getBitmapReader";
  const char* IOSDManager::checkCharDimension_event = "IOSDManager::checkCharDimension";
  const char* IOSDManager::getMaxWidth_event = "IOSDManager::getMaxWidth";
  const char* IOSDManager::getMaxHeight_event = "IOSDManager::getMaxHeight";
  const char* IOSDManager::setLineSpace_event = "IOSDManager::setLineSpace";
  const char* IOSDManager::getLineSpace_event = "IOSDManager::getLineSpace";


}


