
#ifndef NMM_IScrollableImpl_HPP
#define NMM_IScrollableImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IScrollable.hpp"


namespace NMM
{
  /**
   * Scrollable item.
   * If an item (need not be a widget) has support for scrollbars, it should
   * implement this interface.
   */

  class IScrollableImpl  : public virtual NMMObject { 
  public:
    IScrollableImpl() { addInterface(new IScrollable(*this)); registerEvents(); };

    virtual ~IScrollableImpl() {};

  protected:
    /**
     * Get height of vertical scrollbar.
     * Returns the actual scrollbar height (per 10000).
     * That means a value of 10000 indicates a full scrollbar height.
     */

    virtual int getScrollBarHeight() const = 0;
    /**
     * Get width of horizontal scrollbar.
     * Returns the actual scrollbar width (per 10000).
     * That means a value of 10000 indicates a full scrollbar width.
     */

    virtual int getScrollBarWidth() const = 0;
    /**
     * Get vertical scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual int getScrollBarYPosition() const = 0;
    /**
     * Get horizontal scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual int getScrollBarXPosition() const = 0;


  private:
    void registerEvents();
  };

}

#endif
