
#ifndef NMM_ITimeView_HPP
#define NMM_ITimeView_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IWidget.hpp"

namespace NMM
{
  /**
   * Time view widget interface.
   */

  class ITimeView : virtual public IWidget { 
  public:
    ITimeView() {  setTypeInfo(TypeInfo(ITimeView::ITimeView_type, "NMM")); };

    ITimeView(NMMObject& parent) : IWidget(parent) {  setTypeInfo(TypeInfo(ITimeView::ITimeView_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITimeView() {};

    static const char* ITimeView_type;

    virtual  string getInterfaceType() const { return ITimeView_type; }

    Interface* copy() const { return new ITimeView(*this); }
    static const char* setFont_event;
    static const char* setPrefix_event;
    static const char* setTimeOffset_event;
    static const char* setCurrentTime_event;
    static const char* getCurrentTime_event;
    static const char* reset_event;
    /**
     * Set font.
     * Set the font that is used to draw the text.
     * @param value font name
     */

    virtual void setFont(const string& value)
      throw(const Exception&);
    /**
     * Set prefix text.
     * Set prefix text that is placed in front of the time.
     * @param value prefix text
     */

    virtual void setPrefix(const string& value)
      throw(const Exception&);
    /** Sets time offset. */

    virtual void setTimeOffset(const int& value)
      throw(const Exception&);
    /** Set current time. */

    virtual void setCurrentTime(const int& value)
      throw(const Exception&);
    /** Get current time. */

    virtual int getCurrentTime()
      throw(const Exception&);
    /** Resets timer. */

    virtual void reset()
      throw(const Exception&);


  };

  typedef auto_ptr<ITimeView> ITimeView_var;




}

#endif
