
#ifndef NMM_IWidgetImpl_HPP
#define NMM_IWidgetImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IWidget.hpp"

#include "IScrollableImpl.hpp"

namespace NMM
{
  /**
   * Widget interface.
   * Interface for controlling widgets in an OSDManager.
   * The widgets are identified by integers. To get an interface, use the IOSDManager interface.
   */

  class IWidgetImpl  : virtual public IScrollableImpl { 
  public:
    IWidgetImpl() { addInterface(new IWidget(*this)); registerEvents(); };

    virtual ~IWidgetImpl() {};

  protected:
    /** Selects next button (adds 1 to buttonid). */

    virtual void selectRight() = 0;
    /** Selects privious button (subs 1 to buttonid). */

    virtual void selectLeft() = 0;
    /** Selects button (in privoius line, subs step to buttonid). */

    virtual void selectUp() = 0;
    /** Selects button (in next line, adds step to buttonid). */

    virtual void selectDown() = 0;
    /** Activates the selected button. */

    virtual void selectPush() = 0;
    /** Get widget ID. */

    virtual int getWidgetID() const = 0;
    /**
     * Set x and y position of the widget in the window.
     * @param x x position
     * @param y x position
     */

    virtual void setPosition(const unsigned int& x, const unsigned int& y) = 0;
    /**
     * Set width and height of the widget.
     * @param w width
     * @param h height
     */

    virtual void setDimension(const unsigned int& w, const unsigned int& h) = 0;
    /**
     * Get x and y position of the widget.
     * @param x x position
     * @param y x position
     */

    virtual void getPosition(unsigned int& x, unsigned int& y) const = 0;
    /**
     * Get width and height of the widget.
     * @param w width
     * @param h height
     */

    virtual void getDimension(unsigned int& w, unsigned int& h) const = 0;


  private:
    void registerEvents();
  };

}

#endif
