
#include "IWindow.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  int IWindow::getWindowID() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IWindow::getWindowID");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IWindow::setPosition(const unsigned int& x, const unsigned int& y)
      throw(const Exception&)
  {
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    Event event("IWindow::setPosition", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWindow::getPosition(unsigned int& x, unsigned int& y) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > x_value(x);
    TOutValue<unsigned int > y_value(y);
    Event event("IWindow::getPosition", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  void IWindow::setDimension(const unsigned int& w, const unsigned int& h)
      throw(const Exception&)
  {
    TInValue<unsigned int > w_value(w, false);
    TInValue<unsigned int > h_value(h, false);
    Event event("IWindow::setDimension", &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWindow::getDimension(unsigned int& w, unsigned int& h) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > w_value(w);
    TOutValue<unsigned int > h_value(h);
    Event event("IWindow::getDimension", &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }
  
  void IWindow::update()
      throw(const Exception&)
  {
    Event event("IWindow::update");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool IWindow::getVisible() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IWindow::getVisible");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IWindow::setVisible(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("IWindow::setVisible", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWindow::setScaleFactor(const double& x, const double& y)
      throw(const Exception&)
  {
    TInValue<double > x_value(x, false);
    TInValue<double > y_value(y, false);
    Event event("IWindow::setScaleFactor", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWindow::setContent(const IWidget& value)
      throw(const Exception&)
  {
    TInValue<IWidget > value_value(value, false);
    Event event("IWindow::setContent", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IWidget* IWindow::getContent() const
      throw(const Exception&)
  {
    TOutValue<IWidget* > return_value;
    Event event("IWindow::getContent");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IWindow::IWindow_type = "IWindow";
  TIFactoryObject<IWindow > __IWindow_prototype("IWindow");


  const char* IWindow::getWindowID_event = "IWindow::getWindowID";
  const char* IWindow::setPosition_event = "IWindow::setPosition";
  const char* IWindow::getPosition_event = "IWindow::getPosition";
  const char* IWindow::setDimension_event = "IWindow::setDimension";
  const char* IWindow::getDimension_event = "IWindow::getDimension";
  const char* IWindow::update_event = "IWindow::update";
  const char* IWindow::getVisible_event = "IWindow::getVisible";
  const char* IWindow::setVisible_event = "IWindow::setVisible";
  const char* IWindow::setScaleFactor_event = "IWindow::setScaleFactor";
  const char* IWindow::setContent_event = "IWindow::setContent";
  const char* IWindow::getContent_event = "IWindow::getContent";


}


