
#ifndef NMM_IEPG_HPP
#define NMM_IEPG_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{
  /**
   * Interface for an Electronic Programming Guide
   * to provide additional information to the program shown
   */

  class IEPG : virtual public Interface { 
  public:
    IEPG() {  setTypeInfo(TypeInfo(IEPG::IEPG_type, "NMM")); };

    IEPG(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IEPG::IEPG_type, "NMM")); setEventReceiver(parent); };

    virtual ~IEPG() {};

    static const char* IEPG_type;

    virtual  string getInterfaceType() const { return IEPG_type; }

    Interface* copy() const { return new IEPG(*this); }
    static const char* getCurrentShow_event;
    static const char* getFollowingShow_event;
    static const char* getDetailedDescription_event;
    static const char* setEPGDirectory_event;
    static const char* getActivity_event;
    static const char* scan_event;
    /**
     * returns a string wich contains the starttime, 
     * endtime and title of the current show
     * Format: hh:mm - hh:mm : title
     */

    virtual string getCurrentShow() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * returns a string wich contains the starttime, 
     * endtime and title of the following show
     * Format: hh:mm - hh:mm : title
     */

    virtual string getFollowingShow() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * returns a string containing a detailed description
     * of the curretn show
     */

    virtual string getDetailedDescription() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * sets the Directory, where the XML-file with EPG-information is stored
     */

    virtual void setEPGDirectory(const string& dir)
      throw(const Exception&);
    /**
     * restores the channel before starting the scanning-function
     */

    virtual void getActivity() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * starts the scanning-function of VDR to collect EPG-data
     */

    virtual void scan() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);


  };

  typedef auto_ptr<IEPG> IEPG_var;




}

#endif
