/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "LCDWriter.hpp"

using namespace NMM;

//const unsigned char  LCDWriter::bars[] = { ' ', 0x04, 0x05, 0x06, 0x07, 0x08, 0x02, 0x03 };
const unsigned char  LCDWriter::bars[] = { ' ',
					   CHARNUM_BAR1,
					   CHARNUM_BAR2,
					   CHARNUM_BAR3,
					   CHARNUM_BAR4,
					   CHARNUM_BAR5,
					   CHARNUM_TRADEMARK_T,
					   CHARNUM_TRADEMARK_M};
const char* LCDWriter::emptyLine = "                    ";

LCDWriter::LCDWriter ()
{
  isScrollingLine0 = false;
  isScrollingLine1 = false;
  fd = -1;
}

LCDWriter::~LCDWriter ()
{
  close(fd);
}

Result LCDWriter::init()
{
  fd = open("/dev/lcd/lcd_2x20", O_RDWR);
  if (fd < 0) {
    GLOBAL_DEBUG_STREAM("Could not find /dev/lcd/lcd_2x20! Make sure the lcd-module is loaded." << endl);
    return FAILURE;
  }
  clear();
  return SUCCESS;
}

int LCDWriter::clear()
{
  strncpy((char *)l.line0,emptyLine, LINE_LENGTH);
  strncpy((char *)l.line1,emptyLine, LINE_LENGTH);
  return (ioctl(fd, LCD_CLEAR, &l));
}

int LCDWriter::setText (int line, const char* text)
{
  if (line==0) strncpy((char *)l.line0, text, LINE_LENGTH);
  else if (line==1) strncpy((char *)l.line1, text, LINE_LENGTH);
  return(ioctl(fd, LCD_WRITE, &l));
}

int LCDWriter::setDateTime (int line)
{
  time_t timer;
  timer=time(NULL);
  if (line==0) sprintf((char *)l.line0, asctime(localtime(&timer)));
  else if (line==1) sprintf((char *)l.line1, asctime(localtime(&timer)));
  return(ioctl(fd, LCD_WRITE, &l));
}

int LCDWriter::setChar(int x, int y, char c)
{
  // Respect the range of the display
  if (x>19) x=19;
  if (x<0)  x=0;
  if (y>1)  y=1;
  if (y<0)  y=0;
  l.posx=x;
  l.posy=y;
  l.ch=c;
  if (y==0) l.line0[x]=l.ch;
  else if (y==1) l.line1[x]=l.ch;
  return(ioctl(fd, LCD_WRITE_CHAR, &l));
}

int LCDWriter::clearLine(int line)
{
  return(setText(line, emptyLine));
}

void LCDWriter::setProgressBar(int line, int percent_)
{
  for (int i=0; i<LINE_LENGTH; i++)
  {
    setChar(i, line, bars[(percent_<(i+1)*5)?((percent_>i*5)?percent_%5:0):5]);
  }
}

Result LCDWriter::receiveStartTrack(string& name) {
  GLOBAL_DEBUG_STREAM("Received Event with content: " << name << endl);

  MutexGuard mg_em(event_mutex);
  
  // remove leading path from filename
  unsigned int lastPos = name.find_last_of('/');
  string line00;
  string line01;
  // if string contains '/', take rest of string from '/' on
  if (lastPos != string::npos) {
    line00 = name.substr(lastPos + 1, name.length());
  }
  else {
    // else take whole string
    line00 = name;
  }
  // break lines
  if (line00.length() > LINE_LENGTH) {
    line01 = line00.substr(LINE_LENGTH, line00.length());
    line00 = line00.substr(          0, LINE_LENGTH);
  }
  else {
    line01 = string(emptyLine);
  }      
  
  clear();
  setText(0, line00.c_str());
  setText(1, line01.c_str());
  return SUCCESS;
}
