/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "LircEvents.hpp"
#include "nmm/base/Event.hpp"

namespace NMM {

struct lirc_config* config;
    int lirc_fd;
    bool send_extended_events;
  
  LircProducer::LircProducer()
    : config(NULL),
      lirc_fd(-1),
      send_extended_events(false)
  {}
  
  LircProducer::~LircProducer(){
    lirc_freeconfig(config);
    lirc_deinit();
  }
  
  bool LircProducer::readConfig(string fname) {
    if (-1 != lirc_fd) {
      if (config) {
        lirc_freeconfig(config);
        config = NULL;
      }
      lirc_deinit();
    }
    if ((lirc_fd = lirc_init((char*)"nmmlirc", 1)) == -1)
    {
      GLOBAL_ERROR_STREAM( "LircProducer: lirc_init failed" << endl);
      return false;
    }
    
    setFileDescriptor(lirc_fd);
    
    if(lirc_readconfig((char*)fname.c_str(), &config, NULL)!=0)
    {
      GLOBAL_ERROR_STREAM( "LircProducer: lirc_readconfig failed: " << fname << endl);
      return false;
    }
    return true;
  }
  
  void LircProducer::setSendExtendedEvents(bool sendExEv) {
    send_extended_events = sendExEv;
  }
  
  bool LircProducer::getSendExtendedEvents() {
    return send_extended_events;
  }
  
  void LircProducer::handleInput() {
    char* code;
    char* c;
    int   ret;
    
    if (lirc_nextcode(&code)==0) {
      if(code==NULL) return;
      while ((ret=lirc_code2char(config,code,&c))==0 && c!=NULL) {
        GLOBAL_DEBUG_STREAM( "Got New Event: " << c << endl);
        if (send_extended_events) {
          sendEvent(Event(c, new TInValue<string>(c))/*, this*/);
        }
        else {
          sendEvent(Event(c)/*, this*/);
        }
      }
      free(code);
      if(ret==-1) {
        GLOBAL_ERROR_STREAM("LircProducer> Error computing char from code!" << endl);
      }
    } // endif (lirc_nextcode(&code)==0)
  }
  
}

