/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_VIDEOSCALER_HPP_
#define _NMM_VIDEOSCALER_HPP_

#include "nmm/NMMTypes.hpp"
#include "scaler.h"

namespace NMM {
  
  class VideoScaler {
    
  public:
    enum {
      FORMAT_RGB,
      FORMAT_RGBA,
      FORMAT_YV12,
      FORMAT_YV12A
    };
    
    enum {
      MAX_PLANES = 6
    };
    
  private:
    
    // format info
    int format;
    int in_width;
    int in_height;
    int out_width;
    int out_height;
    bool filtering;
    
    // internal format info
    bool planar;
    int num_planes;
    ImagePlane in_planes[MAX_PLANES];
    ImagePlane out_planes[MAX_PLANES];
    
    void initPlanes();
    void initPlanes(int w, int h, ImagePlane* planes);
    
  public:
    VideoScaler();
    
    int getFormat() const { return format; }
    int getInputWidth() const { return in_width; }
    int getInputHeight() const { return in_height; }
    int getOutputWidth() const { return out_width; }
    int getOutputHeight() const { return out_height; }
    bool getFiltering() const { return filtering; }
    void setFormat(int format);
    void setInputSize(int width, int height);
    void setOutputSize(int width, int height);
    void setFiltering(bool filtering) { this->filtering = filtering; }
    
    void processData(char* out_data, const char* in_data);
    
  private:
    static void processPlane(char* out_data, int out_width, int out_height, int out_pitch,
			     const char* in_data, int in_pitch,
			     int in_delta_x, int in_delta_y);
    static void processPlaneFiltering(char* out_data, int out_width, int out_height, int out_pitch,
				      const char* in_data, int in_pitch,
				      int in_delta_x, int in_delta_y);
  };
}

#endif //_NMM_VIDEOSCALER_HPP_
