/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>    
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_TIMER_HPP_
#define _NMM_TIMER_HPP_

#include "nmm/utils/thread/ThreadedObject.hpp"
#include "nmm/utils/thread/Thread.hpp"

namespace NMM {

class TimerListener;

/** This class can be used to trigger a certain listener in a given interval.
  */
class Timer : public ThreadedObject {
public:
  /** Constructor */
  Timer(TimerListener& listener);

  /** Destructor */
  virtual ~Timer();

  /** starts the timer and triggers method timeout() of the listener in the given periode
    * \param sec specifies the seconds of the period
    * \param nsec specifies the nano seconds of the period
    */
  void start(int sec, int nsec = 0);

  /** Stops the timer and terminates the thread, so that the listener no longer triggerd.
    * Important note: The Listener must not call this method in its timeout() method, because
    * stop() terminates the thread used to call this timeout() method. !!!
    */
  void stop();
protected:
  /** Starts the internal thread */
  virtual void run();

private:
  bool            __is_running;           ///< specify if the thread is still running and the listener must be informed
  TimerListener&  __listener;             ///< The listener that should be informed
  ThreadMutex     __condition_mutex;      ///< Mutex for the internal thread_condition
  ThreadCondition __thread_condition;     ///< This condition is used to wait for the given time
  int __sec;                              ///< stores the seconds to wait
  int __nsec;                             ///< stores the nano seconds to wait.
};



class TimerListener
{
public:
  virtual ~TimerListener() {}
  /* This method is called from the timer */
  virtual void timeout(Timer*) = 0;        
};

}

#endif
