/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef TVTIMERSCHEDULER_HPP
#define TVTIMERSCHEDULER_HPP TVTIMERSCHEDULER_HPP

#include "nmm/misc/tvtimer/TVTimerEntry.hpp"
#include "nmm/misc/tvtimer/TVTimerList.hpp"
#include "nmm/misc/tvtimer/ConflictChecker.hpp"
#include "nmm/misc/tvtimer/TVCardSearch.hpp"
#include "nmm/misc/tvtimer/TVTimerExceptions.hpp"

#include "nmm/base/utils/GraphURL.hpp"


namespace NMM{

  class TVTimerScheduler{
  private:

    TVTimerConflictChecker *conflictChecker;
    TVChannelList *channelConfig;

  public:
    TVTimerScheduler();
    ~TVTimerScheduler();

    /**
     * Serch for new cards and test for conflicts with existing timers
     */
    Result schedule(TVTimerEntry &entry);
    /**
     * if an error occured because of an unavailbale TVCard, search for a new card
     */
    Result reSchedule(TVTimerEntry &entry);

    /**
     * Sets the ChannelConfig to have all channels available
     */
    void setChannelConfig(TVChannelList *cc){channelConfig = cc;}
    TVChannelList* getChannelConfig();

  private:
    NodeDescription urlToNodeDescription(GraphURL url);
    bool contains(list<TVCardDescription> &tvcardlist, TVCardDescription &tvcard);
  };
}
#endif

