/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef TVRECORDERMANAGER_HPP
#define TVRECORDERMANAGER_HPP TVRECORDERMANAGER_HPP

#include "nmm/misc/tvtimer/TVTimerEntry.hpp"

#include "nmm/interfaces/misc/tvtimer/ITVTimer.hpp"
#include "nmm/interfaces/video/tv/ITVCard.hpp"

#include "nmm/misc/timer/TimerEvents.hpp"
#include "nmm/base/NMMObject.hpp"
#include "nmm/base/ProxyApplication.hpp"

#include "nmm/interfaces/base/IUsageNotifyImpl.hpp"

namespace NMM{

  class TVRecorderManager;

  class TVRecorder
  {
  private:
    TimerProducer *recTimer;
    TVTimerEntry timerEntry;
    TVRecorderManager *manager;

    GraphDescription graph;
    string directory;
    string filename;
    NMMApplication *app;

    TEDObject1<TVRecorder, int> channelSwitched_listener;
    INode *tvcard;

  public:
    TVRecorder(TVTimerEntry entry, TVRecorderManager *rm);
    ~TVRecorder();

    /**
     * Start the recorder to record a given timer
     */
    Result start();
    /**
     * Start the recorder bbut request the TVSource shared
     */
    Result startShared();
    /**
     * Stops the recording
     */
    void stop();
    
    /**
     * Set the Directory, where the recording should be stored 
     */
    void setDirectory(string dir);
    /**
     * Get the Directory, where the recording should be stored 
     */
    string getDirectory();
    /**
     * Sets the NMMApplication to have a registry to request the node from
     */
    void setApplication(NMMApplication *a){app = a;}

  private:
    Result buildGraph(NodeDescription::SharingType st);
    Result releaseGraph();
    Result setFilename();

    Result channelSwitched(int &channel);
    Result isMaster(INode *card);

  };


  class TVRecorderManager : 
    public IUsageNotifyImpl
  {
  private:
    TimerProducer *timer;
    map<unsigned int, TVRecorder*> recorder;

    TEDObject1<TVRecorderManager, unsigned int> recordStartTimer;
    TEDObject1<TVRecorderManager, unsigned int> recordStopTimer;

    string directory;
    NMMApplication *app;

    ITVTimer_var controller;

  public:
    TVRecorderManager(ITVTimer *itvtimer);
    ~TVRecorderManager();

    /**
     * program the given timer for recording
     */
    Result record(TVTimerEntry &entry);
    /**
     * start a shared-recording in case of errors
     */
    Result recordShared(TVTimerEntry &entry);

    /**
     * delete the programmed timer
     */
    Result removeRecord(unsigned int id);

    /**
     * Starts the recorder with the given ID
     */
    Result startRecorder(unsigned int &id);
    /**
     * Stops the recorder with the given ID
     */
    Result stopRecorder(unsigned int &id);

    /**
     * Get the Directory, where the recording should be stored 
     */
    void setDirectory(string dir);
    /**
     * Get the Directory, where the recording should be stored 
     */
    string getDirectory();

    /**
     * Sets the NMMApplication to have a registry to request the node from
     */
    void setApplication(NMMApplication *a){app = a;}

  protected:
    //implement IUsageNofifyImpl
    Result becomeMaster(){return SUCCESS;}
    Result becomeSlave(){return SUCCESS;}

  private:
    Result determineNextTimer(TVTimerEntry &entry);

  };
}

#endif
