/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef TVTIMERENTRY_HPP
#define TVTIMERENTRY_HPP TVTIMERENTRY_HPP

#include "nmm/interfaces/misc/tvguide/ProgramDescription.hpp"
#include "nmm/base/serialize/ComplexValue.hpp"
#include "nmm/base/TSAddress.hpp"
#include "nmm/base/registry/NodeDescription.hpp"

#include "nmm/misc/tvtimer/TVTimerExceptions.hpp"

namespace NMM{

  class TVCardDescription : public NodeDescription {
  private:
  public: 
    TVCardDescription();
    TVCardDescription(string t, TSAddress a);
    TVCardDescription(const TVCardDescription &t);
    TVCardDescription(const NodeDescription &nd);
    ~TVCardDescription();

    /**
     * Set the type/nodename of the TVCard
     */
    void  setType(string t){setNodeName(t);}
    /**
     * Get the type of the TVCard
     */
    string getType(){return getNodeName();}

    
    void setAddress(TSAddress a);
    TSAddress getAddress();

    bool operator==(const TVCardDescription &t) const;

  private:
    void registerValues();
  };

  class TVTimerEntry : public SerializedValue {

  public:
    enum repmode_e { ONCE=0, DAILY, WEEKLY, WEEKDAYS, WEEKENDS, MONTHLY, RANDOM, RANDOM_ONLY_CHANNEL, RANDOM_ONLY_CHANNEL_AND_TIME};
    enum status_e  { SLEEPING=0, RECORDING, RECORDED, RECORDED_WITH_ERROR, CONFLICT };
    
  private:
    NMM::TValue<unsigned int>              id;
    NMM::TValue<time_t>                    start_time; 
    NMM::TValue<time_t>                    end_time;
    NMM::TValue<unsigned int>              channelNumber;
    NMM::TValue<string>                    channelName;
    NMM::TValue<unsigned int>              status;
    NMM::TValue<bool>                      conflicts;
    NMM::TValue<unsigned int>              period;
    NMM::TValue<ProgramDescription>        info;
    NMM::TValue<list<ProgramDescription> > nextTimers; //save info about next Events when period==RANDOM 
    NMM::TValue<list<TVCardDescription> >  tvcard;
    NMM::TValue<string>                    user;
    NMM::TValue<string>                    fileURL;
    NMM::TValue<list<int> >                conflictingTimers;
 
  public:

    TVTimerEntry();
    TVTimerEntry(string u);
    TVTimerEntry(const TVTimerEntry&);
    ~TVTimerEntry();

    unsigned int getID() const {return id.getRValue();}
    void setID(int i) { id = i;}

    void setStartTime(time_t start){start_time = start;}
    time_t getStartTime() const {return start_time.getRValue();}

    void setEndTime(time_t end){end_time = end;}
    time_t getEndTime() const {return end_time.getRValue();}

    void setChannelName(string s){channelName = s;}
    string getChannelName() const {return channelName.getRValue();}

    void setChannelNumber(int i){channelNumber = i;}
    int getChannelNumber() const {return channelNumber.getRValue();}

    void setStatus(status_e stat){status = stat;}
    unsigned int getStatus() const {return status.getRValue();}

    void setConflicts(bool b){conflicts = b;}
    bool hasConflicts() const {return conflicts.getRValue();}

    void setPeriod(repmode_e r){period = r;}
    unsigned int getPeriod() const {return period.getRValue();}

    void setTimerInfo(ProgramDescription pd){ info = pd; }
    ProgramDescription getTimerInfo() const {return info.getRValue();}

    void setNextTimers(list<ProgramDescription> l) {nextTimers = l;}
    void addNextTimer(ProgramDescription pd);
    void clearNextTimers() {nextTimers.getRValue().clear();}
    list<ProgramDescription> getNextTimers() const {return nextTimers.getRValue();}
    
    void addTVCard(TVCardDescription tvd);
    void addTVCard(string type, TSAddress adr){tvcard.getRValue().push_back(TVCardDescription(type, adr));}
    TVCardDescription getTVCard() const ;
    list<TVCardDescription>& getTVCards() {return tvcard.getRValue();}
    void resetTVCards(){tvcard.getRValue().clear();}

    void setUserName(string u){user = u;}
    string getUserName() const {return user.getRValue();}

    void setFileURL(string url){fileURL = url;}
    string getFileURL() const {return fileURL.getRValue();}
    
    list<int> getConflictingTimers() {return conflictingTimers.getRValue();}
    void addConflict(unsigned int id);
    void removeConflict(unsigned int id);
    bool hasConflictWith(unsigned int id);

    /**
     * verifies the timer. This functions cheks, if the start-time is smaller than the end-time
     */
    Result verify();

    TVTimerEntry* copy();

    bool operator<(const TVTimerEntry &t) const;
  private:
    void registerValues();
  }; //end class TvTimerEntry

  OValueStream& operator<<(OValueStream& os, const list<TVCardDescription>& td);
  IValueStream& operator>>(IValueStream& is, list<TVCardDescription>& td);

  OValueStream& operator<<(OValueStream& os, const list<TVTimerEntry>& te);
  IValueStream& operator>>(IValueStream& is, list<TVTimerEntry>& te);

}//end namespace
#endif
