/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_RECORDNODE_HPP_
#define _NMM_RECORDNODE_HPP_

#include "nmm/base/sync/GenericSyncSourceNode.hpp"

#include "nmm/interfaces/file/IBufferSizeImpl.hpp"
#include "nmm/interfaces/device/IDeviceImpl.hpp"

#include "nmm/base/registry/DevicePlugin.hpp"

namespace NMM {
   class StreamTimer;
  /**
   * This class records an audio stream from a given input device.
   * So far, the formats are hardwired and we already know what the
   * sound device is capable of.
   */
  class RecordNode :
    public GenericSyncSourceNode,
    public IDeviceImpl,
    public IBufferSizeImpl
  {
  private:
    StreamTimer* timer;
    int sample_rate;        ///< The sample rate to record with
    int num_channels;       ///< The number of channels to record with
    int bits_per_sample;    ///< The number of bits per sample to record with
    
    string dev;             ///< The name of the sound device.
    int audio_fd;           ///< File descriptor number of the sound device.
    
    int buffer_size;         ///< Size of the buffers.
    
  protected:
    /**
     * Opens the recording device.
     */
    virtual Result doInit();
    virtual Result doDeinit();
    
    /**
     * Setup working output property.
     */
    virtual Result doInitOutput();
    
    /**
     * Extracts format.
     */
    virtual Result doActivate();
    
    /**
     * The produceBuffer method from GenericSyncSourceNode.
     */
    virtual Message* produceBuffer(Buffer*);

    // virtual functions inherited from GenericSyncSourceNode
    virtual Time getLocalTime(u_int64_t);
    
  public:
    
    /**
     * The main constructor. Can take a BufferManager to use instead
     * of creating its own Allocator.
     * \param dev the device to open.
     * \param manager the buffer manager to use.
     */
    RecordNode(const char* name = "RecordNode",
	       StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
	       int = StreamQueue::DEFAULT_SIZE);
    virtual ~RecordNode();
    
    /**
     * IDevice interface.
     */
    virtual void setDevice(const string& value)
      throw(const NMM::DeviceNotFoundException&);
    virtual string getDevice() const;
    
    /**
     * IBufferSize interface.
     */
    virtual void setBufferSize(const int& size);
    virtual int getBufferSize();
  };
  
  class RecordNodePlugin
    : public TDevicePlugin<RecordNode>
  {
  public:
    RecordNodePlugin(const string& node_name, int max_instance = -1);
  };
  
}

#endif //_NMM_RECORDNODE_HPP_
