/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_OGGVORBISDECODENODE_HPP
#define NMM_OGGVORBISDECODENODE_HPP NMM_OGGVORBISDECODENODE_HPP

#include "nmm/base/GenericProcessorNode.hpp"
#include "nmm/interfaces/general/ITrackSizeImpl.hpp"
#include "nmm/interfaces/base/sync/ISyncResetImpl.hpp"
#include "nmm/interfaces/audio/oggvorbis/IOggVorbisImpl.hpp"
#include "nmm/interfaces/audio/oggvorbis/IOggVorbisImpl.hpp"

#include <vorbis/vorbisfile.h>

namespace NMM {

  class OggVorbisDecodeNode
    : public ISyncResetImpl,
      public IOggVorbisImpl,
      public ITrackSizeImpl,
      public GenericProcessorNode {
  private:
    Buffer* read_buffer;
    size_t bytes_read;
    bool ov_initialized;
    Buffer* merge_buffer;
    OggVorbis_File oggvorbis;
    size_t out_buffer_size;
    string info_user_comments;
    string info_vendor;
    Timestamp timestamp;
    Timestamp in_timestamp;
    int64_t old_pos;
    int sample_rate;
    int channels;
    string title,album,artist,year,genre;
    u_int64_t track_size;
    bool send_track_duration;

    size_t readFunc(void*, size_t, size_t);
    static size_t readFuncOggStruct(void* ptr, size_t size, size_t nmemb, void* datasource) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->readFunc(ptr, size, nmemb); 
    }

    int seekFunc(ogg_int64_t, int);
    static int seekFuncOggStruct(void* datasource, ogg_int64_t offset, int whence) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->seekFunc(offset, whence); 
    }

    int closeFunc();
    static int closeFuncOggStruct(void* datasource) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->closeFunc(); 
    }

    long int tellFunc();
    static long int tellFuncOggStruct(void* datasource) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->tellFunc(); 
    }

    bool initOggVorbis();

  protected:

    Result doDeactivate();
    Result doInit();
    Result doInitOutput();
    
    //from ISyncResetImpl
    virtual Result syncReset();
    
    //from IOggVorbisImpl
    string getUserComments();
    string getVendor();

    //from ITrackSizeImpl
    Result trackSize(const u_int64_t& bytes);
    
   public:
    OggVorbisDecodeNode(const char* = "OggVorbisDecodeNode", 
			StreamQueue::Mode = StreamQueue::MODE_SUSPEND, 
			int = 20);

    ~OggVorbisDecodeNode();

    Message* processBuffer(Buffer*);

  };

}

#endif //NMM_OGGVORBISDECODENODE_HPP



