/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:	      
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_AVDEMUXNODE_HPP
#define NMM_AVDEMUXNODE_HPP NMM_AVDEMUXNODE_HPP

#include "nmm/interfaces/av/ISetBufferLabelImpl.hpp"
#include "nmm/base/GenericDemultiplexerNode.hpp"

namespace NMM {
  
  /**
   * Audio/Video Demultiplexer.
   * This node receives muxed audio and video buffers and events and sends each to the proper
   * output stream ("video" or "audio"). Labels are used to tell to which output stream a
   * message belongs. These labels are defined by the predecessor node by sending setBufferLabel events.
   */
  class AVDemuxNode :
    public GenericDemultiplexerNode,
    public ISetBufferLabelImpl
  {
  private:
    int buf_count;
    unsigned int labelnum_video;
    unsigned int labelnum_audio;
    
    // ISetBufferLabel interface
    virtual Result setBufferLabel(const string& label_name, const int& label_num);

  protected:
    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    
  public:

    AVDemuxNode(const char *name = "AVDemuxNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int = 20);
    ~AVDemuxNode() {};

    virtual Message *processBuffer (Buffer *);
    
    virtual Result processCEvent (CEvent *);
  };

}

#endif //NMM_AVDEMUXNODE_HPP



