/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:	      Roger Dostert <djurig@graphics.cs.uni-sb.de 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_FFMPEGMUXNODE_HPP
#define NMM_FFMPEGMUXNODE_HPP NMM_FFMPEGMUXNODE_HPP


#include "nmm/base/GenericMultiplexerNode.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"


namespace NMM {

  class FFAVFormat;

  /**
   * @class FFMpegMuxNode
   * @brief this class implements a Audio-Video-Multiplexer
   * it uses the libavformat from the FFMpeg package
   * the multiplexer can handle one video and one audio stream for input
   */
  class FFMpegMuxNode : public GenericMultiplexerNode,
                        public ITrackImpl {
  public:
    /**
     * @brief constructor
     */
    FFMpegMuxNode(const char* name="FFMpegMuxNode",
		  StreamQueue::Mode qmode= StreamQueue::MODE_SUSPEND,
		  int qsize = StreamQueue::DEFAULT_SIZE);
    /**
     * @brief destructor
     */
    ~FFMpegMuxNode(void);

  protected:
    // INode-interface
    virtual Result doInit(void);
    virtual Result doDeinit(void);
    virtual Result doInitOutput(void);
    virtual Result doDeinitOutput(void);
    virtual Result doActivate(void);
    virtual Result doDeactivate(void);
    virtual Result doStart(void);
    virtual Result doStop(void);
    virtual Message* processBuffer(Buffer* inbuffer);

    // ITrackImpl
    virtual Result startTrack(const std::string& filename);
    virtual Result endTrack(void);
    virtual Result finished(void);

  private:
    FFAVFormat* ffAVFormat; 				///! all the FFMpeg stuff we needed

    bool firstFrame;                ///! top of stream
    bool lastFrame;                 ///! end of stream
    int  receivedStartTrack;        ///! counter for received startTrack events
    int  receivedEndTrack;          ///! counter for received endTrack events
  }; // class FFMpegMuxNode

}; // namespace NMM

#endif // NMM_FFMPEGMUXNODE_HPP
