/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *                     
 *                     
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_GENERICREADNODE_HPP
#define NMM_GENERICREADNODE_HPP NMM_GENERICREADNODE_HPP

#include "nmm/base/GenericSourceNode.hpp"

#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/file/IBufferSizeImpl.hpp"
#include "nmm/interfaces/file/ISeekableImpl.hpp"
#include "nmm/interfaces/general/progress/IProgressImpl.hpp"

namespace NMM {

  class GenericReadNode :
    public GenericSourceNode,
    public IFileHandlerImpl,
    public ISeekableImpl,
    public IBufferSizeImpl,
    public IProgressImpl
  {

  private:
    bool sendEvent; //status flag whether to send an event
    bool send_format;
    bool send_progress; //status flag whether to send progress information
    string filename;
    int             buffers_before_progress;
    int             buffers_progress;

  protected:
    ifstream in_fstream;
    size_t buffer_size;

    // seeking & file position
    u_int64_t         filesize;
    u_int64_t         filepos;
    int64_t          seekto;

    virtual Message* processBuffer(Buffer *);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    
    // ISeekable interface
    virtual bool hasPercentSeek() const;
    virtual void seekPercent(const Rational& percent);
    virtual void seekPercentTo(const Rational& percent);
    virtual bool hasBytesSeek() const;
    virtual void seekBytes(const int64_t& offset);
    virtual void seekBytesTo(const u_int64_t& bytes);
    virtual bool hasTimeSeek() const;
    virtual void seekTime(const Interval& interval);
    virtual void seekTimeTo(const Time& time);
    
    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // IBufferSize interface
    virtual void setBufferSize(const int& size);
    virtual int getBufferSize();

    // IProgress interface
    virtual void sendProgressInformation(const bool& in);
    virtual void setProgressInterval(const int& buffers);

  public:
   
    GenericReadNode(const char *name = "GenericReadNode",
		    StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		    int = StreamQueue::DEFAULT_SIZE);
    virtual ~GenericReadNode();

  };

}

#endif //NMM_GENERICREADNODE_HPP
