/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_RAWWRITENODE_HPP
#define NMM_RAWWRITENODE_HPP

#include "nmm/base/GenericSinkNode.hpp"
#include "nmm/interfaces/file/ISinkFileHandlerImpl.hpp"
#include "nmm/interfaces/file/IFileModeHandlerImpl.hpp"

namespace NMM {

  /**
   * RawWriteNode 
   */
  class RawWriteNode : public GenericSinkNode,
		       public ISinkFileHandlerImpl,
		       public IFileModeHandlerImpl{
  protected:
    string      file_name;
    ofstream file_stream;
    int         buf_count;
    ThreadMutex file_name_mutex;
    
    IFileModeHandler::FileMode     write_mode; 
    IFileModeHandler::FileFormat   file_format;
    IFileModeHandler::FilenameMode filename_mode;

    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Buffer *processBuffer(Buffer *buffer);
    virtual Result doDeinitOutput();

    // ISinkFileHandler interface
    virtual void setWorkingDirectory(const string& directory);
    virtual string getWorkingDirectory();
    virtual void setFileExtension(const string& extension);
    virtual string getFileExtension();

    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // IFileModeHandler interface
    virtual void setMode(const IFileModeHandler::FileMode& mode);
    virtual IFileModeHandler::FileMode getMode();
    virtual void setFormat(const IFileModeHandler::FileFormat& format);
    virtual IFileModeHandler::FileFormat getFormat();
    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode);
    virtual IFileModeHandler::FilenameMode getFilenameMode();
    virtual void setLoop(const bool& d);
    
  public:
    RawWriteNode(const char *name = "RawWriteNode", 
		 const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
		 const int queue_size = 1);
    ~RawWriteNode();

  };
  
}

#endif
