/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_BUFFERDROPNODE_HPP_
#define _NMM_BUFFERDROPNODE_HPP_

#include "nmm/interfaces/general/IBufferDropImpl.hpp"
#include "nmm/base/GenericFilterNode.hpp"

namespace NMM {

  /**
   * \class BufferDropNode
   */
  class BufferDropNode :
    public GenericFilterNode,
    public IBufferDropImpl
  {
  private:
    unsigned int buffer_to_drop;
    unsigned int buffer_count;
    
    // for setDropRatio
    unsigned int drop_num;
    unsigned int drop_denom;
    Rational drop_ratio;
    Rational ratio_counter;

  protected:
    /**
     * ICopy interface.
     * Set the number of outputs.
     */
    void setBufferToDrop(const unsigned int &buffer_num);
    
    // IBufferDrop Interface
    void setDropRatio(const unsigned int& nominator, const unsigned int& denominator);

    /**
     * Here all formats are specified.
     */
    virtual Result doInit();
    
    /**
     * Here all output streams are added.
     */
    virtual Result doInitOutput();

    /**
     * The processBuffer method from superclass
     */
    virtual Message *processBuffer (Buffer *in);

  public:

    /**
     * The main constructor.
     */
    BufferDropNode(const char *name = "BufferDropNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int qsize = 5);
  };

}

#endif //_NMM_VIDEO_FILTER_OVERLAYNODE_HPP_



