/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "Plugin1394.hpp"


namespace NMM {


void Plugin1394::nmmInit()
   throw (const NMM::Exception&)
  {
    if (!L1394::SSession::getInstance()) {
      throw NMM::Exception("Could not get the L1394 Session object. Check if ieee1394 driver are loaded and try to load the plugin again.");
    }
  }

  void Plugin1394::nmmFini() {

  }

  void Plugin1394::nmmRegistryFini(NMM::Registry& ){
    delete __registry1394;
    __registry1394 = 0;
  }

  void Plugin1394::nmmRegistryInit(NMM::Registry& registry)
    throw (const NMM::Exception&)
  {
    if (!__registry1394)
      __registry1394 = new NMM::Registry1394(&registry);
  }


  void Plugin1394::nmmFactoryInit(NMM::NodeFactory& factory)
    throw (const NMM::Exception&)
  {
    if (!__factory1394)
      __factory1394 = new NMM::Factory1394(&factory);
  }

  void Plugin1394::nmmFactoryFini(NMM::NodeFactory& ) {
    delete __factory1394;
    __factory1394 = 0;
  }
};
extern "C" {

static NMM::Plugin1394 plugin1394;

NMM::Plugin* nmm_plugins[2] = {&plugin1394, 0};

}
