/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Eric Peters <epeters@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_NETSOURCENODE_HPP
#define NMM_NETSOURCENODE_HPP

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/base/TransportStrategy.hpp"
#include "nmm/base/ISignalReceiver.hpp"
#include "nmm/base/StreamQueue.hpp"
#include "nmm/base/proxy/rtp/RTPStrategy.hpp"
#include "nmm/interfaces/net/INetImpl.hpp"
#include "nmm/interfaces/base/IQueued.hpp"

#include <list>
using std::list;

namespace NMM {
  
  class NetSourceNode :
    public GenericSourceNode,
    public INetImpl,
    public IMessageReceiver
  {
  public: 
    
    NetSourceNode(const char* = "NetSourceNode",
		  IQueued::Mode queue_mode = IQueued::MODE_SUSPEND,
		  unsigned int queue_size = 100);
    virtual ~NetSourceNode();
    
    Result enableOutputStream();
    Result disableOutputStream();
  
  protected:
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    /**
     *  @name INet Interface
     */
    //@{

    /**
     *  @brief TSAddress for connecting
     *  @param tsaddress address to connect
     *  
     *  This method sets the @a tsaddress for the NetSinkNode. The type of 
     *  TSAddress (eg. UDPAddress, TCPAddress or RTPAddress) defines the type 
     *  of protocol to use (eg. UDP, TCP or RTP).
     */
    virtual void setTransportStrategy(const TSAddress*) throw(const BindingException&);
    
    virtual void setRTPStrategy(const string&, const int&, const int&);

    virtual void setPayloadType(const int& /*type*/);

    virtual int  getPayloadType() const;
    //@}

    virtual Result receiveMessage(Message*, const IMessageSender*);
    
  private:
    void setTransportStrategy(TransportStrategy*);

    TSAddress*            __address;
    TransportStrategy*    __transport_strategy;

    Message*              __current_message;
    ThreadMutex           __current_message_mutex;
    
    StreamQueue           __stream_queue;
    
    int                   __payload_type;
    int                   __audio_channels;
    int                   __audio_bit_per_sample;
    int                   __audio_sample_rate;
  };


} //end namepsace

#endif
