/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "RawNode.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/memorymgr/Buffer.hpp"
#include "nmm/base/registry/Plugin.hpp"

NMM_REGISTER_PLUGIN(NMM::RawNode, "RawNode")

namespace NMM {

  RawNode::RawNode(const char* name /* = "RawNode" */,
		   StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
		   int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size) {
    buffers_passed = 0L;
    upstream_passed = 0L;
    buffer_size = 1024;
  }
  
  int RawNode::getBuffersPassed() const {
    return buffers_passed;
  }
  
  void RawNode::resetBufferCounter() {
    buffers_passed = 0;
  }
  
  void RawNode::setBufferSize(const int& value) {
    buffer_size = value;
  }
  
  int RawNode::getBufferSize() {
    return buffer_size;
  }
  
  Result RawNode::doInit() {
    Format* format = getOwnOutputProperty()->addNewFormat("*/*");
    getOwnOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Result RawNode::doInitOutput() {
    Format* format = getOwnWorkingOutputProperty()->addNewFormat("*/*");
    getOwnWorkingOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Message* RawNode::processBuffer(Buffer *) {
    Buffer *out_buffer = getNewBuffer(buffer_size);
    buffers_passed++;
    return out_buffer;
  }
  
  Result RawNode::buffersPassed(const int& value) {
    upstream_passed += value;
    return SUCCESS;
  }
  
  int RawNode::getUpstreamBuffersPassed() const {
    return upstream_passed;
  }
  
  void RawNode::resetUpstreamBufferCounter() {
    upstream_passed = 0;
  }
}
