/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_GLDISPLAYNODE_HPP_
#define _NMM_GLDISPLAYNODE_HPP_

#include "nmm/utils/Benchmark.hpp"
#include "nmm/base/sync/GenericUSyncSinkNode.hpp"
#include "nmm/interfaces/video/display/IVideoDisplayImpl.hpp"
#include "nmm/interfaces/video/display/IDisplayEventSenderImpl.hpp"

#include "nmm/misc/gui/XVirtualImg.hpp"


namespace NMM {

  class GLDisplayNode : public GenericUSyncSinkNode,
			public IVideoDisplayImpl,
			public IDisplayEventSenderImpl {
    
  private:
    void createGLContext();

    bool rgb_channel_order;

    Buffer* working_buffer;

    int title_update_rate;
    ThreadMutex display_size_mutex;

    string disp_ev_config_file;
    bool enable_disp_ev;

  protected:

    XVirtualImg* display;
    bool is_GL_context_valid;
    int width, height; // the dimensions of the image to be displayed
    int display_width, display_height; // the real dimensions of the display
                                       // evtl. changed (resized) by the user

    Benchmark benchmark;

  public:

    GLDisplayNode(const char *name = "GLDisplaynode",
                  const StreamQueue::Mode queue_mode =
                  StreamQueue::DEFAULT_MODE,
                  const int size = 50);
    virtual ~GLDisplayNode();

    /**
     * redraws the last buffer (e.g. if window was resized, moved, ...)
     */
    Result redraw();

  protected:
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    virtual Result doDeactivate();
    
    virtual void prepareBuffer(Buffer *in_buffer);
    virtual Message* presentBuffer(Buffer *in_buffer);

    /** IDisplayEventSender interface. */
    virtual void enableDisplayEvents(const string& config_file);
    virtual void disableDisplayEvents();
    
    /** IVideoDisplay **/
    virtual void setFullscreen(const bool&);
    virtual bool getFullscreen() const;
    virtual bool hasFullscreen() const;
    virtual bool isFullscreenOnly() const;
    virtual bool makeScreenshot(const string &) const;
    virtual void setLeftWindowAlignment() {}
    virtual void setCenterWindowAlignment() {}
    virtual void setRightWindowAlignment() {}
    
  };

}

#endif //_NMM_GLDISPLAYNODE_HPP_
