/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_VIDEOGRABNODE_HPP_
#define _NMM_VIDEOGRABNODE_HPP_

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/base/sync/StreamTimer.hpp"
#include "nmm/utils/utils.hpp"
#include "nmm/interfaces/video/ICropImpl.hpp"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

namespace NMM {

  class VideoGrabNode : public GenericSourceNode, public ICropImpl {
  private:
    StreamTimer* stimer;
    
    int image_width;
    int image_height;
    int image_bpp;
    double image_fps;

    int x_offset;
    int y_offset;
    int x_range;
    int y_range;
    
    looptimer       __last_timer;     ///< stores the latest timespamp
    ThreadMutex     __cond_mutex;     ///< ThreadMutex, required for the ThreadCondition
    ThreadCondition __condition;      ///< used to wait until next buffer must be send

    Display *_display;
    Window _root;
    XShmSegmentInfo _shminfo;
    XImage *_ximage;

    void addSupportedFormats(Property* property);
  protected:
    virtual void setCrop(const int& xOffset, const int& yOffset, const int& xRange, const int& yRange);
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    virtual Result doActivate();
    virtual Result doStop();
    void waitToSendBuffer();
    
  public:
    VideoGrabNode(const char* name = "VideoGrabNode",
                   StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
                   int = StreamQueue::DEFAULT_SIZE);
    
    virtual ~VideoGrabNode();
  };
}

#endif //_NMM_WHITENOISE_HPP_

