/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_XVSHMIMG_HPP
#define NMM_XVSHMIMG_HPP NMM_XVSHMIMG_HPP

#include "XShmManager.hpp"

// must be after the shm-includes as otherwise,
// the XvShm-Stuff is not correctly declared
#include "XvImg.hpp"

namespace NMM {

  /**
   * This is a class for writing video frames to a X Display using
   * XVideo extension in combination with XShm extension.
   */
  class XvShmImg 
    : public XvImg {

  protected:
    //! the X shared memory manager for this class
    XShmManager shm;

    // reimplementation of virtual methods of XvImg
    virtual bool initImage();
    virtual void deinitImage();

  public:

    /**
     * Constructor:
     * \param parent Node* to the DisplayNode, this Img is running in
     * \param width The width of the video frames, to be presented
     * \param height The width of the video frames, to be presented
     * \param title The title used for the X-Window to be created
     * \param format The XVideo format description of the video frames
     *               to be presented
     * \param fullscreen Flag to indicate, that the window should be
     *                   opened directly in fullscreen mode
     * \param ratio_x The x-component of the aspect ratio to use for
     *                presenting the video frames
     * \param ratio_y The y-component of the aspect ratio to use for
     *                presenting the video frames
     * \param manager A pointer to a ShmBufferManager or NULL if no
     *                ShmBufferManager should be used
     * \param pages The ammount of shared memory pages
     *              (with size with * height), which the shared memory
     *              manager should allocate/attach.
     */
    XvShmImg(Node* parent,
             int width, int height,
             string title,
	     unsigned int format, bool fullscreen,
	     int ratio_x, int ratio_y,
	     ShmBufferManager* manager, unsigned int shared_pages);

    /**
     * Destructor
     */
    virtual ~XvShmImg();

    // reimplementation of virtual method of XVirtualImg to replace
    // its dummy implementation
    void updateMemoryManager(ShmBufferManager*, unsigned int);

    // reimplementation of virtual methods of XvImg
    virtual void prepareOutput( Buffer* in_buffer);
    virtual void update( Buffer* in_buffer );
  };

}

#endif //NMM_XVSHMIMG_HPP
