/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_VIDEO_FILTER_OVERLAYNODE_HPP_
#define _NMM_VIDEO_FILTER_OVERLAYNODE_HPP_

#include "nmm/interfaces/video/filter/IOverlayImpl.hpp"
#include "nmm/interfaces/video/IStillFrameImpl.hpp"
#include "nmm/interfaces/base/sync/ISyncResetImpl.hpp"
#include "nmm/base/GenericMultiplexerNode.hpp"

#define RGBA32 0
#define YV12 1

namespace NMM {

  /**
   * This class takes two (RGBA32) video streams and overlays the foreground
   * onto the background. The output format is that of the background stream.
   * 
   * This Node has two input streams called "foreground" and "background"
   */
  class OverlayNode :
    public GenericMultiplexerNode,
    protected IOverlayImpl,
    protected ISyncResetImpl,
    protected IStillFrameImpl
  {
  private:
    int foreground_width;
    int foreground_height;
    int new_foreground_width;
    int new_foreground_height;
    
    unsigned int new_segment_width;
    unsigned int new_segment_height;
    unsigned int new_segment_x_offset;
    unsigned int new_segment_y_offset;
    
    unsigned int segment_width;
    unsigned int segment_height;
    unsigned int segment_x_offset;
    unsigned int segment_y_offset;
    
    int background_width;
    int background_height;
    int x_offset;
    int y_offset;
    int new_x_offset;
    int new_y_offset;
    int alpha; // 0..256, "fixed point value"
    int color;
    
    Buffer *saved_in_buffer;
    Buffer *last_foreground;
    Buffer *last_background;
    Buffer *saved_background;
    
    bool send_format;
    bool save_next_background;
    bool have_still_frame;
    bool save_still_frame;
    
    Format* in_b_rgba_format;
    Format* in_b_yv12_format;
    
    Format* out_rgba_format;
    Format* out_yv12_format;
    
    /**
     * Sets a pixel on the buffer image.
     */
    void setPixel(unsigned char *image, int x, int y,
    		  unsigned char r, unsigned char g, unsigned char b);
    
    
    /**
     * Sets a Y-pixel on a YV12 Plane
     */
    void setPixel_y(unsigned char *image, int x, int y,
                    unsigned char v);
    
  protected:
    /**
     * IOverlay interface.
     */
    virtual void setOffset(const int& x, const int& y);
    virtual void getOffset(int& x, int& y) const;
    virtual void setAlpha(const float& alpha);
    virtual float getAlpha() const;
    
    /**
     * ISync interface.
     */
    virtual Result syncReset();
    
    /**
     * IStillFrame interface.
     */
    virtual Result setStillFrame(const bool& value);
    
    /**
     * IFormatChange interface.
     */
    virtual Result setFormat( const Format& format );
    
    /**
     * IOverlay interface.
     */
    virtual void setForeground( const int& x, const int& y, const int& w, const int& h );
    virtual void setForegroundSegment( const int& x, const int& y, const int& w, const int& h );
    virtual Result disableOverlay();
    
    virtual Message *processBuffer(Buffer *in_buffer);
    
    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    virtual Result doActivate();
    
  public:

    /**
     * The default constructor. 
     */
    OverlayNode(const char *name = "OverlayNode", 
		const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE, 
		const int queue_size = 1);
    ~OverlayNode();
  };
}

#endif //_NMM_VIDEO_FILTER_OVERLAYNODE_HPP_
