/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:  David Maass (dama@graphics.cs.uni-sb.de)       
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "OSDElementComposit.hpp"

namespace NMM {
  OSDElementComposit::~OSDElementComposit() {
  }

  Result OSDElementComposit::selectUp() {
    if (focus>=elements.size())
      return FAILURE;
    return elements[focus]->selectUp();
  }

  Result OSDElementComposit::selectDown() {
    if (focus>=elements.size())
      return FAILURE;
    return elements[focus]->selectDown();
  }    

  int OSDElementComposit::addElement(OSDElementBase* elem) {
    elements.push_back(elem);
    return (elements.size()-1);
  }

  void OSDElementComposit::removeElement(OSDElementBase* elem) {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
      if ((*pos)==elem) {
        delete elem;
        (*pos)=NULL;
        break;
      }
    }
  }

  void OSDElementComposit::removeElement(int nr) {
    delete elements[nr];
    elements[nr]=NULL;
  }

  void OSDElementComposit::update() {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
      GLOBAL_DEBUG_STREAM("Updating OSDElement "<<(*pos)<<". (do not remove this line!)"<<endl);
      (*pos)->update();
    }
    manager->update();
  }
  
  void OSDElementComposit::show() {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
      (*pos)->show();
    }
    hidden=false;
  }
  
  void OSDElementComposit::hide() {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
    (*pos)->hide();
    }
    manager->hideSymbol();
    hidden=true;
  }
}
