/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "colorbarunit.hpp"

namespace NMM {
  ColorBarUnit::ColorBarUnit( IOSDManager* manager, IWidgetFactory* factory, string _font )
    : OSDElement(manager, factory), font(_font) {
    int width = 560;
    win->setDimension(width, 30);

    composite.reset(factory->createWidgetComposite());

    text[0].reset(factory->createTextView());
    text[0]->setFont(font);
    text[0]->addLine("red");
    text[1].reset(factory->createTextView());
    text[1]->setFont(font);
    text[1]->addLine("green");
    text[2].reset(factory->createTextView());
    text[2]->setFont(font);
    text[2]->addLine("yellow");
    text[3].reset(factory->createTextView());
    text[3]->setFont(font);
    text[3]->addLine("blue");

    plane[0].reset(factory->createPlaneDecorator(*text[0]));
    plane[0]->setPlaneColor(0xA00000FF);
    plane[0]->setDimension(width/4, 30);
    plane[0]->setPosition(0, 0);
    plane[1].reset(factory->createPlaneDecorator(*text[1]));
    plane[1]->setPlaneColor(0xA000FF00);
    plane[1]->setDimension(width/2 - width/4, 30);
    plane[1]->setPosition(width/4, 0);
    plane[2].reset(factory->createPlaneDecorator(*text[2]));
    plane[2]->setPlaneColor(0xA000FFFF);
    plane[2]->setDimension(3*width/4 - width/2, 30);
    plane[2]->setPosition(width/2, 0);
    plane[3].reset(factory->createPlaneDecorator(*text[3]));
    plane[3]->setPlaneColor(0xA0FF0000);
    plane[3]->setDimension(width - 3*width/4, 30);
    plane[3]->setPosition(3*width/4, 0);

    text[0]->setPosition(5,5);
    text[1]->setPosition(155,5);
    text[2]->setPosition(305,5);
    text[3]->setPosition(455,5);

    composite->add(*plane[0]);
    composite->add(*plane[1]);
    composite->add(*plane[2]);
    composite->add(*plane[3]);

    win->setContent(*composite);
  }

  void ColorBarUnit::setText(const string& red, const string& green, const string& yellow, const string& blue) {
    setText(0, red);
    setText(1, green);
    setText(2, yellow);
    setText(3, blue);

    win->update();
  }

  void ColorBarUnit::setRedText(const string& t) {
    setText(0, t);
    text[0]->changeLineContent(1, t);
    win->update();
  }

  void ColorBarUnit::setGreenText(const string& t) {
    setText(1, t);
    text[1]->changeLineContent(1, t);
    win->update();
  }

  void ColorBarUnit::setYellowText(const string& t) {
    setText(2, t);
    text[2]->changeLineContent(1, t);
    win->update();
  }

  void ColorBarUnit::setBlueText(const string& t) {
    setText(3, t);
    win->update();
  }

  string ColorBarUnit::getRedText() {
    return text[0]->getLineContent(1);
  }

  string ColorBarUnit::getGreenText() {
    return text[1]->getLineContent(1);
  }

  string ColorBarUnit::getYellowText() {
    return text[2]->getLineContent(1);
  }

  string ColorBarUnit::getBlueText() {
    return text[3]->getLineContent(1);
  }

  void ColorBarUnit::setText(int i, const string& t) {
    text[i]->changeLineContent(1, t);
    unsigned int x;
    unsigned int y;
    plane[i]->getPosition(x, y);
    if (t.empty()) {
      plane[i]->setPosition(x, 30);
      text[i]->setPosition(x + 5, 35);
    }
    else {
      plane[i]->setPosition(x, 0);
      text[i]->setPosition(x + 5, 5);
    }
  }
  
  ColorBarUnit::~ColorBarUnit() {
  }
}
