/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_TABLEUNIT_HPP_
#define _NMM_TABLEUNIT_HPP_ _NMM_TABLEUNIT_HPP_

#include "OSDElement.hpp"

#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IScrollDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"

namespace NMM {

  typedef list<unsigned int> ColumnList;
  
  class TableUnit : public OSDElement {
  private:  
    ColumnList columns;
    bool scrollbarEnabled;

    void createWindows();
  public:
    
    TableUnit(IOSDManager* manager, IWidgetFactory* factory, string font);
    ~TableUnit();
        
    void setColors( unsigned int win_active_color, unsigned int win_inactive_color,
                    unsigned int sel_active_color, unsigned int sel_active_color2, unsigned int sel_inactive_color );
    
    void selectWindow( unsigned int mode=0 );
    void deSelectWindow();
    void setHeader( string h );
    
    void setColumnWidth( unsigned int c, unsigned int w );
    void addColumn(unsigned int w);
    
    unsigned int getSelectedLine();
    string getLineURL(const unsigned int i);
    void selectLine(const unsigned int& i);

    string getLineContent(const unsigned int c, const unsigned int i);
    list<string> getLineContent(const unsigned int i);
    void setLineContent(const unsigned int c, const unsigned int i, string s);

    void deleteAllLines();
    unsigned int addLine(list<string> c,string url);
    unsigned int addLine(list<string> c);
    unsigned int addLine(string c, string url);
    unsigned int addLine(string c);
    void deSelectLine();
    Result selectUp();
    Result selectDown();

    void setScrollbarEnabled(bool enabled);
    bool getScrollbarEnabled(){return scrollbarEnabled;}

    int getNumLines();
  protected:

    unsigned int  win_active_color;
    unsigned int  win_inactive_color;
    unsigned int  sel_active_color;
    unsigned int  sel_active_color2;
    unsigned int  sel_inactive_color;
    
    void deleteWidgets();
    
    string            font;
    
    list<ITextView*>     text;

    IWidgetComposite_var  composite;

    IPlaneDecorator_var   plane;
    IBorderDecorator_var  border;
    IScrollDecorator_var  scroll;
    IHeaderDecorator_var  header;
  };

}

#endif //_NMM_FLAGLISTUNIT_HPP_
