/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_TEXTINPUT_HPP_
#define _NMM_TEXTINPUT_HPP_ _NMM_TEXTINPUT_HPP_

#include "OSDElement.hpp"

#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"

#include "nmm/base/sync/Clock.hpp"

namespace NMM{
  
  class TextInput : public OSDElement {
  private:

    vector<string> keyChars;

    /* keys for character-input */
    Result key0();
    Result key1();
    Result key2();
    Result key3();
    Result key4();
    Result key5();
    Result key6();
    Result key7();
    Result key8();
    Result key9();

    int currentKey;
    int currentChar;
    
    Clock clock;
    Time timeout;
    Time maxTimeWait;

    bool withinTime;

    string url;
  public:

    enum mode_e {ALPHA=0, NUMERIC};
    
    TextInput(IOSDManager* manager, IWidgetFactory* factory, string font);
    ~TextInput();

    void setColors( unsigned int win_active_color, unsigned int win_inactive_color,
                    unsigned int sel_active_color, unsigned int sel_active_color2, unsigned int sel_inactive_color );
    
    void selectWindow( unsigned int mode=0 );
    void deSelectWindow();
    void setHeader( string h );
    string getContent();

    /*add a string to the Text-Box */
    void addString(string s);
    /*clears the text */
    void clear();
    /*erases the last char */
    void erase();

    void setMode(mode_e m);

    void setLineURL(string s){url = s;}
    string getLineURL(){return url;}

    Result keyPressed(int k);
    void setNewKeyCharMapping(vector<string> v);
    vector<string> getKeyCharMapping();
  protected:
    mode_e mode;

    unsigned int  win_active_color;
    unsigned int  win_inactive_color;
    unsigned int  sel_active_color;
    unsigned int  sel_active_color2;
    unsigned int  sel_inactive_color;
    
    void deleteWidgets();
    
    string            font;
    
    ITextView_var     text;
    
    IPlaneDecorator_var   plane;
    IBorderDecorator_var  border;
    IHeaderDecorator_var  header;

  };

}
#endif
