/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "decorators.hpp"
#include "../OSDManagerNode.hpp"

namespace NMM {

  Decorator::Decorator() : vcomponent (NULL) {
  }
  
  void Decorator::setOwnInternalWidget( Widget* vc ) {
    vcomponent = vc;
    
    /* set position to position of the vc object */
    vc->getPosition( xpos, ypos );
    
    /* set dimension to the dimension of the vc object */
    vc->getDimension( width, height );
  }
  
  Widget* Decorator::getOwnInternalWidget() const {
    return vcomponent;
  }
  
  IWidget* Decorator::getInternalWidget() const {
    if (vcomponent == NULL) { return NULL; }
    else { return vcomponent->getCheckedInterface<IWidget>(); }
  }
  
  void Decorator::setInternalWidget(const IWidget& widget) {
    setOwnInternalWidget(manager->getOwnWidget(widget));
  }
  
  void Decorator::setPosition( const unsigned int& x, const unsigned int& y ) {
    xpos = x;
    ypos = y;
    
    /* pass through */
    vcomponent->setPosition( x, y );
  }
  
  void Decorator::setDimension( const unsigned int& w, const unsigned int& h ) {
    width  = w;
    height = h;
    
    /* pass through */
    vcomponent->setDimension( w, h );
  }
  
  void Decorator::draw( workspace_t* ws ) {
    vcomponent->draw( ws );
  }
}
