
/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "progressbar.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* ProgressBar									         */
  /*****************************************************************************************/
  ProgressBar::ProgressBar() {
    setMaxProgress( 100 );
    setProgress( 0 );
    setBarColor( 0xff808080 );
    setBarSpaces( 4, 16 );
  }

  ProgressBar::~ProgressBar() {
  }
  
  void ProgressBar::drawBar(workspace_t* ws) {
    // pointer to workspace (destination)
    // we work on the data interpreted as integers, representing the pixel-color
    unsigned int* frame = reinterpret_cast<unsigned int*>(ws->data);
    
    // adjust width and height so that they don't extend the workspace dimension
    unsigned int v_width  = adjustDimension(xpos, width, ws->width);
    unsigned int v_height = adjustDimension(ypos, height, ws->height);
    
    // compute the percentual fill
    Rational rat_width = Rational(v_width) * Rational(progress, max);
    v_width = rat_width.getNumerator() / rat_width.getDenominator();
    // a method Rational::getIntegerPart() would be handy here!
    
    // draw bar
    // for all horizontal lines
    for (unsigned int y = ypos;
         y < (ypos+v_height);
         y++) {
      // sc stores our position in one bar-space-group
      unsigned int sc = 0;
      // for each pixel from 0 to v_width(=percentual fill)
      for (unsigned int x = xpos;
           x < (xpos+v_width);
           x++, sc++) {
	if (sc >= space_step+space) {
          // if we reached the end of our bar-space-group
          // we reset the according position
          sc = 0;
        }
	if (sc < space_step) {
          // if the position in out bar-space-group indicates, that we
          // are still in the "bar", draw the pixel with the bar-color
          frame[x+(y*ws->stride)]  = barcolor;
        }
      }
    }
  }

  void ProgressBar::draw( workspace_t* ws ) {
    drawBar( ws );
  }
  
  void ProgressBar::setMaxProgress(const u_int64_t& value) {
    max = value;
    if ( max < progress ) setProgress( max );
  }
  
  void ProgressBar::setProgress(const u_int64_t& value) {
    /* check if progress is valid */
    if ( progress<=max ) progress = value;
    else progress = 0;
  }
  
  void ProgressBar::setBarColor(const u_int32_t& value) {
    barcolor = value;
  }
  
  void ProgressBar::setBarSpaces(const unsigned int& space, const unsigned int& space_step) {
    this->space      = space;
    this->space_step = space_step;
  }
}
