/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WGTEXTVIEW_HPP_
#define _NMM_WGTEXTVIEW_HPP_ _NMM_WGTEXTVIEW_HPP_

#include "widgets.hpp"
#include "textdraw.hpp"

#include "nmm/interfaces/video/osd/manager/ITextViewImpl.hpp"

namespace NMM {
  
  /**
   * Text view widget.
   */
  class TextView : virtual public Widget,
                   virtual public ITextViewImpl {
    
  public:
    typedef struct {
      string text;
      string url;
      void* common;
    } content_t;
    
    typedef vector<content_t> line_t;
    
  protected:
    /** font that is used to draw the text */
    BitmapFont*    font;
    
    /** class that contains some functions to draw text */
    TextDraw       textdraw;
    
    /** list of lines */
    line_t         lines;
    
    /** selected line (0 if no line is selected) */
    unsigned int   selected_line;
    
    /** selected second line (for highlight) */
    unsigned int   selected_line2;
    
    /** selection color */
    unsigned int   selection_color;
    
    /** selection color2 (for second highlight) */
    unsigned int   selection_color2;
    
    /** first visible line */
    unsigned int   start_line;
    
    /** number of visible lines */
    unsigned int   max_lines;
    
    /** indicates whether a focusSelection() Call should happen when calling selectLine() */
    bool           autofocus;
    
    /** space(pixels) between 2 lines */
    unsigned int   line_space;
    
    list<unsigned int> marked_lines;
  public:
    TextView();
    virtual ~TextView();
    
    /** IScrollable interface. */
    virtual int getScrollBarYPosition() const;
    virtual int getScrollBarHeight() const;
    
    /** ITextView interface. */
    virtual void selectUp();
    virtual void selectDown();
    virtual void setFont(const string& value);
    virtual unsigned int selectLine(const unsigned int& line);
    virtual unsigned int selectLine2(const unsigned int& line);
    virtual void deSelectLine();
    virtual void deSelectLine2();
    virtual unsigned int getSelectedLine() const;
    virtual unsigned int getSelectedLine2() const;
    virtual string getLineContent(const unsigned int& line) const;
    virtual string getLineURL(const unsigned int& line) const;
    virtual IWidget* getLineWidget(const unsigned int& line) const;
    virtual void setAutoFocus(const bool& value);
    virtual void setSelectionColor(const u_int32_t& color);
    virtual void setSelectionColor2(const u_int32_t& color);
    virtual void reset();
    virtual void setMaxLines(const unsigned int& value);
    virtual unsigned int getMaxLines() const;
    virtual unsigned int setStartLine(const unsigned int& start);
    virtual unsigned int getStartLine() const;
    virtual unsigned int getNumLines() const;
    virtual void focusSelection();
    virtual void insertLine(const int& pos, const string& text, const string& url = "");
    virtual int addLine(const string& text, const string& url = "");
    virtual void addLineWidget(const unsigned int& line, const IWidget& widget);
    virtual void deleteLine(const unsigned int& line);
    virtual void deleteAllLines();
    virtual void changeLineContent(const unsigned int& line, const string& text);
    
    virtual void markLine(const unsigned int& line);
    virtual void unmarkLine(const unsigned int& line);
    virtual list<unsigned int> getMarkedLines();

    /**
     * Add internal line.
     * @param con line content
     * @return line number
     */
    int add( content_t con );
    
    /**
     * Insert internal line.
     * @param pos position to insert at
     * @param con line content
     */
    void insert( int pos, content_t con );
    
    /**
     * Get internal line.
     * @param line linenumberListUnit::
     * @return content of that line
     */
    content_t& getLine( unsigned int line );
    
    /**
     * Set space (pixels) between 2 lines.
     * (initial value is taken from font object)
     */
    void setLineSpace( unsigned int ls );
    
    /** Get line space. */
    unsigned int getLineSpace();
    
    void draw( workspace_t* ws );

  }; // TextView

}

#endif //_NMM_WGTEXTVIEW_HPP_
