/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *                     
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "NMMConfig.hpp"
#include "nmm/utils/NamedObject.hpp"

namespace NMM {

  NMMConfig::Setting::Setting(const string& d /*= string("")*/, const string& v /*= string("")*/)
    : desc(d)
  {
    /* extract space seperated values from string and put them into a list */
    bool finish = false;
    unsigned int pos;
    string valuestr = v;
    do {
      pos = valuestr.find_first_of(' ');
      string value;
      if ( pos != string::npos ) {
	value = valuestr.substr( 0, pos );
	valuestr = valuestr.substr( pos+1, valuestr.length() );
      } else {
	value = valuestr;
	finish = true;
      }
    
      values.push_back( value );
    } while( !finish );
  }

  NMMConfig::NMMConfig() : rc_file() {
  } // NMMConfig()

  NMMConfig::NMMConfig(const string filename)  : rc_file() {
    parseFile(filename);
  } // NMMConfig(string)

  NMMConfig::~NMMConfig() {
  } // ~NMMConfig


  bool NMMConfig::getValue(const string var, string& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = config_table[var];
    return true;
  } // getValue (for string)

  bool NMMConfig::getValue(const string var, int& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = atoi(config_table[var].c_str());
    return true;
  } // getValue (for string)

  bool NMMConfig::getValue(const string var, float& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = atof(config_table[var].c_str());
    return true;
  } // getValue (for string)

  bool NMMConfig::getValue(const string var, bool& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = (("true" == config_table[var]) ||
	     ("yes"  == config_table[var]) ||
	     ("1" == config_table[var]));
    return true;
  } // getValue (for string)

  void NMMConfig::setValue( string var, string value ) {
    config_table[var] = value;
  }

  void NMMConfig::saveFile()
  {
    // first, create backup
    string backup_mmboxrc = rc_file + ".backup";
    string create_backup = "mv " + rc_file + " " + backup_mmboxrc;
    GLOBAL_DEBUG_STREAM("Creating backup: " << create_backup << endl);
    if(system(create_backup.c_str()) != 0) {
      GLOBAL_ERROR_STREAM("error creating backup of mmboxrc : " << create_backup << endl);
      return;
    }

    ifstream file;
    file.open(backup_mmboxrc.c_str());
    ofstream file2;
    file2.open(rc_file.c_str());

    if (!file || !file2) {
      GLOBAL_ERROR_STREAM("error opening file " << backup_mmboxrc << " or " << rc_file << endl);
      return;
    }
    while(!file.eof()) {
      char c;
      while (file.peek() == '#') {
	do {
	  file.get(c);
	  file2.put(c);
	  if (file.eof()) {
	    break;
	  }
	} while ('\n' != c);
      }

      string var = "";
      string value = "";
      string new_value = "";
      bool found_equal=false;
      while (!file.eof()) {
	file.get(c);
	if ('\n' == c) {
	  file2.put(c);
	  break;
	} else if ('=' == c) {
	  found_equal = true;
	  getValue(var, new_value); 
	  file2.put(c);
	  continue;
	}
	if (!found_equal) {
	  var += c;
	  file2.put(c);
	} else {
	  if(new_value.size()) {
	    if((var == "configfile") && (new_value.find(".xml") == string::npos)) {
	      new_value += ".xml";
	    }
	    file2 << new_value;
	    while (!file.eof()) {
	      file.get(c);
	      if ('\n' == c) {
		file2.put(c);
		break;
	      }
	    }
	    break;
	  }
	  else {
	    value += c;
	    file2.put(c);
	  }
	}
      }

    }
    file2.close();
  }

  void NMMConfig::parseFile(const string filename) {
    // set filename for later storing new values to
    rc_file = filename;

    ifstream file(filename.c_str());
    if (!file) {
      return;
    }
    while(!file.eof()) {
      char c;
      while (file.peek() == '#') {
	do {
	  file.get(c);
	  if (file.eof()) {
	    break;
	  }
	} while ('\n' != c);
      }

      string var = "";
      string value = "";
      bool found_equal=false;
      while (!file.eof()) {
	file.get(c);
	if ('\n' == c) {
	  break;
	} else if (!found_equal && '=' == c) {
	  found_equal = true;
	  continue;
	}
	if (!found_equal) {
	  var += c;
	} else {
	  value += c;
	}
      }

      setValue(var, value); 
    }
  } // parseFile

  bool NMMConfig::getSetting(const string& id, const NMMConfig::Setting*& setting) const {
    map<string, Setting>::const_iterator found = settings.find(id);
    if (found == settings.end()) {
      return false;
    } else {
      setting = &found->second;
      return true;
    }
  }

  void NMMConfig::addSetting(const string& d /*= string("")*/, const string& s /*= string("")*/, const string& v /*= string("")*/) {
    Setting setting(d,v);
    settings[s] = setting;
  
    /* Add default value */
    if (config_table.find(s) == config_table.end()) {
      if (!setting.getValues().empty()) {
	config_table[s] = setting.getValues().front();
      }
    }
  }

}
