/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#include <iostream>
#include "NamedObject.hpp"
#include "nmm/nmm_strstream.hpp"
#include <unistd.h>

#include <time.h>
#include <sys/time.h>
#include <stdio.h>

namespace NMM {

NamedObject::NamedObject(const string & name)  {
  error_color     = new string[max_debug_level];
  debug_color     = new string[max_debug_level];
  warning_color   = new string[max_debug_level];
  message_color   = new string[max_debug_level];

  error_string    = new string[max_debug_level];
  debug_string    = new string[max_debug_level];
  warning_string  = new string[max_debug_level];
  message_string  = new string[max_debug_level];


  debug_final_string    = new string[max_debug_level];
  warning_final_string  = new string[max_debug_level];
  message_final_string  = new string[max_debug_level];
  error_final_string    = new string[max_debug_level];

  error_streams   = getGlobalInstance().getErrorArray();
  debug_streams   = getGlobalInstance().getDebugArray();
  warning_streams = getGlobalInstance().getWarningArray();
  message_streams = getGlobalInstance().getMessageArray();

  is_error_owner  = false;
  is_debug_owner  = false;
  is_message_owner= false;
  is_warning_owner= false;
  
  setName(name);
  setErrorColor(RED_COLOR, ALL_LEVELS);
  setDebugColor(DEFAULT_COLOR, ALL_LEVELS);
  setWarningColor(DEFAULT_COLOR, ALL_LEVELS);
  setMessageColor(DEFAULT_COLOR, ALL_LEVELS);
}

NamedObject::NamedObject(const char* name)  {
  error_color     = new string[max_debug_level];
  debug_color     = new string[max_debug_level];
  warning_color   = new string[max_debug_level];
  message_color   = new string[max_debug_level];

  error_string    = new string[max_debug_level];
  debug_string    = new string[max_debug_level];
  warning_string  = new string[max_debug_level];
  message_string  = new string[max_debug_level];


  debug_final_string    = new string[max_debug_level];
  warning_final_string  = new string[max_debug_level];
  message_final_string  = new string[max_debug_level];
  error_final_string    = new string[max_debug_level];

  error_streams   = getGlobalInstance().getErrorArray();
  debug_streams   = getGlobalInstance().getDebugArray();
  warning_streams = getGlobalInstance().getWarningArray();
  message_streams = getGlobalInstance().getMessageArray();

  is_error_owner  = false;
  is_debug_owner  = false;
  is_message_owner= false;
  is_warning_owner= false;
  
  setName(name);
  setErrorColor(RED_COLOR, ALL_LEVELS);
  setDebugColor(DEFAULT_COLOR, ALL_LEVELS);
  setWarningColor(DEFAULT_COLOR, ALL_LEVELS);
  setMessageColor(DEFAULT_COLOR, ALL_LEVELS);
}

NamedObject::NamedObject() {
  error_color     = new string[max_debug_level];
  debug_color     = new string[max_debug_level];
  warning_color   = new string[max_debug_level];
  message_color   = new string[max_debug_level];

  error_string    = new string[max_debug_level];
  debug_string    = new string[max_debug_level];
  warning_string  = new string[max_debug_level];
  message_string  = new string[max_debug_level];

  debug_final_string    = new string[max_debug_level];
  warning_final_string  = new string[max_debug_level];
  message_final_string  = new string[max_debug_level];
  error_final_string    = new string[max_debug_level];

  error_streams   = new ostream*[max_debug_level];
  debug_streams   = new ostream*[max_debug_level];
  message_streams = new ostream*[max_debug_level];
  warning_streams = new ostream*[max_debug_level];
  is_error_owner  = true;
  is_debug_owner  = true;
  is_message_owner= true;
  is_warning_owner= true;
  
  setName("");

  setErrorStream(&cout, ALL_LEVELS);
  setDebugStream(&cout, ALL_LEVELS);
  setWarningStream(&cout, ALL_LEVELS);
  setMessageStream(&cout, ALL_LEVELS);

  setErrorColor(RED_COLOR, ALL_LEVELS);
  setDebugColor(DEFAULT_COLOR, ALL_LEVELS);
  setWarningColor(DEFAULT_COLOR, ALL_LEVELS);
  setMessageColor(DEFAULT_COLOR, ALL_LEVELS);
}

NamedObject::~NamedObject() {
  delete [] debug_string;
  delete [] error_string;
  delete [] message_string;
  delete [] warning_string;

  delete [] debug_color;
  delete [] error_color;
  delete [] message_color;
  delete [] warning_color;

  delete [] debug_final_string;
  delete [] warning_final_string;
  delete [] message_final_string;
  delete [] error_final_string;

  if (is_error_owner) {
    delete [] error_streams;
  }
  if (is_debug_owner) {
    delete [] debug_streams;
  }
  if (is_warning_owner) {
    delete [] warning_streams;
  }
  if (is_message_owner) {
    delete [] message_streams;
  }
}

void NamedObject::setName(const string& name) {
  node_name = name;
  updateStrings();
}

void NamedObject::setErrorStream(ostream* stream, MessageLevel debug_level)  {
  if (!is_error_owner) {
    ostream** tmp = error_streams;
    error_streams = new ostream*[max_debug_level];
    for (unsigned int i = 0; i< max_debug_level; ++i)
      error_streams[i] = tmp[i];
    is_error_owner = true;
  }

  if (debug_level == ALL_LEVELS) {
    for (unsigned int i = 0; i< max_debug_level; ++i)
      error_streams[i] = stream;
  } else {
    error_streams[debug_level] = stream;
  }

  updateStrings();
}

void NamedObject::setDebugStream(ostream* stream, MessageLevel debug_level){
  if (!is_debug_owner) {
    ostream** tmp = debug_streams;
    debug_streams = new ostream*[max_debug_level];
    for (unsigned int i = 0; i< max_debug_level; ++i)
      debug_streams[i] = tmp[i];
    is_debug_owner = true;
  }

  if (debug_level == ALL_LEVELS) {
    for (unsigned int i = 0; i< max_debug_level; ++i)
      debug_streams[i] = stream;
  } else {
    debug_streams[debug_level] = stream;
  }

  updateStrings();
}

void NamedObject::setMessageStream(ostream* stream, MessageLevel debug_level){
  if (!is_message_owner) {
    ostream** tmp = message_streams;
    message_streams = new ostream*[max_debug_level];
    for (unsigned int i = 0; i< max_debug_level; ++i)
      message_streams[i] = tmp[i];
    is_message_owner = true;
  }

  if (debug_level == ALL_LEVELS) {
    for (unsigned int i = 0; i< max_debug_level; ++i)
      message_streams[i] = stream;
  } else {
    message_streams[debug_level] = stream;
  }

  updateStrings();
}

void NamedObject::setWarningStream(ostream* stream, MessageLevel debug_level){
  if (!is_warning_owner) {
    ostream** tmp = warning_streams;
    warning_streams = new ostream*[max_debug_level];
    for (unsigned int i = 0; i< max_debug_level; ++i)
      warning_streams[i] = tmp[i];
    is_warning_owner = true;
  }

  if (debug_level == ALL_LEVELS) {
    for (unsigned int i = 0; i< max_debug_level; ++i)
      warning_streams[i] = stream;
  } else {
    warning_streams[debug_level] = stream;
  }

  updateStrings();
}


void NamedObject::setErrorColor(Color c, MessageLevel debug_level) {
  if (debug_level == ALL_LEVELS) {
    setErrorColor(c, LEVEL0); setErrorColor(c, LEVEL1); setErrorColor(c, LEVEL2);
  } else {
    string color_string;
    if (isColoredStream(getErrorArray()[debug_level])) {
      color_string = color_array[c];
    }
    
    error_color[debug_level] = color_string;
  }
  
  updateStrings();
}

void NamedObject::setDebugColor(Color c, MessageLevel debug_level){
  if (debug_level == ALL_LEVELS) {
    setDebugColor(c, LEVEL0); setDebugColor(c, LEVEL1); setDebugColor(c, LEVEL2);
  } else {
    string color_string;
    if (isColoredStream(getDebugArray()[debug_level])) {
      color_string = color_array[c];
    }
    
    debug_color[debug_level] = color_string;    
  }
  
  updateStrings();
}

void NamedObject::setMessageColor(Color c, MessageLevel debug_level){
  if (debug_level == ALL_LEVELS) {
    setMessageColor(c, LEVEL0); setMessageColor(c, LEVEL1); setMessageColor(c, LEVEL2);
  } else {
    string color_string;
    if (isColoredStream(getMessageArray()[debug_level])) {
      color_string = color_array[c];
    }
    
    message_color[debug_level] = color_string;
  }
  
  updateStrings();
}

void NamedObject::setWarningColor(Color c, MessageLevel debug_level){
  if (debug_level == ALL_LEVELS) {
    setWarningColor(c, LEVEL0); setWarningColor(c, LEVEL1); setWarningColor(c, LEVEL2);
  } else {
    string color_string;
    if (isColoredStream(getWarningArray()[debug_level])) {
      color_string = color_array[c];
    }
    
    warning_color[debug_level] = color_string;
  }
  
  updateStrings();
}

string NamedObject::getTimeString() const {
  struct tm local_time;
  struct timeval tv;
  gettimeofday(&tv, NULL);
  localtime_r(&tv.tv_sec, &local_time);
  char buffer[128];
  char buffer1[128];
  /* prevent buggy gcc warning */
  char bogus[] = "%  +%%06dus> ";
  bogus[1] = 'c';
  strftime (buffer1, 128, bogus, &local_time);
  snprintf(buffer, 128, buffer1, static_cast<int>(tv.tv_usec));
  
  return string(buffer);
}

void NamedObject::updateStrings() {
  for (unsigned int i = 0; i < max_debug_level; ++i) {
    strstream s;
    string tmp;
    s << i ;
    s >> tmp;
    
    if (isColoredStream(getDebugArray()[i])) {
      debug_string[i] = debug_color[i] + " NMM DEBUG MESSAGE " + tmp + ":   " + node_name + "> ";
      debug_final_string[i] = color_array[DEFAULT_COLOR];
    } else {
      debug_string[i] = " NMM DEBUG MESSAGE " + tmp + ":   " + node_name + "> ";
      debug_final_string[i] = "";
    }
    
    if (isColoredStream(getErrorArray()[i])) {
      error_string[i] = error_color[i] + " NMM ERROR MESSAGE " + tmp + ":   " + node_name + "> ";
      error_final_string[i] = color_array[DEFAULT_COLOR];
    } else {
      error_string[i] = " NMM ERROR MESSAGE " + tmp + ":   " + node_name + "> ";
      error_final_string[i] = "";
    }
    
    if (isColoredStream(getMessageArray()[i])) {
      message_string[i] = message_color[i] + " NMM MESSAGE " + tmp + ":         " + node_name + "> ";
      message_final_string[i] = color_array[DEFAULT_COLOR];
    } else {
      message_string[i] = " NMM MESSAGE " + tmp + ":         " + node_name + "> ";
      message_final_string[i] = "";
    }
    
    if (isColoredStream(getWarningArray()[i])) {
      warning_string[i] = warning_color[i] + " NMM WARNING MESSAGE " + tmp + ": " + node_name + "> ";
      warning_final_string[i] = color_array[DEFAULT_COLOR];
    } else {
      warning_string[i] = " NMM WARNING MESSAGE " + tmp + ": " + node_name + "> ";
      warning_final_string[i] = "";
    }
  }
}

bool NamedObject::isColoredStream(ostream* stream) {
  return ((stream == &cout) &&  isatty(1)) || ((stream == &cerr) &&  isatty(2)) || ((stream == &clog) &&  isatty(2));
}

NamedObject& NamedObject::getGlobalInstance() {
  if (!global_object)
    global_object = new NamedObject();
  return *global_object;
}

NamedObject* NamedObject::global_object = 0;

const char* NamedObject::color_array[] = {"\033[0;0m", "\033[0;31m","\033[0;32m", "\033[0;33m", "\033[0;34m",
                                   "\033[0;35m","\033[0;36m", "\033[0;37m", ""};

}// end namespace NMM
