/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "SecurityCheck.hpp"
#include "utils.hpp"

#include <iostream>

namespace NMM {

  SecurityCheck::SecurityCheck()
    : NamedObject("SecurityCheck")
  {
    const char* home(getenv("HOME"));
    string nmmrcfile(string(home) + string("/.nmmrc"));
    nmmconfig.parseFile(nmmrcfile);
  }

  bool SecurityCheck::checkFilenameForReading(const string& dir_plus_filename) {
    if (dir_plus_filename.empty()) {
      return true;
    }
    
    string allowedpaths;
    nmmconfig.getValue("allowedreadpaths", allowedpaths);
    if(isFilenameWithinDirectories(dir_plus_filename, allowedpaths)) {
      return true;
    } else {
      ERROR_STREAM("Given filename " << dir_plus_filename << " is not in allowed paths for reading " << allowedpaths << endl);
      return false;
    } 
  }
  
  bool SecurityCheck::checkFilenameForWriting(const string& dir_plus_filename) {
    if (dir_plus_filename.empty()) {
      return true;
    }
    
    string allowedpaths;
    nmmconfig.getValue("allowedwritepaths", allowedpaths);
    if(isFilenameWithinDirectories(dir_plus_filename, allowedpaths)) {
      return true;
    } else {
      ERROR_STREAM("Given filename " << dir_plus_filename << " is not in allowed paths for writing " << allowedpaths << endl);
      return false;
    } 
  }

  const string SecurityCheck::getPasswd() {
    string passwd;
    nmmconfig.getValue("passwd", passwd);
    return passwd;
  }

  bool SecurityCheck::checkPasswd(const string& passwd) {
    string _passwd;
    nmmconfig.getValue("passwd", _passwd);
    if(passwd == _passwd) {
      return true;
    }
    else {
      ERROR_STREAM("Given password does not match" << endl);
      return false;
    }
  }

  void SecurityCheck::parseFile(const string& filename) {
    nmmconfig.parseFile(filename);
  }

}
