/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "WebServerResponse.hpp"
#include "HttpConstants.hpp"
#include "nmm/utils/utils.hpp"

#include <time.h>

namespace NMM{
  WebServerResponse::WebServerResponse(Socket *cSocket)
    :clientSocket(cSocket),
     dataWritten(false),
     canWrite(true),
     headerWritten(false),
     contentTypeSet(false),
     httpProtocol("HTTP/1.0")
  {}
  
  WebServerResponse::~WebServerResponse(){
  }
  
  void WebServerResponse::sendError(int pErrorCode){
    string errorText = "<html><head><title>Error</title></head><body>An arror has occured.</body></html>";
    
    if(canWrite && !dataWritten && !headerWritten){
      clientSocket->write(httpProtocol + " " + itoa(pErrorCode) + " Error Occured");
      clientSocket->write(HTTP_EOL);
      clientSocket->write("Server: " + getHostname());
      clientSocket->write(HTTP_EOL);
      
      //set current date
      time_t t = time(NULL);
      char date[20];
      strftime(date, sizeof(date), "%a, %d %b %Y %H:%M:%S %Z", localtime(&t));
      
      clientSocket->write("Date: " + string(date));
      clientSocket->write(HTTP_EOL);
      headerWritten = true;
      setContentType("text/html");
      setContentLength(errorText.length());
      clientSocket->write(errorText);
      clientSocket->write(HTTP_EOL);
    }
    canWrite = false;
  }
  
  void WebServerResponse::sendError(int pErrorCode, string pErrorMessage){
    string preText = "<html><head><title>Error</title></head><body>";
    string postText = "</body></html>";
    
    if(canWrite && !dataWritten && !headerWritten){
      clientSocket->write(httpProtocol + " " + itoa(pErrorCode) + " Error Occured");
      clientSocket->write(HTTP_EOL);
      clientSocket->write("Server: " + getHostname());
      clientSocket->write(HTTP_EOL);
      
      //set current date
      time_t t = time(NULL);
      char date[20];
      strftime(date, sizeof(date), "%a, %d %b %Y %H:%M:%S %Z", localtime(&t));
      clientSocket->write("Date: " + string(date));
      
      clientSocket->write(HTTP_EOL);
      headerWritten = true;
      setContentType("text/html");
      setContentLength(preText.length() + postText.length() + pErrorMessage.length());
      clientSocket->write(HTTP_EOL);
      clientSocket->write(preText);
      clientSocket->write(pErrorMessage);
      clientSocket->write(postText);
      clientSocket->write(HTTP_EOL);
    }
    canWrite = false;
  }
  void WebServerResponse::sendOK(){
    if(canWrite && !dataWritten && !headerWritten){
      clientSocket->write(httpProtocol + " " + HTTP_OK + " OK");
      clientSocket->write(HTTP_EOL);
      clientSocket->write("Server: " + getHostname());
      clientSocket->write(HTTP_EOL);
      
      //set current date
      time_t t = time(NULL);
      char date[20];
      strftime(date, sizeof(date), "%a, %d %b %Y %H:%M:%S %Z", localtime(&t));
      clientSocket->write("Date: " + string(date));
      
      clientSocket->write(HTTP_EOL);
      clientSocket->write("Last Modified: " + string(date));
      clientSocket->write(HTTP_EOL);
      clientSocket->write("Cache-control: no-cache");
      clientSocket->write(HTTP_EOL);
    }
    headerWritten=true;
  }
  
  void WebServerResponse::setContentType(string pContentType){
    if(canWrite && !dataWritten && headerWritten){
      clientSocket->write("Content-type: " + pContentType);
      clientSocket->write(HTTP_EOL);
      contentTypeSet=true;
    }
  }
  
  void WebServerResponse::setContentLength(int pContentLength){
    if(canWrite && !dataWritten && headerWritten){
      clientSocket->write("Content-length: " + itoa(pContentLength));
      clientSocket->write(HTTP_EOL);
    }
  }
  void WebServerResponse::write(string pData){
    if(canWrite && !dataWritten){
      if(!contentTypeSet){
        setContentType("text/html");
      }
      clientSocket->write(HTTP_EOL);
      dataWritten=true;
    }
    if(canWrite){
      clientSocket->write(pData);
    }
    clientSocket->write(HTTP_EOL);

  }
  
  void WebServerResponse::writeLine(string pData){
    this->write(pData);
    this->write(HTTP_EOL);
  }
  
  void WebServerResponse::close(){
    canWrite = false;
    dataWritten = true;
    //flush
  }
}
