
#include "IPerf.hpp"

namespace NMM {


IPerf::IPerf() {
  sport = 22802;
  async_client_running = false;
  server_running = false;
}

float IPerf::testConnection(string host, uint port, uint time, uint parallel) {
  if (server_running) {
    return 0.0F;
  }
  float speed;
  i_perf.startClient(speed, host, port, time, parallel);
  return speed;
}

Result IPerf::startAsyncConnection(const string host, uint port, uint time, uint parallel) {
  if ((server_running) || (async_client_running)) {
    return FAILURE;
  }
  i_perf.startAsyncClient(host, port, time, parallel);
  async_client_running = true;

  return SUCCESS;
}

void IPerf::getClientSpeed(float& c_speed) {
  if (async_client_running) {
    i_perf.getClientSpeed(c_speed);
    async_client_running = false;
  } else {
    c_speed = 0.0;
  }
}

Result IPerf::startIPERFServer(uint port) {
  if (async_client_running) {
    return FAILURE;
  }

  server_running = true;

  sport = port;
  activateThread();

  return SUCCESS;
}

void IPerf::stopIPERFServer() {
  i_perf.deinit();

  deactivateThread();

  server_running = false;
}

void IPerf::run() {
  i_perf.startServer(sport);
}

}








