#ifndef NMM_IPERF
#define NMM_IPERF NMM_IPERF

#include "main.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"

namespace NMM {

  /**
   * iperf is a tool to measure internet performance
   * a client can measure the performance of the connection to a server
   */
  class IPerf : public ThreadedObject {

      IPerf17 i_perf; // the iperf main object

      uint sport; //> the port the server is listening on

      bool async_client_running;
      bool server_running;


    public:

      IPerf();

      /**
       * start an iperf-server
       * \param port the port on wich the server is listening
       */
      Result startIPERFServer(uint port = 22802 );

      /**
       * stop the iperf-server
       */
      void stopIPERFServer();

      /**
       * test the speed of the connection to the server on \param host on port \param port
       * \param time test the connection for that many seconds
       * \param parallel start that many parallel connections
       */
      float testConnection(string host, uint port = 22802 , uint time = 10, uint parallel = 1);

      Result startAsyncConnection(const string host, uint port = 22802 , uint time = 10, uint parallel = 1);

      void getClientSpeed(float& speed);

      /**
       * the threads run-method
       */
      void run();
  };

}

#endif
