/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "WorkerThread.hpp"
#include "ThreadPool.hpp"

namespace NMM {
  WorkerThread::WorkerThread() :__current_task(0),  __thread_condition(__thread_mutex), __iteration_count(0), __is_running(true), __release_flag(false)
  {
    activateThread();
  }

  WorkerThread::~WorkerThread()
  {
    /* Begin> set the __is_running flag to false */
    __thread_mutex.lock();
    __is_running = false;
    __thread_condition.notify();
    __thread_mutex.unlock();
    /* End< */

    /* deactivate the thread */
    deactivateThread();
  }

  void WorkerThread::run() {
    while ( __is_running ) {
      __thread_mutex.lock();
      if ( __iteration_count > 0 ) {
        --__iteration_count;
        __thread_mutex.unlock();
        /* call the task */
        __current_task->run();
      } else {
        if ( __release_flag ) {
          SThreadPool::getInstance().releaseThread(this);
        }

        /* Wait for work */
        __thread_condition.wait();
        __thread_mutex.unlock();
      }
    }
  }

  void WorkerThread::iterateTask(int iteration_count) {
    __thread_mutex.lock();
    __iteration_count += iteration_count;
    __thread_condition.notify();
    __thread_mutex.unlock();
  }
}

